@@ -1,4 +1,4 @@
-/* $Id: actions.cpp,v 1.136 2004/06/15 14:32:53 Sirp Exp $ */
+/* $Id: actions.cpp,v 1.157 2004/09/27 00:24:41 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -36,6 +36,9 @@
 #include <string>
 #include <sstream>
 
+#define LOG_NG lg::info(lg::engine)
+#define ERR_NW lg::err(lg::network)
+
 struct castle_cost_calculator
 {
 	castle_cost_calculator(const gamemap& map) : map_(map)
@@ -77,7 +80,7 @@ std::string recruit_unit(const gamemap& map, int side,
 {
 	const command_disabler disable_commands(disp);
 
-	std::cerr << "recruiting unit for side " << side << "\n";
+	LOG_NG << "recruiting unit for side " << side << "\n";
 	typedef std::map<gamemap::location,unit> units_map;
 
 	//find the unit that can recruit
@@ -90,14 +93,14 @@ std::string recruit_unit(const gamemap& map, int side,
 	}
 
 	if(u == units.end())
-		return string_table["no_leader_to_recruit"];
+		return _("You don't have a leader to recruit with.");
 
 	if(map.is_keep(u->first) == false) {
-		std::cerr << "Leader not on start: leader is on " << (u->first.x+1) << "," << (u->first.y+1) << "\n";
-		return string_table["leader_not_on_start"];
+		LOG_NG << "Leader not on start: leader is on " << (u->first.x+1) << "," << (u->first.y+1) << "\n";
+		return _("You must have your leader on a keep to recruit or recall units.");
 	}
 
-	if(map.on_board(recruit_location)) {
+	if(need_castle && map.on_board(recruit_location)) {
 		const paths::route& rt = a_star_search(u->first,recruit_location,
 		                                   100.0,castle_cost_calculator(map));
 		if(rt.steps.empty() || units.find(recruit_location) != units.end() ||
@@ -111,7 +114,7 @@ std::string recruit_unit(const gamemap& map, int side,
 	}
 
 	if(!map.on_board(recruit_location)) {
-		return string_table["no_recruit_location"];
+		return _("There are no vacant castle tiles in which to recruit a unit.");
 	}
 
 	if(full_movement) {
@@ -130,6 +133,9 @@ std::string recruit_unit(const gamemap& map, int side,
 	units.insert(std::pair<gamemap::location,unit>(
 							recruit_location,new_unit));
 
+	LOG_NG << "firing recruit event\n";
+	game_events::fire("recruit",recruit_location);
+
 	if(show) {
 
 		for(double alpha = 0.0; alpha <= 1.0; alpha += 0.1) {
@@ -196,7 +202,7 @@ battle_stats evaluate_battle_stats(
 	res.attack_with = attack_with;
 
 	if(include_strings)
-		res.defend_name = string_table["weapon_none"];
+		res.defend_name = _("none");
 
 	const std::map<gamemap::location,unit>::iterator a = units.find(attacker);
 	const std::map<gamemap::location,unit>::iterator d = units.find(defender);
@@ -219,6 +225,7 @@ battle_stats evaluate_battle_stats(
 
 	static const std::string charge_string("charge");
 	const bool charge = (attack.special() == charge_string);
+	const bool steadfast = d->second.type().steadfast();
 
 	bool backstab = false;
 
@@ -252,6 +259,9 @@ battle_stats evaluate_battle_stats(
 	res.attack_name    = attack.name();
 	res.attack_type    = attack.type();
 
+	static const std::string slow_string("slow");
+	res.attacker_slows = attack.special() == slow_string;
+
 	if(include_strings) {
 		res.attack_special = attack.special();
 		res.attack_icon = attack.icon();
@@ -308,7 +318,7 @@ battle_stats evaluate_battle_stats(
 
 		if(include_strings) {
 			std::stringstream str_base;
-			str_base << string_table["base_damage"] << "," << base_damage << ",";
+			str_base << _("base damage") << "," << base_damage << ",";
 			res.defend_calculations.push_back(str_base.str());
 		}
 
@@ -318,7 +328,7 @@ battle_stats evaluate_battle_stats(
 		if(include_strings && resist != 0) {
 			std::stringstream str_resist;
 
-			str_resist << string_table[resist < 0 ? "attacker_resistance" : "attacker_vulnerability"] << " " << translate_string(defender_attacks[defend].type())
+			str_resist << gettext(resist < 0 ? N_("attacker resistance vs") : N_("attacker vulnerability vs")) << " " << string_table[defender_attacks[defend].type()]
 				<< ", ,^" << (resist > 0 ? "+" : "") << resist << "%";
 			res.defend_calculations.push_back(str_resist.str());
 		}
@@ -329,28 +339,28 @@ battle_stats evaluate_battle_stats(
 		if(include_strings && tod_modifier != 0) {
 			std::stringstream str_mod;
 			const time_of_day& tod = timeofday_at(state,units,d->first);
-			str_mod << translate_string_default(tod.id,tod.name) << ", ,^"
+			str_mod << tod.name << ", ,^"
 			        << (tod_modifier > 0 ? "+" : "") << tod_modifier << "%";
 			res.defend_calculations.push_back(str_mod.str());
 		}
 
-		if(charge) {
-			percent += 100;
-
-			if(include_strings) {
-				std::stringstream str;
-				str << translate_string("charge") << ", ,^+100%";
-				res.defend_calculations.push_back(str.str());
-			}
-		}
-
 		int leader_bonus = 0;
 		if(under_leadership(units,defender,&leader_bonus).valid()) {
 			percent += leader_bonus;
 
 			if(include_strings) {
 				std::stringstream str;
-				str << translate_string("leadership") << ", ,^+" + str_cast(leader_bonus) + "%";
+				str << _("leadership") << ", ,^+" + str_cast(leader_bonus) + "%";
+				res.defend_calculations.push_back(str.str());
+			}
+		}
+
+		if(charge) {
+			percent = (100+percent)*2 - 100;
+
+			if(include_strings) {
+				std::stringstream str;
+				str << _("charge") << ", ," << _("Doubled");
 				res.defend_calculations.push_back(str.str());
 			}
 		}
@@ -378,7 +388,7 @@ battle_stats evaluate_battle_stats(
 			percent *= is_negative;
 
 			std::stringstream str;
-			str << translate_string("total_damage") << "," << res.damage_attacker_takes
+			str << _("total damage") << "," << res.damage_attacker_takes
 				<< ",^" << (percent >= 0 ? "+" : "") << percent << "% (" << (difference >= 0 ? "+" : "") << difference << ")";
 			res.defend_calculations.push_back(str.str());
 		}
@@ -401,7 +411,8 @@ battle_stats evaluate_battle_stats(
 			res.amount_defender_drains = 0;
 		}
 
-		res.defender_plague = (defender_attacks[defend].special() == plague_string);
+		res.defender_plague = !a->second.type().not_living() && (defender_attacks[defend].special() == plague_string);
+		res.defender_slows = (defender_attacks[defend].special() == slow_string);
 
 		static const std::string first_strike = "firststrike";
 		res.defender_strikes_first = defender_attacks[defend].special() == first_strike && attack.special() != first_strike;
@@ -424,7 +435,7 @@ battle_stats evaluate_battle_stats(
 	if(include_strings) {
 		std::stringstream str_base;
 
-		str_base << string_table["base_damage"] << "," << base_damage << ",";
+		str_base << _("base damage") << "," << base_damage << ",";
 		res.attack_calculations.push_back(str_base.str());
 	}
 
@@ -434,7 +445,7 @@ battle_stats evaluate_battle_stats(
 	if(include_strings && resist != 0) {
 		std::stringstream str_resist;
 
-		str_resist << string_table[resist < 0 ? "defender_resistance" : "defender_vulnerability"] << " " << translate_string(attack.type())
+		str_resist << gettext(resist < 0 ? N_("defender resistance vs") : N_("defender vulnerability vs")) << " " << string_table[attack.type()]
 			       << ", ,^" << (resist > 0 ? "+" : "") << resist << "%";
 		res.attack_calculations.push_back(str_resist.str());
 	}
@@ -446,37 +457,46 @@ battle_stats evaluate_battle_stats(
 	if(include_strings && tod_modifier != 0) {
 		std::stringstream str_mod;
 		const time_of_day& tod = timeofday_at(state,units,a->first);
-		str_mod << translate_string_default(tod.id,tod.name) << ", ,^"
+		str_mod << tod.name << ", ,^"
 		        << (tod_modifier > 0 ? "+" : "") << tod_modifier << "%";
 		res.attack_calculations.push_back(str_mod.str());
 	}
 
-	if(charge) {
-		percent += 100;
-
-		if(include_strings) {
-			std::stringstream str;
-			str << translate_string("charge") << ", ,^+100%";
-			res.attack_calculations.push_back(str.str());
-		}
-	}
-
-	if(backstab) {
-		percent += 100;
-		if(include_strings) {
-			std::stringstream str;
-			str << translate_string("backstab") << ", ,^+100%";
-			res.attack_calculations.push_back(str.str());
-		}
-	}
-
 	int leader_bonus = 0;
 	if(under_leadership(units,attacker,&leader_bonus).valid()) {
 		percent += leader_bonus;
 		
 		if(include_strings) {
 			std::stringstream str;
-			str << translate_string("leadership") << ", ,^+" + str_cast(leader_bonus) + "%";
+			str << _("leadership") << ", ,^+" + str_cast(leader_bonus) + "%";
+			res.attack_calculations.push_back(str.str());
+		}
+	}
+
+	if(charge) {
+		percent = (100+percent)*2 - 100;
+
+		if(include_strings) {
+			std::stringstream str;
+			str << _("charge") << ", ," << _("Doubled");
+			res.attack_calculations.push_back(str.str());
+		}
+	}
+
+	if(backstab) {
+		percent = (100+percent)*2 - 100;
+		if(include_strings) {
+			std::stringstream str;
+			str << _("backstab") << ", ," << _("Doubled");
+			res.attack_calculations.push_back(str.str());
+		}
+	}
+
+	if(steadfast) {
+		percent = (100+percent)/2 - 100;
+		if(include_strings) {
+			std::stringstream str;
+			str << _("steadfast") << ", ," << _("Halved");
 			res.attack_calculations.push_back(str.str());
 		}
 	}
@@ -503,7 +523,7 @@ battle_stats evaluate_battle_stats(
 		percent *= is_negative;
 
 		std::stringstream str;
-		str << translate_string("total_damage") << "," << res.damage_defender_takes
+		str << _("total damage") << "," << res.damage_defender_takes
 			<< ",^" << (percent >= 0 ? "+" : "") << percent << "% (" << (difference >= 0 ? "+" : "") << difference << ")";
 		res.attack_calculations.push_back(str.str());
 	}
@@ -568,7 +588,7 @@ void attack(display& gui, const gamemap& map,
 	static const std::string poison_string("poison");
 
 	while(stats.nattacks > 0 || stats.ndefends > 0) {
-		std::cerr << "start of attack loop...\n";
+		LOG_NG << "start of attack loop...\n";
 
 		if(stats.nattacks > 0 && stats.defender_strikes_first == false) {
 			const int ran_num = get_random();
@@ -583,24 +603,29 @@ void attack(display& gui, const gamemap& map,
 				const int results_damage = atoi((*ran_results)["damage"].c_str());
 
 				if(results_chance != stats.chance_to_hit_defender) {
-					std::cerr << "SYNC ERROR: In attack " << a->second.type().name() << " vs "
-					          << d->second.type().name() << ": chance to hit defender is inconsistent. Data source: "
-							  << results_chance << "; Calculation: " << stats.chance_to_hit_defender
-							  << " (over-riding game calculations with data source results)\n";
+					ERR_NW << "SYNC: In attack " << a->second.type().name() << " vs "
+						<< d->second.type().name()
+						<< ": chance to hit defender is inconsistent. Data source: "
+						<< results_chance << "; Calculation: " << stats.chance_to_hit_defender
+						<< " (over-riding game calculations with data source results)\n";
 					hits = results_hits;
 				} else if(hits != results_hits) {
-					std::cerr << "SYNC ERROR: In attack " << a->second.type().name() << " vs "
-					          << d->second.type().name() << ": the data source says the hit was "
-							  << (results_hits ? "successful" : "unsuccessful") << ", while in-game calculations say the hit was "
-							  << (hits ? "successful" : "unsuccessful")
-							  << " random number: " << ran_num << " = " << (ran_num%100) << "/" << results_chance
-							  << " (over-riding game calculations with data source results)\n";
+					ERR_NW << "SYNC: In attack " << a->second.type().name() << " vs "
+						<< d->second.type().name() << ": the data source says the hit was "
+						<< (results_hits ? "successful" : "unsuccessful")
+						<< ", while in-game calculations say the hit was "
+						<< (hits ? "successful" : "unsuccessful")
+						<< " random number: " << ran_num << " = "
+						<< (ran_num%100) << "/" << results_chance
+						<< " (over-riding game calculations with data source results)\n";
 					hits = results_hits;
 				} else if(results_damage != stats.damage_defender_takes) {
-					std::cerr << "SYNC ERROR: In attack " << a->second.type().name() << " vs "
-					          << d->second.type().name() << ": the data source says the hit did "
-							  << results_damage << " damage, while in-game calculations show the hit doing "
-							  << stats.damage_defender_takes << " damage (over-riding game calculations with data source results)\n";
+					ERR_NW << "SYNC: In attack " << a->second.type().name() << " vs "
+						<< d->second.type().name() << ": the data source says the hit did "
+						<< results_damage
+						<< " damage, while in-game calculations show the hit doing "
+						<< stats.damage_defender_takes
+						<< " damage (over-riding game calculations with data source results)\n";
 					stats.damage_defender_takes = results_damage;
 				}
 			}
@@ -609,13 +634,13 @@ void attack(display& gui, const gamemap& map,
 				            hits ? stats.damage_defender_takes : 0,
 							a->second.attacks()[attack_with]);
 
-			std::cerr << "done attacking\n";
+			LOG_NG << "done attacking\n";
 
 			attack_stats.attack_result(hits ? (dies ? statistics::attack_context::KILLS : statistics::attack_context::HITS)
 			                           : statistics::attack_context::MISSES);
 
 			if(ran_results == NULL) {
-				log_scope("setting random results");
+				log_scope2(engine, "setting random results");
 				config cfg;
 				cfg["hits"] = (hits ? "yes" : "no");
 				cfg["dies"] = (dies ? "yes" : "no");
@@ -627,14 +652,25 @@ void attack(display& gui, const gamemap& map,
 			} else {
 				const bool results_dies = (*ran_results)["dies"] == "yes";
 				if(results_dies != dies) {
-					std::cerr << "SYNC ERROR: In attack" << a->second.type().name() << " vs "
-					          << d->second.type().name() << ": the data source the unit "
-							  << (results_dies ? "perished" : "survived") << " while in-game calculations show the unit "
-							  << (dies ? "perished" : "survived") << " (over-riding game calculations with data source results)\n";
+					ERR_NW << "SYNC: In attack" << a->second.type().name() << " vs "
+						<< d->second.type().name() << ": the data source the unit "
+						<< (results_dies ? "perished" : "survived")
+						<< " while in-game calculations show the unit "
+						<< (dies ? "perished" : "survived")
+						<< " (over-riding game calculations with data source results)\n";
 					dies = results_dies;
 				}
 			}
 
+			if(dies || hits) {
+				if(stats.amount_attacker_drains > 0) {
+					char buf[50];
+					sprintf(buf,"%d",stats.amount_attacker_drains);
+					gui.float_label(a->first,buf,0,255,0);
+					a->second.gets_hit(-stats.amount_attacker_drains);
+				}
+			}
+
 			if(dies) {
 				attackerxp = game_config::kill_experience*d->second.type().level();
 				if(d->second.type().level() == 0)
@@ -648,14 +684,15 @@ void attack(display& gui, const gamemap& map,
 				gamemap::location loc = d->first;
 				gamemap::location attacker_loc = a->first;
 				const int defender_side = d->second.side();
-				std::cerr << "firing die event\n";
+				LOG_NG << "firing die event\n";
 				game_events::fire("die",loc,a->first);
-				d = units.end();
+				d = units.find(loc);
 				a = units.end();
 
-				//the handling of the event may have removed the object
-				//so we have to find it again
-				units.erase(loc);
+				if(d != units.end() && d->second.hitpoints() <= 0) {
+					units.erase(d);
+					d = units.end();
+				}
 
 				//plague units make clones of themselves on the target hex
 				//units on villages that die cannot be plagued
@@ -674,30 +711,21 @@ void attack(display& gui, const gamemap& map,
 				if(stats.attack_special == poison_string &&
 				   d->second.has_flag("poisoned") == false &&
 				   !d->second.type().not_living()) {
-					gui.float_label(d->first,translate_string("poisoned"),255,0,0);
+					gui.float_label(d->first,_("poisoned"),255,0,0);
 					d->second.set_flag("poisoned");
 				}
 
-				static const std::string slow_string("slow");
-				if(stats.attack_special == slow_string &&
-				   d->second.has_flag("slowed") == false) {
-					gui.float_label(d->first,translate_string("slowed"),255,0,0);
+				if(stats.attacker_slows && d->second.has_flag("slowed") == false) {
+					gui.float_label(d->first,_("slowed"),255,0,0);
 					d->second.set_flag("slowed");
 					if(stats.ndefends > 1)
 						--stats.ndefends;
 				}
 
-				if(stats.amount_attacker_drains > 0) {
-					char buf[50];
-					sprintf(buf,"%d",stats.amount_attacker_drains);
-					gui.float_label(a->first,buf,0,255,0);
-					a->second.gets_hit(-stats.amount_attacker_drains);
-				}
-
 				//if the defender is turned to stone, the fight stops immediately
 				static const std::string stone_string("stone");
 				if(stats.attack_special == stone_string) {
-					gui.float_label(d->first,translate_string("stone"),255,0,0);
+					gui.float_label(d->first,_("stone"),255,0,0);
 					d->second.set_flag(stone_string);
 					stats.ndefends = 0;
 					stats.nattacks = 0;
@@ -712,7 +740,7 @@ void attack(display& gui, const gamemap& map,
 		stats.defender_strikes_first = false;
 
 		if(stats.ndefends > 0) {
-			std::cerr << "doing defender attack...\n";
+			LOG_NG << "doing defender attack...\n";
 
 			const int ran_num = get_random();
 			bool hits = (ran_num%100) < stats.chance_to_hit_attacker;
@@ -726,24 +754,29 @@ void attack(display& gui, const gamemap& map,
 				const int results_damage = atoi((*ran_results)["damage"].c_str());
 
 				if(results_chance != stats.chance_to_hit_attacker) {
-					std::cerr << "SYNC ERROR: In defend " << a->second.type().name() << " vs "
-					          << d->second.type().name() << ": chance to hit attacker is inconsistent. Data source: "
-							  << results_chance << "; Calculation: " << stats.chance_to_hit_attacker
-							  << " (over-riding game calculations with data source results)\n";
+					ERR_NW << "SYNC: In defend " << a->second.type().name() << " vs "
+						<< d->second.type().name()
+						<< ": chance to hit attacker is inconsistent. Data source: "
+						<< results_chance << "; Calculation: " << stats.chance_to_hit_attacker
+						<< " (over-riding game calculations with data source results)\n";
 					hits = results_hits;
 				} else if(hits != results_hits) {
-					std::cerr << "SYNC ERROR: In defend " << a->second.type().name() << " vs "
-					          << d->second.type().name() << ": the data source says the hit was "
-							  << (results_hits ? "successful" : "unsuccessful") << ", while in-game calculations say the hit was "
-							  << (hits ? "successful" : "unsuccessful")
-							  << " random number: " << ran_num << " = " << (ran_num%100) << "/" << results_chance
-							  << " (over-riding game calculations with data source results)\n";
+					ERR_NW << "SYNC: In defend " << a->second.type().name() << " vs "
+						<< d->second.type().name() << ": the data source says the hit was "
+						<< (results_hits ? "successful" : "unsuccessful")
+						<< ", while in-game calculations say the hit was "
+						<< (hits ? "successful" : "unsuccessful")
+						<< " random number: " << ran_num << " = " << (ran_num%100) << "/"
+						<< results_chance
+						<< " (over-riding game calculations with data source results)\n";
 					hits = results_hits;
 				} else if(results_damage != stats.damage_attacker_takes) {
-					std::cerr << "SYNC ERROR: In defend " << a->second.type().name() << " vs "
-					          << d->second.type().name() << ": the data source says the hit did "
-							  << results_damage << " damage, while in-game calculations show the hit doing "
-							  << stats.damage_attacker_takes << " damage (over-riding game calculations with data source results)\n";
+					ERR_NW << "SYNC: In defend " << a->second.type().name() << " vs "
+						<< d->second.type().name() << ": the data source says the hit did "
+						<< results_damage
+						<< " damage, while in-game calculations show the hit doing "
+						<< stats.damage_attacker_takes
+						<< " damage (over-riding game calculations with data source results)\n";
 					stats.damage_attacker_takes = results_damage;
 				}
 			}
@@ -766,14 +799,26 @@ void attack(display& gui, const gamemap& map,
 			} else {
 				const bool results_dies = (*ran_results)["dies"] == "yes";
 				if(results_dies != dies) {
-					std::cerr << "SYNC ERROR: In defend" << a->second.type().name() << " vs "
-					          << d->second.type().name() << ": the data source the unit "
-							  << (results_dies ? "perished" : "survived") << " while in-game calculations show the unit "
-							  << (dies ? "perished" : "survived") << " (over-riding game calculations with data source results)\n";
+					ERR_NW << "SYNC: In defend" << a->second.type().name() << " vs "
+						<< d->second.type().name() << ": the data source the unit "
+						<< (results_dies ? "perished" : "survived")
+						<< " while in-game calculations show the unit "
+						<< (dies ? "perished" : "survived")
+						<< " (over-riding game calculations with data source results)\n";
 					dies = results_dies;
 				}
 			}
 
+			if(hits || dies){
+				if(stats.amount_defender_drains > 0) {
+					char buf[50];
+					sprintf(buf,"%d",stats.amount_defender_drains);
+					gui.float_label(d->first,buf,0,255,0);
+
+					d->second.gets_hit(-stats.amount_defender_drains);
+				}
+			}
+				
 			if(dies) {
 				defenderxp = game_config::kill_experience*a->second.type().level();
 				if(a->second.type().level() == 0)
@@ -788,12 +833,13 @@ void attack(display& gui, const gamemap& map,
 				gamemap::location defender_loc = d->first;
 				const int attacker_side = a->second.side();
 				game_events::fire("die",loc,d->first);
-				a = units.end();
+				a = units.find(loc);
 				d = units.end();
 
-				//the handling of the event may have removed the object
-				//so we have to find it again
-				units.erase(loc);
+				if(a != units.end() && a->second.hitpoints() <= 0) {
+					units.erase(a);
+					a = units.end();
+				}
 
 				//plague units make clones of themselves on the target hex.
 				//units on villages that die cannot be plagued
@@ -812,31 +858,22 @@ void attack(display& gui, const gamemap& map,
 				if(stats.defend_special == poison_string &&
 				   a->second.has_flag("poisoned") == false &&
 				   !a->second.type().not_living()) {
-					gui.float_label(a->first,translate_string("poisoned"),255,0,0);
+					gui.float_label(a->first,_("poisoned"),255,0,0);
 					a->second.set_flag("poisoned");
 				}
 
-				static const std::string slow_string("slow");
-				if(stats.defend_special == slow_string &&
-				   a->second.has_flag("slowed") == false) {
-					gui.float_label(a->first,translate_string("slowed"),255,0,0);
+				if(stats.defender_slows && a->second.has_flag("slowed") == false) {
+					gui.float_label(a->first,_("slowed"),255,0,0);
 					a->second.set_flag("slowed");
 					if(stats.nattacks > 1)
 						--stats.nattacks;
 				}
 
-				if(stats.amount_defender_drains > 0) {
-					char buf[50];
-					sprintf(buf,"%d",stats.amount_defender_drains);
-					gui.float_label(d->first,buf,0,255,0);
-
-					d->second.gets_hit(-stats.amount_defender_drains);
-				}
 
 				//if the attacker is turned to stone, the fight stops immediately
 				static const std::string stone_string("stone");
 				if(stats.defend_special == stone_string) {
-					gui.float_label(a->first,translate_string("stone"),255,0,0);
+					gui.float_label(a->first,_("stone"),255,0,0);
 					a->second.set_flag(stone_string);
 					stats.ndefends = 0;
 					stats.nattacks = 0;
@@ -1156,7 +1193,8 @@ void calculate_healing(display& disp, const gamestatus& status, const gamemap& m
 
 		const int DelayAmount = 50;
 
-		std::cerr << "unit is poisoned? " << (u.has_flag("poisoned") ? "yes" : "no") << "," << h->second << "," << max_healing[h->first] << "\n";
+		LOG_NG << "unit is poisoned? " << (u.has_flag("poisoned") ? "yes" : "no") << ","
+			<< h->second << "," << max_healing[h->first] << "\n";
 
 		if(u.has_flag("poisoned") && h->second > 0) {
 
@@ -1275,7 +1313,9 @@ void advance_unit(const game_data& info,
 	const unit& new_unit = get_advanced_unit(info,units,loc,advance_to);
 
 	statistics::advance_unit(new_unit);
-	
+	preferences::encountered_units().insert(new_unit.type().name());
+	std::cout << "Added '" << new_unit.type().name() << "' to encountered units" << std::endl;
+
 	units.erase(loc);
 	units.insert(std::pair<gamemap::location,unit>(loc,new_unit));
 }
@@ -1287,7 +1327,7 @@ void check_victory(std::map<gamemap::location,unit>& units,
 	for(std::map<gamemap::location,unit>::const_iterator i = units.begin();
 	    i != units.end(); ++i) {
 		if(i->second.can_recruit()) {
-			std::cerr << "seen leader for side " << i->second.side() << "\n";
+			LOG_NG << "seen leader for side " << i->second.side() << "\n";
 			seen_leaders.push_back(i->second.side());
 		}
 	}
@@ -1337,7 +1377,7 @@ void check_victory(std::map<gamemap::location,unit>& units,
 			std::cout << "\n";
 		}
 
-		std::cerr << "throwing end level exception...\n";
+		LOG_NG << "throwing end level exception...\n";
 		throw end_level_exception(found_human ? VICTORY : DEFEAT);
 	}
 
@@ -1404,7 +1444,10 @@ bool clear_shroud_loc(const gamemap& map, team& tm,
 	get_adjacent_tiles(loc,adj);
 	adj[6] = loc;
 	for(int i = 0; i != 7; ++i) {
-		if(map.on_board(adj[i])) {
+
+		//we clear one past the edge of the board, so that the half-hexes
+		//at the edge can also be cleared of fog/shroud
+		if(map.on_board(adj[i]) || map.on_board(loc)) {
 			const bool res = tm.clear_shroud(adj[i].x,adj[i].y) ||
 								tm.clear_fog(adj[i].x,adj[i].y);
 
@@ -1432,7 +1475,15 @@ bool clear_shroud_unit(const gamemap& map,
 {
 	std::vector<gamemap::location> cleared_locations;
 
-	paths p(map,status,gamedata,units,loc,teams,true,false);
+	const unit_map::const_iterator u = units.find(loc);
+	if(u == units.end()) {
+		return false;
+	}
+
+	unit_map temp_units;
+	temp_units.insert(*u);
+
+	paths p(map,status,gamedata,temp_units,loc,teams,true,false);
 	for(paths::routes_map::const_iterator i = p.routes.begin();
 	    i != p.routes.end(); ++i) {
 		clear_shroud_loc(map,teams[team],i->first,&cleared_locations);
@@ -1443,6 +1494,7 @@ bool clear_shroud_unit(const gamemap& map,
 
 	for(std::vector<gamemap::location>::const_iterator it =
 	    cleared_locations.begin(); it != cleared_locations.end(); ++it) {
+
 		if(units.count(*it)) {
 			if(seen_units == NULL || known_units == NULL) {
 				static const std::string sighted("sighted");
@@ -1572,7 +1624,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 		//which case we should stop immediately.
 		if(check_shroud) {
 			if(units.count(*step) == 0 && !map.is_village(*step)) {
-				std::cerr << "checking for units from " << (step->x+1) << "," << (step->y+1) << "\n";
+				LOG_NG << "checking for units from " << (step->x+1) << "," << (step->y+1) << "\n";
 
 				//temporarily reset the unit's moves to full
 				const unit_movement_resetter move_resetter(ui->second);
@@ -1583,6 +1635,10 @@ size_t move_unit(display* disp, const game_data& gamedata,
 
 				should_clear_stack |= clear_shroud_unit(map,status,gamedata,units,*step,teams,
 				                                        ui->second.side()-1,&known_units,&seen_units);
+
+				if(should_clear_stack) {
+					disp->invalidate_all();
+				}
 			}
 		}
 
@@ -1637,7 +1693,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 
 	units.erase(ui);
 	if(disp != NULL) {
-		unit_display::move_unit(*disp,map,steps,u);
+		unit_display::move_unit(*disp,map,steps,u,status.get_time_of_day(),units,teams);
 	}
 
 	u.set_movement(moves_left);
@@ -1679,7 +1735,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 		//show messages on the screen here
 		if(discovered_unit) {
 			//we've been ambushed, so display an appropriate message
-			font::add_floating_label(string_table["ambushed"],24,font::BAD_COLOUR,
+			font::add_floating_label(_("Ambushed!"),font::SIZE_XLARGE,font::BAD_COLOUR,
 			                         disp->map_area().w/2,disp->map_area().h/3,
 									 0.0,0.0,100,disp->map_area(),font::CENTER_ALIGN);
 		}
@@ -1689,7 +1745,7 @@ size_t move_unit(display* disp, const game_data& gamedata,
 			//they are allies or enemies, so calculate that out here
 			int nfriends = 0, nenemies = 0;
 			for(std::set<gamemap::location>::const_iterator i = seen_units.begin(); i != seen_units.end(); ++i) {
-				std::cerr << "processing unit at " << (i->x+1) << "," << (i->y+1) << "\n";
+				LOG_NG << "processing unit at " << (i->x+1) << "," << (i->y+1) << "\n";
 				const unit_map::const_iterator u = units.find(*i);
 				assert(u != units.end());
 				if(team.is_enemy(u->second.side())) {
@@ -1698,29 +1754,29 @@ size_t move_unit(display* disp, const game_data& gamedata,
 					++nfriends;
 				}
 
-				std::cerr << "processed...\n";
+				LOG_NG << "processed...\n";
 			}
 
-			std::string msg_id;
+			char* msg_id;
 
 			//the message we display is different depending on whether units sighted
 			//were enemies or friends, and whether there is one or more
 			if(seen_units.size() == 1) {
 				if(nfriends == 1) {
-					msg_id = "friendly_unit_sighted";
+					msg_id = N_("Friendly unit sighted");
 				} else {
-					msg_id = "enemy_unit_sighted";
+					msg_id = N_("Enemy unit sighted!");
 				}
 			}
 			else if(nfriends == 0 || nenemies == 0) {
 				if(nfriends > 0) {
-					msg_id = "friendly_units_sighted";
+					msg_id = N_("$friends Friendly units sighted");
 				} else {
-					msg_id = "enemy_units_sighted";
+					msg_id = N_("$enemies Enemy units sighted!");
 				}
 			}
 			else {
-				msg_id = "units_sighted";
+				msg_id = N_("Units sighted! ($friends friendly, $enemies enemy)");
 			}
 
 			string_map symbols;
@@ -1728,9 +1784,9 @@ size_t move_unit(display* disp, const game_data& gamedata,
 			symbols["enemies"] = lexical_cast<std::string>(nenemies);
 
 			std::stringstream msg;
-			msg << string_table[msg_id];
+			msg << gettext(msg_id);
 			
-			if(u.movement_left() > 0) {
+			if(steps.size() < route.size()) {
 				//see if the "Continue Move" action has an associated hotkey
 				const std::vector<hotkey::hotkey_item>& hotkeys = hotkey::get_hotkeys();
 				std::vector<hotkey::hotkey_item>::const_iterator hk;
@@ -1741,14 +1797,12 @@ size_t move_unit(display* disp, const game_data& gamedata,
 				}
 				if(hk != hotkeys.end()) {
 					symbols["hotkey"] = hotkey::get_hotkey_name(*hk);
-					msg << '\n' << string_table["press_to_continue"];
+					msg << '\n' << _("(press $hotkey to continue)");
 				}
 			}
-			std::cerr << "formatting string...\n";
 			const std::string message = config::interpolate_variables_into_string(msg.str(),&symbols);
 
-			std::cerr << "displaying label...\n";
-			font::add_floating_label(message,24,font::BAD_COLOUR,
+			font::add_floating_label(message,font::SIZE_XLARGE,font::BAD_COLOUR,
 			                         disp->map_area().w/2,disp->map_area().h/3,
 									 0.0,0.0,100,disp->map_area(),font::CENTER_ALIGN);
 		}
@@ -1757,6 +1811,10 @@ size_t move_unit(display* disp, const game_data& gamedata,
 		disp->recalculate_minimap();
 	}
 
+	for(std::set<gamemap::location>::const_iterator seen = seen_units.begin(); seen != seen_units.end(); ++seen) {
+		game_events::fire("sighted",*seen,steps.back());
+	}
+
 	assert(steps.size() <= route.size());
 
 	return steps.size();
@@ -1786,7 +1844,7 @@ bool unit_can_move(const gamemap::location& loc, const unit_map& units,
 		if(map.on_board(locs[n])) {
 			const unit_map::const_iterator i = units.find(locs[n]);
 			if(i != units.end()) {
-				if(current_team.is_enemy(i->second.side())) {
+				if(i->second.incapacitated() == false && current_team.is_enemy(i->second.side())) {
 					return true;
 				}
 			}
