@@ -1,4 +1,4 @@
-/* $Id: actions.hpp,v 1.43 2004/06/07 20:19:27 Sirp Exp $ */
+/* $Id: actions.hpp,v 1.45 2004/08/25 02:59:50 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -58,6 +58,7 @@ struct battle_stats
 	int ndefends, nattacks;
 	int attack_with, defend_with;
 	bool attacker_plague, defender_plague;
+	bool attacker_slows, defender_slows;
 	std::vector<std::string> attack_calculations, defend_calculations;
 	bool to_the_death, defender_strikes_first;
 };
@@ -162,9 +163,9 @@ int combat_modifier(const gamestatus& status,
 //structure which records information to be able to undo a movement
 struct undo_action {
 	undo_action(unit u,const std::vector<gamemap::location>& rt,int sm,int orig=-1)
-		: affected_unit(u), route(rt), starting_moves(sm), original_village_owner(orig), recall_pos(-1) {}
+		: route(rt), starting_moves(sm), original_village_owner(orig), recall_pos(-1), affected_unit(u) {}
 	undo_action(unit u,const gamemap::location& loc, int pos)
-		: affected_unit(u), recall_loc(loc), recall_pos(pos) {}
+		: recall_loc(loc), recall_pos(pos), affected_unit(u) {}
 	std::vector<gamemap::location> route;
 	int starting_moves;
 	int original_village_owner;
