@@ -1,4 +1,4 @@
-/* $Id: ai.cpp,v 1.108 2004/06/16 12:54:10 Sirp Exp $ */
+/* $Id: ai.cpp,v 1.120 2004/10/04 22:45:37 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -31,6 +31,151 @@
 
 #include <iostream>
 
+#define LOG_AI lg::info(lg::ai)
+
+///a trivial ai that sits around doing absolutely nothing
+class idle_ai : public ai_interface {
+public:
+	idle_ai(info& i) : ai_interface(i) {}
+	void play_turn() {}
+};
+
+class sample_ai : public ai_interface {
+public:
+	sample_ai(info& i) : ai_interface(i) {}
+
+	void play_turn() {
+		do_attacks();
+		get_villages();
+		do_moves();
+		do_recruitment();
+	}
+
+private:
+	void do_attacks() {
+		std::map<location,paths> possible_moves;
+		move_map srcdst, dstsrc;
+		calculate_possible_moves(possible_moves,srcdst,dstsrc,false);
+
+		for(unit_map::const_iterator i = get_info().units.begin(); i != get_info().units.end(); ++i) {
+			if(current_team().is_enemy(i->second.side())) {
+				location adjacent_tiles[6];
+				get_adjacent_tiles(i->first,adjacent_tiles);
+
+				int best_defense = -1;
+				std::pair<location,location> best_movement;
+
+				for(size_t n = 0; n != 6; ++n) {
+					typedef move_map::const_iterator Itor;
+					std::pair<Itor,Itor> range = dstsrc.equal_range(adjacent_tiles[n]);
+					while(range.first != range.second) {
+						const location& dst = range.first->first;
+						const location& src = range.first->second;
+						const unit_map::const_iterator un = get_info().units.find(src);
+
+						const gamemap::TERRAIN terrain = get_info().map.get_terrain(dst);
+
+						const int chance_to_hit = un->second.defense_modifier(get_info().map,terrain);
+
+						if(best_defense == -1 || chance_to_hit < best_defense) {
+							best_defense = chance_to_hit;
+							best_movement = *range.first;
+                        }
+
+						++range.first;
+					}
+				}
+
+				if(best_defense != -1) {
+					move_unit(best_movement.second,best_movement.first,possible_moves);
+					const int weapon = choose_weapon(best_movement.first,i->first);
+					attack_enemy(best_movement.first,i->first,weapon);
+					do_attacks();
+					return;
+				}
+			}
+		}
+	}
+
+	int choose_weapon(const location& attacker, const location& defender) {
+		const unit_map::const_iterator att = get_info().units.find(attacker);
+        assert(att != get_info().units.end());
+        
+        const std::vector<attack_type>& attacks = att->second.attacks();
+
+        int best_attack_rating = -1;
+        int best_attack = -1;
+        for(size_t n = 0; n != attacks.size(); ++n) {
+			const battle_stats stats = evaluate_battle_stats(get_info().map,attacker,defender,n,get_info().units,get_info().state,get_info().gameinfo,0,false);
+			const int attack_rating = stats.damage_defender_takes*stats.nattacks*stats.chance_to_hit_defender;
+			if(best_attack == -1 || attack_rating > best_attack_rating) {
+                 best_attack = n;
+                 best_attack_rating = attack_rating;
+            }
+		}
+
+		return best_attack;
+	}
+
+	void get_villages() {
+        std::map<location,paths> possible_moves;
+        move_map srcdst, dstsrc;
+        calculate_possible_moves(possible_moves,srcdst,dstsrc,false);
+
+        for(move_map::const_iterator i = dstsrc.begin(); i != dstsrc.end(); ++i) {
+            if(get_info().map.is_village(i->first) && current_team().owns_village(i->first) == false) {
+                move_unit(i->second,i->first,possible_moves);
+                get_villages();
+                return;
+            }
+        }
+	}
+
+	void do_moves() {
+		unit_map::const_iterator leader;
+        for(leader = get_info().units.begin(); leader != get_info().units.end(); ++leader) {
+            if(leader->second.can_recruit() && current_team().is_enemy(leader->second.side())) {
+                break;
+            }
+        }
+
+        if(leader == get_info().units.end())
+            return;
+
+        std::map<location,paths> possible_moves;
+        move_map srcdst, dstsrc;
+        calculate_possible_moves(possible_moves,srcdst,dstsrc,false);
+
+        int closest_distance = -1;
+        std::pair<location,location> closest_move;
+
+        for(move_map::const_iterator i = dstsrc.begin(); i != dstsrc.end(); ++i) {
+            const int distance = distance_between(i->first,leader->first);
+            if(closest_distance == -1 || distance < closest_distance) {
+                closest_distance = distance;
+                closest_move = *i;
+            }
+        }
+
+        if(closest_distance != -1) {
+            move_unit(closest_move.second,closest_move.first,possible_moves);
+            do_moves();
+        }
+    }
+
+	void do_recruitment() {
+		const std::set<std::string>& options = current_team().recruits();
+        const int choice = (rand()%options.size());
+        std::set<std::string>::const_iterator i = options.begin();
+        std::advance(i,choice);
+
+		const bool res = recruit(*i);
+		if(res) {
+			do_recruitment();
+		}
+	}
+};
+
 ai_interface* create_ai(const std::string& name, ai_interface::info& info)
 {
 	//to add an AI of your own, put
@@ -43,15 +188,16 @@ ai_interface* create_ai(const std::string& name, ai_interface::info& info)
 	else if(name == "idle_ai")
 		return new idle_ai(info);
 	else if(name != "")
-		std::cerr << "ERROR: AI not found: '" << name << "'\n";
+		lg::err(lg::ai) << "AI not found: '" << name << "'\n";
 
 	return new ai(info);
 }
 
 ai::ai(ai_interface::info& info)
-	   : ai_interface(info), disp_(info.disp), map_(info.map), gameinfo_(info.gameinfo), units_(info.units),
-	     teams_(info.teams), team_num_(info.team_num), state_(info.state),
-		 consider_combat_(true), threats_found_(false)
+	   : ai_interface(info), threats_found_(false), disp_(info.disp),
+	     map_(info.map), gameinfo_(info.gameinfo), units_(info.units),
+	     teams_(info.teams), team_num_(info.team_num),
+	     state_(info.state), consider_combat_(true), attack_depth_(0)
 {}
 
 bool ai::recruit_usage(const std::string& usage)
@@ -60,8 +206,8 @@ bool ai::recruit_usage(const std::string& usage)
 
 	const int min_gold = 0;
 	
-	log_scope("recruiting troops");
-	std::cerr << "recruiting " << usage << "\n";
+	log_scope2(ai, "recruiting troops");
+	LOG_AI << "recruiting " << usage << "\n";
 
 	std::vector<std::string> options;
 
@@ -76,7 +222,7 @@ bool ai::recruit_usage(const std::string& usage)
 		if(i->second.usage() == usage && recruits.count(name)
 		   && current_team().gold() - i->second.cost() > min_gold
 		   && not_recommended_units_.count(name) == 0) {
-			std::cerr << "recommending '" << name << "'\n";
+			LOG_AI << "recommending '" << name << "'\n";
 			options.push_back(name);
 		}
 	}
@@ -87,7 +233,7 @@ bool ai::recruit_usage(const std::string& usage)
 		return recruit(options[option]);
 	}
 
-	std::cerr << "no available units to recruit that come under the price\n";
+	LOG_AI << "no available units to recruit that come under the price\n";
 
 	return false;
 }
@@ -191,7 +337,8 @@ void ai_interface::sync_network()
 
 gamemap::location ai_interface::move_unit(location from, location to, std::map<location,paths>& possible_moves)
 {
-	std::cerr << "ai_interface::move_unit " << (from.x+1) << "," << (from.y+1) << " -> " << (to.x+1) << "," << (to.y+1) << "\n";
+	LOG_AI << "ai_interface::move_unit " << (from.x + 1) << "," << (from.y + 1)
+		<< " -> " << (to.x + 1) << "," << (to.y + 1) << "\n";
 	//stop the user from issuing any commands while the unit is moving
 	const command_disabler disable_commands(&info_.disp);
 
@@ -200,17 +347,16 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 	info_.disp.select_hex(from);
 	info_.disp.update_display();
 
-	log_scope("move_unit");
+	log_scope2(ai, "move_unit");
 	unit_map::iterator u_it = info_.units.find(from);
 	if(u_it == info_.units.end()) {
-		std::cerr  << "Could not find unit at " << from.x << ", "
-		          << from.y << "\n";
+		lg::err(lg::ai) << "Could not find unit at " << from.x << ", " << from.y << "\n";
 		assert(false);
 		return location();
 	}
 
 	if(from == to) {
-		std::cerr << "moving unit at " << (from.x+1) << "," << (from.y+1) << " on spot. resetting moves\n";
+		LOG_AI << "moving unit at " << (from.x+1) << "," << (from.y+1) << " on spot. resetting moves\n";
 		u_it->second.set_movement(0);
 		return to;
 	}
@@ -222,19 +368,11 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 	paths current_paths(info_.map,info_.state,info_.gameinfo,info_.units,from,info_.teams,ignore_zocs,teleport);
 	paths_wiper wiper(info_.disp);
 
-	if(show_move) {
-		if(!info_.disp.fogged(from.x,from.y))
-			info_.disp.set_paths(&current_paths);
-
-		info_.disp.scroll_to_tiles(from.x,from.y,to.x,to.y);
-	}
-
 	unit current_unit = u_it->second;
 	info_.units.erase(u_it);
 
 	const std::map<location,paths>::iterator p_it = possible_moves.find(from);
 
-	bool ambushed = false;
 	if(p_it != possible_moves.end()) {
 		paths& p = p_it->second;
 		std::map<location,paths::route>::iterator rt = p.routes.begin();
@@ -275,8 +413,14 @@ gamemap::location ai_interface::move_unit(location from, location to, std::map<l
 
 			steps.push_back(to); //add the destination to the steps
 
-			if(show_move) {
-				unit_display::move_unit(info_.disp,info_.map,steps,current_unit);
+			if(show_move && unit_display::unit_visible_on_path(info_.disp,info_.map,steps,current_unit,info_.state.get_time_of_day(),info_.units,info_.teams)) {
+
+				if(!info_.disp.fogged(from.x,from.y)) {
+					info_.disp.set_paths(&current_paths);
+				}
+
+				info_.disp.scroll_to_tiles(from.x,from.y,to.x,to.y);
+				unit_display::move_unit(info_.disp,info_.map,steps,current_unit,info_.state.get_time_of_day(),info_.units,info_.teams);
 			}
 		}
 	}
@@ -310,13 +454,14 @@ bool ai::multistep_move_possible(location from, location to, location via, std::
 	const unit_map::const_iterator i = units_.find(from);
 	if(i != units_.end()) {
 		if(from != via && to != via && units_.count(via) == 0) {
-			std::cerr << "when seeing if leader can move from " << (from.x+1) << "," << (from.y+1) << " -> " << (to.x+1) << "," << (to.y+1) << " seeing if can detour to keep at " << (via.x+1) << "," << (via.y+1) << "\n";
+			LOG_AI << "when seeing if leader can move from "
+				<< (from.x + 1) << "," << (from.y + 1) << " -> " << (to.x + 1) << "," << (to.y + 1)
+				<< " seeing if can detour to keep at " << (via.x + 1) << "," << (via.y + 1) << "\n";
 			const std::map<location,paths>::const_iterator moves = possible_moves.find(from);
 			if(moves != possible_moves.end()) {
 
-				std::cerr << "found leader moves..\n";
+				LOG_AI << "found leader moves..\n";
 
-				bool can_make_it = false;
 				int move_left = 0;
 
 				//see if the unit can make it to 'via', and if it can, how much movement it will have
@@ -324,17 +469,17 @@ bool ai::multistep_move_possible(location from, location to, location via, std::
 				const paths::routes_map::const_iterator itor = moves->second.routes.find(via);
 				if(itor != moves->second.routes.end()) {
 					move_left = itor->second.move_left;
-					std::cerr << "can make it to keep with " << move_left << " movement left\n";
+					LOG_AI << "can make it to keep with " << move_left << " movement left\n";
 					unit temp_unit(i->second);
 					temp_unit.set_movement(move_left);
 					const temporary_unit_placer unit_placer(units_,via,temp_unit);
 					const paths unit_paths(map_,state_,gameinfo_,units_,via,teams_,false,false);
 
-					std::cerr << "found " << unit_paths.routes.size() << " moves for temp leader\n";
+					LOG_AI << "found " << unit_paths.routes.size() << " moves for temp leader\n";
 					
 					//see if this leader could make it back to the keep
 					if(unit_paths.routes.count(to) != 0) {
-						std::cerr << "can make it back to the keep\n";
+						LOG_AI << "can make it back to the keep\n";
 						return true;
 					}
 				}
@@ -584,7 +729,7 @@ void ai::play_turn()
 
 void ai::do_move()
 {
-	log_scope("doing ai move");
+	log_scope2(ai, "doing ai move");
 
 	invalidate_defensive_position_cache();
 
@@ -607,14 +752,12 @@ void ai::do_move()
 		remove_unit_from_moves(leader->first,srcdst,dstsrc);
 	}
 
-	int ticks = SDL_GetTicks();
-
 	std::vector<attack_analysis> analysis;
 
-	AI_DIAGNOSTIC("combat phase");
+	LOG_AI << "combat phase\n";
 
 	if(consider_combat_) {
-		std::cerr << "combat...\n";
+		LOG_AI << "combat...\n";
 		consider_combat_ = do_combat(possible_moves,srcdst,dstsrc,enemy_srcdst,enemy_dstsrc);
 		if(consider_combat_) {
 			do_move();
@@ -624,27 +767,27 @@ void ai::do_move()
 
 	move_leader_to_goals(enemy_srcdst,enemy_dstsrc);
 
-	AI_DIAGNOSTIC("get villages phase");
+	LOG_AI << "get villages phase\n";
 
-	std::cerr << "villages...\n";
+	LOG_AI << "villages...\n";
 	const bool got_village = get_villages(possible_moves,srcdst,dstsrc,enemy_srcdst,enemy_dstsrc,leader);
 	if(got_village) {
 		do_move();
 		return;
 	}
 
-	AI_DIAGNOSTIC("healing phase");
+	LOG_AI << "healing phase\n";
 
-	std::cerr << "healing...\n";
+	LOG_AI << "healing...\n";
 	const bool healed_unit = get_healing(possible_moves,srcdst,dstsrc,enemy_srcdst,enemy_dstsrc);
 	if(healed_unit) {
 		do_move();
 		return;
 	}
 
-	AI_DIAGNOSTIC("retreat phase");
+	LOG_AI << "retreat phase\n";
 
-	std::cerr << "retreating...\n";
+	LOG_AI << "retreating...\n";
 	const bool retreated_unit = retreat_units(possible_moves,srcdst,dstsrc,enemy_srcdst,enemy_dstsrc,leader);
 	if(retreated_unit) {
 		do_move();
@@ -657,18 +800,18 @@ void ai::do_move()
 
 	find_threats();
 
-	AI_DIAGNOSTIC("move/targetting phase");
+	LOG_AI << "move/targetting phase\n";
 
 	const bool met_invisible_unit = move_to_targets(possible_moves,srcdst,dstsrc,enemy_srcdst,enemy_dstsrc,leader);
 	if(met_invisible_unit) {
-		std::cerr << "met_invisible_unit\n";
+		LOG_AI << "met_invisible_unit\n";
 		do_move();
 		return;
 	}
 
-	std::cerr << "done move to targets\n";
+	LOG_AI << "done move to targets\n";
 
-	AI_DIAGNOSTIC("leader/recruitment phase");
+	LOG_AI << "leader/recruitment phase\n";
 
 	//recruitment phase and leader movement phase
 	if(leader != units_.end()) {
@@ -684,8 +827,6 @@ void ai::do_move()
 		}
 	}
 
-	AI_DIAGNOSTIC("");
-
 	recorder.end_turn();
 	sync_network();
 }
@@ -697,7 +838,7 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 	std::vector<attack_analysis> analysis = analyze_targets(srcdst,dstsrc,enemy_srcdst,enemy_dstsrc);
 
 	int time_taken = SDL_GetTicks() - ticks;
-	std::cerr  << "took " << time_taken << " ticks for " << analysis.size() << " positions. Analyzing...\n";
+	LOG_AI << "took " << time_taken << " ticks for " << analysis.size() << " positions. Analyzing...\n";
 
 	ticks = SDL_GetTicks();
 
@@ -708,7 +849,7 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 	if(num_sims > 40)
 		num_sims = 40;
 
-	std::cerr  << "simulations: " << num_sims << "\n";
+	LOG_AI << "simulations: " << num_sims << "\n";
 
 	const int max_positions = 30000;
 	const int skip_num = analysis.size()/max_positions;
@@ -720,7 +861,7 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 			continue;
 
 		const double rating = it->rating(current_team().aggression(),*this);
-		std::cerr  << "attack option rated at " << rating << " (" << current_team().aggression() << ")\n";
+		LOG_AI << "attack option rated at " << rating << " (" << current_team().aggression() << ")\n";
 		if(rating > choice_rating) {
 			choice_it = it;
 			choice_rating = rating;
@@ -728,7 +869,7 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 	}
 
 	time_taken = SDL_GetTicks() - ticks;
-	std::cerr  << "analysis took " << time_taken << " ticks\n";
+	LOG_AI << "analysis took " << time_taken << " ticks\n";
 
 	if(choice_rating > 0.0) {
 		location from   = choice_it->movements[0].first;
@@ -738,13 +879,12 @@ bool ai::do_combat(std::map<gamemap::location,paths>& possible_moves, const move
 
 		const unit_map::const_iterator tgt = units_.find(target_loc);
 
-		const bool defender_human = (tgt != units_.end()) ?
-		             teams_[tgt->second.side()-1].is_human() : false;
-
 		const location arrived_at = move_unit(from,to,possible_moves);
 		if(arrived_at != to) {
-			std::cerr << "unit moving to attack has ended up unexpectedly at " << (arrived_at.x+1) << "," << (arrived_at.y+1) << " when moving to "
-			          << (to.x+1) << "," << (to.y+1) << " moved from " << (from.x+1) << "," << (from.y+1) << "\n";
+			lg::warn(lg::ai) << "unit moving to attack has ended up unexpectedly at "
+				<< (arrived_at.x + 1) << "," << (arrived_at.y + 1) << " when moving to "
+				<< (to.x + 1) << "," << (to.y + 1) << " moved from "
+				<< (from.x + 1) << "," << (from.y + 1) << "\n";
 			return true;
 		}
 
@@ -770,7 +910,7 @@ void ai_interface::attack_enemy(const location& u, const location& target, int w
 
 	if(info_.units.count(u) && info_.units.count(target)) {
 		if(info_.units.find(target)->second.stone()) {
-			std::cerr << "ERROR: attempt to attack unit that is turned to stone\n";
+			lg::err(lg::ai) << "attempt to attack unit that is turned to stone\n";
 			return;
 		}
 
@@ -844,7 +984,7 @@ std::vector<std::pair<gamemap::location,gamemap::location> > ai::get_village_com
 
 bool ai::get_villages(std::map<gamemap::location,paths>& possible_moves, const move_map& srcdst, const move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc, unit_map::const_iterator leader)
 {
-	std::cerr << "deciding which villages we want...\n";
+	LOG_AI << "deciding which villages we want...\n";
 
 	location start_pos;
 
@@ -916,10 +1056,10 @@ bool ai::get_villages(std::map<gamemap::location,paths>& possible_moves, const m
 
 	std::set<location> taken_villages, moved_units;
 	const int ticks = SDL_GetTicks();
-	std::cerr << "get_villages()..." << village_moves.size() << "\n";
+	LOG_AI << "get_villages()..." << village_moves.size() << "\n";
 	const std::vector<std::pair<location,location> >& moves = get_village_combinations(possible_moves,srcdst,dstsrc,enemy_srcdst,enemy_dstsrc,leader,
 	                                                                                   taken_villages,moved_units,village_moves,village_moves.begin());
-	std::cerr << "get_villages() done: " << (SDL_GetTicks() - ticks) << "\n";
+	LOG_AI << "get_villages() done: " << (SDL_GetTicks() - ticks) << "\n";
 
 	//move all the units to get villages, however move the leader last, so that the castle will be cleared
 	//if it wants to stop to recruit along the way.
@@ -934,6 +1074,11 @@ bool ai::get_villages(std::map<gamemap::location,paths>& possible_moves, const m
 				const location loc = move_unit(i->second,i->first,possible_moves);
 				++moves_made;
 
+				//if we didn't make it to the destination, it means we were ambushed.
+				if(loc != i->first) {
+					return true;
+				}
+
 				const unit_map::const_iterator new_unit = units_.find(loc);
 
 				if(new_unit != units_.end() && power_projection(i->first,enemy_srcdst,enemy_dstsrc) >= new_unit->second.hitpoints()/4) {
@@ -977,11 +1122,11 @@ bool ai::get_healing(std::map<gamemap::location,paths>& possible_moves, const mo
 				if(map_.gives_healing(dst) && (units_.find(dst) == units_.end() || dst == u_it->first)) {
 					const double vuln = power_projection(it.first->first,
 					                    enemy_srcdst,enemy_dstsrc);
-					std::cerr << "found village with vulnerability: " << vuln << "\n";
+					LOG_AI << "found village with vulnerability: " << vuln << "\n";
 					if(vuln < best_vulnerability || best_loc == it.second) {
 						best_vulnerability = vuln;
 						best_loc = it.first;
-						std::cerr << "chose village " << (dst.x+1) << "," << (dst.y+1) << "\n";
+						LOG_AI << "chose village " << (dst.x+1) << "," << (dst.y+1) << "\n";
 					}
 				}
 				
@@ -993,7 +1138,7 @@ bool ai::get_healing(std::map<gamemap::location,paths>& possible_moves, const mo
 				const location& src = best_loc->first;
 				const location& dst = best_loc->second;
 
-				std::cerr << "moving unit to village for healing...\n";
+				LOG_AI << "moving unit to village for healing...\n";
 
 				move_unit(src,dst,possible_moves);
 				return true;
@@ -1094,7 +1239,8 @@ bool ai::retreat_units(std::map<gamemap::location,paths>& possible_moves, const
 				}
 
 				if(best_pos.valid()) {
-					std::cerr << "retreating '" << i->second.type().name() << "' " << i->first.x << "," << i->first.y << " -> " << best_pos.x << "," << best_pos.y << "\n";
+					LOG_AI << "retreating '" << i->second.type().name() << "' " << i->first.x << ","
+						<< i->first.y << " -> " << best_pos.x << "," << best_pos.y << "\n";
 					move_unit(i->first,best_pos,possible_moves);
 					return true;
 				}
@@ -1107,7 +1253,7 @@ bool ai::retreat_units(std::map<gamemap::location,paths>& possible_moves, const
 
 bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move_map& srcdst, move_map& dstsrc, const move_map& enemy_srcdst, const move_map& enemy_dstsrc, unit_map::const_iterator leader)
 {
-	std::cerr << "finding targets...\n";
+	LOG_AI << "finding targets...\n";
 	std::vector<target> targets;
 	for(;;) {
 		if(targets.empty()) {
@@ -1119,7 +1265,7 @@ bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move
 			}
 		}
 
-		std::cerr << "choosing move...\n";
+		LOG_AI << "choosing move...\n";
 		std::pair<location,location> move = choose_move(targets,srcdst,dstsrc,enemy_srcdst,enemy_dstsrc);
 		for(std::vector<target>::const_iterator ittg = targets.begin(); ittg != targets.end(); ++ittg) {
 			assert(map_.on_board(ittg->loc));
@@ -1133,15 +1279,16 @@ bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move
 			return true;
 		}
 
-		std::cerr << "move: " << move.first.x << ", " << move.first.y << " - " << move.second.x << ", " << move.second.y << "\n";
+		LOG_AI << "move: " << move.first.x << ", " << move.first.y << " - " << move.second.x << ", " << move.second.y << "\n";
 
 		//search to see if there are any enemy units next
-		//to the tile which really should be attacked once the move is done
+		//to the tile which really should be attacked once the move is done.
 		gamemap::location adj[6];
 		get_adjacent_tiles(move.second,adj);
 		battle_stats bat_stats;
 		gamemap::location target;
 		int weapon = -1;
+
 		for(int n = 0; n != 6; ++n) {
 			const unit_map::iterator enemy = find_visible_unit(units_,adj[n],
 					map_,
@@ -1150,19 +1297,25 @@ bool ai::move_to_targets(std::map<gamemap::location,paths>& possible_moves, move
 
 			if(enemy != units_.end() &&
 			   current_team().is_enemy(enemy->second.side()) && enemy->second.stone() == false) {
-				target = adj[n];
-				weapon = choose_weapon(move.first,target,bat_stats,
+				const int res = choose_weapon(move.first,adj[n],bat_stats,
 				                       map_[move.second.x][move.second.y]);
-				break;
+
+				//current behavior is to only make risk-free attacks
+				if(bat_stats.damage_attacker_takes == 0) {
+					weapon = res;
+					target = adj[n];
+					break;
+				}
 			}
 		}
+		
 
 		const location arrived_at = move_unit(move.first,move.second,possible_moves);
 
 		//we didn't arrive at our intended destination. We return true, meaning that
 		//the AI algorithm should be recalculated from the start.
 		if(arrived_at != move.second) {
-			std::cerr << "didn't arrive at destination\n";
+			lg::warn(lg::ai) << "didn't arrive at destination\n";
 			return true;
 		}
 
@@ -1194,7 +1347,7 @@ int ai::average_resistance_against(const unit_type& a, const unit_type& b) const
 
 	defense /= weighting_sum;
 
-	std::cerr << "average defense of '" << a.name() << "': " << defense << "\n";
+	LOG_AI << "average defense of '" << a.name() << "': " << defense << "\n";
 
 	int sum = 0, weight_sum = 0;
 
@@ -1215,7 +1368,7 @@ int ai::compare_unit_types(const unit_type& a, const unit_type& b) const
 	const int a_effectiveness_vs_b = average_resistance_against(b,a);
 	const int b_effectiveness_vs_a = average_resistance_against(a,b);
 
-	std::cerr << "comparison of '" << a.name() << " vs " << b.name() << ": "
+	LOG_AI << "comparison of '" << a.name() << " vs " << b.name() << ": "
 	          << a_effectiveness_vs_b << " - " << b_effectiveness_vs_a << " = "
 			  << (a_effectiveness_vs_b - b_effectiveness_vs_a) << "\n";
 	return a_effectiveness_vs_b - b_effectiveness_vs_a;
@@ -1227,7 +1380,7 @@ void ai::analyze_potential_recruit_combat()
 		return;
 	}
 
-	log_scope("analyze_potential_recruit_combat()");
+	log_scope2(ai, "analyze_potential_recruit_combat()");
 
 	//records the best combat analysis for each usage type
 	std::map<std::string,int> best_usage;
@@ -1255,7 +1408,7 @@ void ai::analyze_potential_recruit_combat()
 			score /= weighting;
 		}
 
-		std::cerr << "combat score of '" << *i << "': " << score << "\n";
+		LOG_AI << "combat score of '" << *i << "': " << score << "\n";
 		unit_combat_scores_[*i] = score;
 
 		if(best_usage.count(info->second.usage()) == 0 || score > best_usage[info->second.usage()]) {
@@ -1272,7 +1425,7 @@ void ai::analyze_potential_recruit_combat()
 		}
 
 		if(unit_combat_scores_[*i] + 1000 < best_usage[info->second.usage()]) {
-			std::cerr << "recommending not to use '" << *i << "' because of poor combat performance "
+			LOG_AI << "recommending not to use '" << *i << "' because of poor combat performance "
 				      << unit_combat_scores_[*i] << "/" << best_usage[info->second.usage()] << "\n";
 			not_recommended_units_.insert(*i);
 		}
@@ -1310,7 +1463,7 @@ void ai::analyze_potential_recruit_movements()
 		return;
 	}
 
-	log_scope("analyze_potential_recruit_movements()");
+	log_scope2(ai, "analyze_potential_recruit_movements()");
 
 	const int max_targets = 5;
 
@@ -1323,7 +1476,7 @@ void ai::analyze_potential_recruit_movements()
 
 	const std::set<std::string>& recruits = current_team().recruits();
 
-	std::cerr << "targets: " << targets.size() << "\n";
+	LOG_AI << "targets: " << targets.size() << "\n";
 
 	std::map<std::string,int> best_scores;
 	
@@ -1343,14 +1496,14 @@ void ai::analyze_potential_recruit_movements()
 
 		const shortest_path_calculator calc(temp_unit,current_team(),units,teams_,map_,state_);
 		for(std::vector<target>::const_iterator t = targets.begin(); t != targets.end(); ++t) {
-			std::cerr << "analyzing '" << *i << "' getting to target...\n";
+			LOG_AI << "analyzing '" << *i << "' getting to target...\n";
 			const paths::route& route = a_star_search(start,t->loc,100.0,calc);
 			if(route.steps.empty() == false) {
-				std::cerr << "made it: " << route.move_left << "\n";
+				LOG_AI << "made it: " << route.move_left << "\n";
 				cost += route.move_left;
 				++targets_reached;
 			} else {
-				std::cerr << "failed\n";
+				LOG_AI << "failed\n";
 				++targets_missed;
 			}
 		}
@@ -1380,10 +1533,10 @@ void ai::analyze_potential_recruit_movements()
 		if(best_score > 0) {
 			j->second = (j->second*10)/best_score;
 			if(j->second > 15) {
-				std::cerr << "recommending against recruiting '" << j->first << "' (score: " << j->second << ")\n";
+				LOG_AI << "recommending against recruiting '" << j->first << "' (score: " << j->second << ")\n";
 				not_recommended_units_.insert(j->first);
 			} else {
-				std::cerr << "recommending recruit of '" << j->first << "' (score: " << j->second << ")\n";
+				LOG_AI << "recommending recruit of '" << j->first << "' (score: " << j->second << ")\n";
 			}
 		}
 	}
@@ -1439,7 +1592,7 @@ void ai::do_recruitment()
 	//get scouts depending on how many neutral villages there are
 	int scouts_wanted = villages_per_scout > 0 ? neutral_villages/villages_per_scout : 0;
 
-	std::cerr << "scouts_wanted: " << neutral_villages << "/" << villages_per_scout << " = " << scouts_wanted << "\n";
+	LOG_AI << "scouts_wanted: " << neutral_villages << "/" << villages_per_scout << " = " << scouts_wanted << "\n";
 
 	std::map<std::string,int> unit_types;
 
@@ -1449,7 +1602,7 @@ void ai::do_recruitment()
 		}
 	}
 
-	std::cerr << "we have " << unit_types["scout"] << " scouts already and we want " << scouts_wanted << " in total\n";
+	LOG_AI << "we have " << unit_types["scout"] << " scouts already and we want " << scouts_wanted << " in total\n";
 
 	while(unit_types["scout"] < scouts_wanted) {
 		if(recruit_usage("scout") == false)
@@ -1475,7 +1628,7 @@ void ai::move_leader_to_goals(const move_map& enemy_srcdst, const move_map& enem
 	const config* const goal = current_team().ai_parameters().child("leader_goal");
 
 	if(goal == NULL) {
-		AI_LOG("No goal found");
+		LOG_AI << "No goal found\n";
 		return;
 	}
 
@@ -1483,17 +1636,17 @@ void ai::move_leader_to_goals(const move_map& enemy_srcdst, const move_map& enem
 
 	const unit_map::iterator leader = find_leader(units_,team_num_);
 	if(leader == units_.end() || leader->second.incapacitated()) {
-		AI_LOG("leader not found");
+		LOG_AI << "leader not found\n";
 		return;
 	}
 
-	AI_LOG("Doing recruitment before goals");
+	LOG_AI << "Doing recruitment before goals\n";
 	
 	do_recruitment();
 
 	const paths::route route = a_star_search(leader->first,dst,1000.0,shortest_path_calculator(leader->second,current_team(),units_,teams_,map_,state_));
 	if(route.steps.empty()) {
-		AI_LOG("route empty");
+		LOG_AI << "route empty";
 		return;
 	}
 
@@ -1510,7 +1663,7 @@ void ai::move_leader_to_goals(const move_map& enemy_srcdst, const move_map& enem
 	}
 
 	if(loc.valid()) {
-		AI_LOG("Moving leader to goal");
+		LOG_AI << "Moving leader to goal\n";
 		move_unit(leader->first,loc,possible_moves);
 	}
 }
@@ -1564,7 +1717,7 @@ void ai::move_leader_to_keep(const move_map& enemy_dstsrc)
 
 void ai::move_leader_after_recruit(const move_map& enemy_dstsrc)
 {
-	std::cerr << "moving leader after recruit...\n";
+	LOG_AI << "moving leader after recruit...\n";
 
 	const unit_map::iterator leader = find_leader(units_,team_num_);
 	if(leader == units_.end() || leader->second.incapacitated()) {
@@ -1598,7 +1751,8 @@ void ai::move_leader_after_recruit(const move_map& enemy_dstsrc)
 
 				//if this location is in range of the village, then we consider it
 				if(current_loc.valid()) {
-					AI_LOG("considering movement to " + str_cast(current_loc.x+1) + "," + str_cast(current_loc.y+1));
+					LOG_AI << "considering movement to " << str_cast(current_loc.x + 1)
+						<< "," << str_cast(current_loc.y+1);
 					unit_map temp_units;
 					temp_units.insert(std::pair<location,unit>(current_loc,leader->second));
 					const paths p(map_,state_,gameinfo_,temp_units,current_loc,teams_,false,false);
@@ -1831,4 +1985,15 @@ const std::set<gamemap::location>& ai::avoided_locations() const
 	}
 
 	return avoid_;
-}
\ No newline at end of file
+}
+
+int ai::attack_depth() const
+{
+	if(attack_depth_ > 0) {
+		return attack_depth_;
+	}
+
+	const config& parms = current_team().ai_parameters();
+	attack_depth_ = maximum<int>(1,lexical_cast_default<int>(parms["attack_depth"],5));
+	return attack_depth_;
+}
