@@ -1,4 +1,4 @@
-/* $Id: ai_attack.cpp,v 1.46 2004/06/07 20:19:27 Sirp Exp $ */
+/* $Id: ai_attack.cpp,v 1.51 2004/09/19 20:52:17 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -24,6 +24,8 @@
 #include <iostream>
 #include <set>
 
+#define LOG_AI lg::info(lg::ai)
+
 const int max_positions = 10000;
 
 //analyze possibility of attacking target on 'loc'
@@ -41,7 +43,7 @@ void ai::do_attack_analysis(
 	//this function is called fairly frequently, so interact with the user here.
 	user_interact();
 
-	if(cur_analysis.movements.size() > 4)
+	if(cur_analysis.movements.size() >= size_t(attack_depth()))
 		return;
 
 	static double best_results[6];
@@ -53,7 +55,7 @@ void ai::do_attack_analysis(
 
 	const size_t max_positions = 1000;
 	if(result.size() > max_positions && !cur_analysis.movements.empty()) {
-		std::cerr << "cut analysis short with number of positions\n";
+		LOG_AI << "cut analysis short with number of positions\n";
 		return;
 	}
 
@@ -76,13 +78,21 @@ void ai::do_attack_analysis(
 
 		//see if the unit has the backstab ability -- units with backstab
 		//will want to try to have a friendly unit opposite the position they move to
-		bool backstab = false;
+		//see if the unit has the slow ability -- units with slow only attack first
+		bool backstab = false, slow = false;
 		const std::vector<attack_type>& attacks = unit_itor->second.attacks();
 		for(std::vector<attack_type>::const_iterator a = attacks.begin(); a != attacks.end(); ++a) {
 			if(a->backstab()) {
 				backstab = true;
-				break;
 			}
+
+			if(a->slow()) {
+				slow = true;
+			}
+		}
+
+		if(slow && cur_analysis.movements.empty() == false) {
+			continue;
 		}
 
 		double best_vulnerability = 0.0, best_support = 0.0;
@@ -222,7 +232,7 @@ bool operator==(const battle_type& a, const battle_type& b)
 std::set<battle_type> weapon_choice_cache;
 
 int ai::choose_weapon(const location& att, const location& def,
-					  battle_stats& cur_stats, gamemap::TERRAIN terrain)
+					  battle_stats& cur_stats, gamemap::TERRAIN terrain, bool use_cache)
 {
 	const std::map<location,unit>::const_iterator itor = units_.find(att);
 	if(itor == units_.end())
@@ -231,6 +241,10 @@ int ai::choose_weapon(const location& att, const location& def,
 	static int cache_hits = 0;
 	static int cache_misses = 0;
 
+	if(use_cache == false) {
+		weapon_choice_cache.clear();
+	}
+
 	battle_type battle(att,def,terrain);
 	const std::set<battle_type>::const_iterator cache_itor = weapon_choice_cache.find(battle);
 
@@ -241,7 +255,7 @@ int ai::choose_weapon(const location& att, const location& def,
 		cur_stats = cache_itor->stats;
 
 		if(!(size_t(cache_itor->weapon) < itor->second.attacks().size())) {
-			std::cerr << "cached illegal weapon: " << cache_itor->weapon
+			lg::err(lg::ai) << "cached illegal weapon: " << cache_itor->weapon
 			          << "/" << itor->second.attacks().size() << "\n";
 		}
 
@@ -252,7 +266,7 @@ int ai::choose_weapon(const location& att, const location& def,
 	++cache_misses;
 
 	if((cache_misses%100) == 0) {
-		std::cerr << "cache_stats: " << cache_hits << ":" << cache_misses << " " << weapon_choice_cache.size() << "\n";
+		LOG_AI << "cache_stats: " << cache_hits << ":" << cache_misses << " " << weapon_choice_cache.size() << "\n";
 	}
 
 	int current_choice = -1;
@@ -339,7 +353,7 @@ void ai::attack_analysis::analyze(const gamemap& map,
 	std::vector<std::pair<location,location> >::const_iterator m;
 	for(m = movements.begin(); m != movements.end(); ++m) {
 		battle_stats bat_stats;
-		const int weapon = ai_obj.choose_weapon(m->first,target, bat_stats, map[m->second.x][m->second.y]);
+		const int weapon = ai_obj.choose_weapon(m->first,target, bat_stats, map[m->second.x][m->second.y],true);
 
 		assert(weapon != -1);
 		weapons.push_back(weapon);
@@ -352,6 +366,8 @@ void ai::attack_analysis::analyze(const gamemap& map,
 
 		int defenderxp = 0;
 
+		bool defender_slowed = false;
+
 		int defhp = target_hp;
 		for(size_t i = 0; i != movements.size() && defhp; ++i) {
 			const battle_stats& stat = stats[i];
@@ -360,6 +376,13 @@ void ai::attack_analysis::analyze(const gamemap& map,
 			int attacks = stat.nattacks;
 			int defends = stat.ndefends;
 
+			if(defender_slowed && defends > 1) {
+				--defends;
+
+				//give an extra bonus based on slowing here
+				avg_damage_taken -= stat.damage_defender_takes;
+			}
+
 			unit_map::const_iterator att = units.find(movements[i].first);
 			double cost = att->second.type().cost();
 
@@ -389,8 +412,13 @@ void ai::attack_analysis::analyze(const gamemap& map,
 						}
 
 						atthp += stat.amount_attacker_drains;
-						if(atthp > hitpoints[i])
+						if(atthp > hitpoints[i]) {
 							atthp = hitpoints[i];
+						}
+
+						if(stat.attacker_slows && !defender_slowed && defend_it->second.has_flag("slowed") == false) {
+							defender_slowed = true;
+						}
 					}
 
 					--attacks;
@@ -413,8 +441,9 @@ void ai::attack_analysis::analyze(const gamemap& map,
 						}
 
 						defhp += stat.amount_defender_drains;
-						if(defhp > target_max_hp)
+						if(defhp > target_max_hp) {
 							defhp = target_max_hp;
+						}
 					}
 
 					--defends;
@@ -516,7 +545,7 @@ double ai::attack_analysis::rating(double aggression, ai& ai_obj) const
 	//only use the leader if we do a serious amount of damage
 	//compared to how much they do to us.
 	if(uses_leader && aggression > -4.0) {
-		std::cerr << "uses leader..\n";
+		LOG_AI << "uses leader..\n";
 		aggression = -4.0;
 	}
 
@@ -529,7 +558,8 @@ double ai::attack_analysis::rating(double aggression, ai& ai_obj) const
 
 		const double exposure_mod = uses_leader ? 2.0 : ai_obj.current_team().caution();
 		const double exposure = exposure_mod*resources_used*(terrain_quality - alternative_terrain_quality)*vulnerability/maximum<double>(0.01,support);
-		std::cerr << "attack option has base value " << value << " with exposure " << exposure << ": " << vulnerability << "/" << support << " = " << (vulnerability/support) << "\n";
+		LOG_AI << "attack option has base value " << value << " with exposure " << exposure << ": "
+			<< vulnerability << "/" << support << " = " << (vulnerability/support) << "\n";
 		if(uses_leader) {
 			ai_obj.log_message("attack option has value " + str_cast(value) + " with exposure " + str_cast(exposure) + ": " + str_cast(vulnerability) + "/" + str_cast(support));
 		}
@@ -562,9 +592,11 @@ double ai::attack_analysis::rating(double aggression, ai& ai_obj) const
 		value *= 5.0;
 	}
 
-	std::cerr << "attack on " << (target.x+1) << "," << (target.y+1) << ": attackers: " << movements.size() << " value: " << value
-	          << " chance to kill: " << chance_to_kill << " damage inflicted: " << avg_damage_inflicted << " damage taken: " << avg_damage_taken
-		      << " vulnerability: " << vulnerability << " support: " << support << " quality: " << terrain_quality << " alternative quality: " << alternative_terrain_quality << "\n";
+	LOG_AI << "attack on " << (target.x+1) << "," << (target.y+1) << ": attackers: " << movements.size()
+		<< " value: " << value << " chance to kill: " << chance_to_kill << " damage inflicted: "
+		<< avg_damage_inflicted << " damage taken: " << avg_damage_taken << " vulnerability: "
+		<< vulnerability << " support: " << support << " quality: " << terrain_quality
+		<< " alternative quality: " << alternative_terrain_quality << "\n";
 
 	return value;
 }
@@ -574,7 +606,7 @@ std::vector<ai::attack_analysis> ai::analyze_targets(
 	             const move_map& enemy_srcdst, const move_map& enemy_dstsrc
                 )
 {
-	log_scope("analyzing targets...");
+	log_scope2(ai, "analyzing targets...");
 
 	weapon_choice_cache.clear();
 
