@@ -1,4 +1,4 @@
-/* $Id: builder.cpp,v 1.20 2004/06/08 20:52:34 gruikya Exp $ */
+/* $Id: builder.cpp,v 1.35 2004/09/19 07:24:59 silene Exp $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -11,23 +11,99 @@
    See the COPYING file for more details.
 */
 
+//disable the very annoying VC++ warning 4786
+#ifdef WIN32
+#pragma warning(disable:4786)
+#endif
+
 #include "array.hpp"
 #include "display.hpp"
 #include "log.hpp"
 #include "terrain.hpp"
 #include "util.hpp"
 
+namespace {
 
-terrain_builder::tile::tile() 
+class locator_string_initializer : public animated<image::locator>::string_initializer
+{
+public:
+	locator_string_initializer() : no_loc_(true) {}
+	locator_string_initializer(const gamemap::location& loc): no_loc_(false), loc_(loc)  {}
+	image::locator operator()(const std::string &s) const;
+
+private:
+	bool no_loc_;
+	gamemap::location loc_;
+};
+
+image::locator locator_string_initializer::operator()(const std::string &s) const
+{
+	if(no_loc_) {
+		return image::locator("terrain/" + s + ".png");
+	} else {
+		return image::locator("terrain/" + s + ".png", loc_);
+	}
+}
+
+}
+
+const int terrain_builder::rule_image::TILEWIDTH = 72;
+const int terrain_builder::rule_image::UNITPOS = 36 + 18;
+
+terrain_builder::rule_image::rule_image(int layer, bool global_image) : 
+	position(HORIZONTAL), layer(layer),
+	basex(0), basey(0), global_image(global_image)
+{};
+
+terrain_builder::rule_image::rule_image(int x, int y, bool global_image) : 
+	position(VERTICAL), layer(0),
+	basex(x), basey(y), global_image(global_image)
+{};
+
+terrain_builder::tile::tile() : last_tod("invalid_tod")
 {
        	memset(adjacents, 0, sizeof(adjacents)); 
 }
 
+void terrain_builder::tile::add_image_to_cache(const std::string &tod, ordered_ri_list::const_iterator itor) const
+{
+	rule_image_variantlist::const_iterator tod_variant =
+		itor->second->variants.find(tod);
+
+	if(tod_variant == itor->second->variants.end())
+		tod_variant = itor->second->variants.find("");
+
+	if(tod_variant != itor->second->variants.end()) {
+		if(itor->first < 0) {
+			images_background.push_back(tod_variant->second.image);
+		} else {
+			images_foreground.push_back(tod_variant->second.image);
+		}
+	}
+}
+
+void terrain_builder::tile::rebuild_cache(const std::string &tod) const 
+{
+	images_background.clear();
+	images_foreground.clear();
+
+	ordered_ri_list::const_iterator itor;
+	for(itor = horizontal_images.begin(); itor != horizontal_images.end(); ++itor) {
+		add_image_to_cache(tod, itor);
+	}
+	for(itor = vertical_images.begin(); itor != vertical_images.end(); ++itor) {
+		add_image_to_cache(tod, itor);
+	}
+}
+
 void terrain_builder::tile::clear() 
 {
 	flags.clear();
+	horizontal_images.clear();
+	vertical_images.clear();
 	images_foreground.clear();
 	images_background.clear();
+	last_tod = "invalid_tod";
 	memset(adjacents, 0, sizeof(adjacents));
 }
 
@@ -59,57 +135,203 @@ const terrain_builder::tile& terrain_builder::tilemap::operator[] (const gamemap
 	return map_[(loc.x+1) + (loc.y+1)*(x_+2)];
 }
 
-terrain_builder::terrain_builder(const config& cfg, const gamemap& gmap) :
+terrain_builder::terrain_builder(const config& cfg, const config& level, const gamemap& gmap) :
 	map_(gmap), tile_map_(gmap.x(), gmap.y())
 {
 	parse_config(cfg);
+	parse_config(level);
 	build_terrains();
 	//rebuild_terrain(gamemap::location(0,0));
 }
 
-const std::vector<image::locator> *terrain_builder::get_terrain_at(const gamemap::location &loc,
-								   ADJACENT_TERRAIN_TYPE terrain_type) const 
+const terrain_builder::imagelist *terrain_builder::get_terrain_at(const gamemap::location &loc,
+		const std::string &tod, ADJACENT_TERRAIN_TYPE terrain_type) const 
 {
 	if(!tile_map_.on_map(loc))
 		return NULL;
 
+	const tile& tile_at = tile_map_[loc];
+
+	if(tod != tile_at.last_tod) {
+		tile_at.rebuild_cache(tod);
+		tile_at.last_tod = tod;
+	}
+
 	if(terrain_type == ADJACENT_BACKGROUND) {
-		if(tile_map_[loc].images_background.size())
-			return &tile_map_[loc].images_background;
+		if(!tile_at.images_background.empty())
+			return &tile_at.images_background;
 	}
 	
 	if(terrain_type == ADJACENT_FOREGROUND) {
-		if(tile_map_[loc].images_foreground.size())
-			return &tile_map_[loc].images_foreground;
+		if(!tile_at.images_foreground.empty())
+			return &tile_at.images_foreground;
 	}
 
 	return NULL;
 }
 
+bool terrain_builder::update_animation(const gamemap::location &loc)
+{
+	if(!tile_map_.on_map(loc))
+		return false;
+
+	imagelist& bg = tile_map_[loc].images_background;
+	imagelist& fg = tile_map_[loc].images_foreground;
+	bool changed = false;
+
+	imagelist::iterator itor = bg.begin();
+	for(; itor != bg.end(); ++itor) {
+		itor->update_current_frame();
+		if(itor->frame_changed())
+			changed = true;
+	}
+
+	itor = fg.begin();
+	for(; itor != fg.end(); ++itor) {
+		itor->update_current_frame();
+		if(itor->frame_changed())
+			changed = true;
+	}
+
+	return changed;
+}
+
+// TODO: rename this function 
 void terrain_builder::rebuild_terrain(const gamemap::location &loc)
 {
 	if (tile_map_.on_map(loc)) {
 		tile& btile = tile_map_[loc];
-		btile.clear();
+		// btile.images.clear();
+		btile.images_foreground.clear();
+		btile.images_background.clear();
 		const std::string filename =
 			map_.get_terrain_info(map_.get_terrain(loc)).default_image();
-		image::locator img_loc(filename);
-		btile.images_foreground.push_back(img_loc);
+		animated<image::locator> img_loc("terrain/" + filename + ".png");
+		img_loc.start_animation(0, animated<image::locator>::INFINITE_CYCLES);
+		btile.images_background.push_back(img_loc);
 	}
 }
 
-void terrain_builder::rebuild_all() {
+void terrain_builder::rebuild_all()
+{
 	tile_map_.reset();
 	terrain_by_type_.clear();
+	terrain_by_type_border_.clear();
 	build_terrains();
 }
 
+bool terrain_builder::rule_valid(const building_rule &rule)
+{
+	//if the rule has no constraints, it is invalid
+	if(rule.constraints.empty())
+		return false;
+
+	//checks if all the images referenced by the current rule are valid.
+	//if not, this rule will not match.
+	rule_imagelist::const_iterator image;
+	constraint_set::const_iterator constraint;
+	rule_image_variantlist::const_iterator variant;
+
+	for(constraint = rule.constraints.begin();
+			constraint != rule.constraints.end(); ++constraint) {
+		for(image = constraint->second.images.begin(); 
+				image != constraint->second.images.end();
+				++image) {
+
+			for(variant = image->variants.begin(); variant != image->variants.end(); ++variant) {
+				std::string s = variant->second.image_string;
+				s = s.substr(0, s.find_first_of(",:"));
+
+				if(!image::exists("terrain/" + s + ".png"))
+					return false;
+			}
+		}
+	}
+
+	return true;
+}
+
+bool terrain_builder::start_animation(building_rule &rule)
+{
+	rule_imagelist::iterator image;
+	constraint_set::iterator constraint;
+	rule_image_variantlist::iterator variant;
+
+	for(constraint = rule.constraints.begin();
+			constraint != rule.constraints.end(); ++constraint) {
+
+		for(image = constraint->second.images.begin(); 
+				image != constraint->second.images.end();
+				++image) {
+
+			for(variant = image->variants.begin(); variant != image->variants.end(); ++variant) {
+
+				locator_string_initializer initializer;
+
+				if(image->global_image) {
+					initializer = locator_string_initializer(constraint->second.loc);
+				}
+
+				animated<image::locator> th(variant->second.image_string,
+						initializer);
+
+				variant->second.image = th;
+				variant->second.image.start_animation(0, animated<image::locator>::INFINITE_CYCLES);
+				variant->second.image.update_current_frame();
+			}
+		}
+	}
+
+	return true;
+}
+
 terrain_builder::terrain_constraint terrain_builder::rotate(const terrain_builder::terrain_constraint &constraint, int angle)
 {
-	static struct { int ii; int ij; int ji; int jj; }  rotations[6] = 
+	static const struct { int ii; int ij; int ji; int jj; }  rotations[6] = 
 		{ {  1, 0, 0,  1 }, {  1,  1, -1, 0 }, { 0,  1, -1, -1 },
 		  { -1, 0, 0, -1 }, { -1, -1,  1, 0 }, { 0, -1,  1,  1 } };
 
+	// The following array of matrices is intended to rotate the (x,y)
+	// coordinates of a point in a wesnoth hex (and wesnoth hexes are not
+	// regular hexes :) ).
+	// The base matrix for a 1-step rotation with the wesnoth tile shape
+	// is:
+	//
+	// r = s^-1 * t * s
+	//
+	// with s = [[ 1   0         ]
+	//           [ 0   -sqrt(3)/2 ]]
+	//
+	// and t =  [[ -1/2       sqrt(3)/2 ]
+	//           [ -sqrt(3)/2  1/2        ]]
+	//
+	// With t being the rotation matrix (pi/3 rotation), and s a matrix
+	// that transforms the coordinates of the wesnoth hex to make them
+	// those of a regular hex.
+	//
+	// (demonstration left as an exercise for the reader)
+	//
+	// So we have 
+	//
+	// r = [[ 1/2  -3/4 ]
+	//      [ 1    1/2  ]]
+	//
+	// And the following array contains I(2), r, r^2, r^3, r^4, r^5 (with
+	// r^3 == -I(2)), which are the successive rotations.
+	static const struct { 
+		double xx;
+		double xy;
+		double yx; 
+		double yy;
+	} xyrotations[6] = {
+		{ 1.,         0.,  0., 1.    },
+		{ 1./2. , -3./4.,  1., 1./2. },
+		{ -1./2., -3./4.,   1, -1./2.},
+		{ -1.   ,     0.,  0., -1.   },
+		{ -1./2.,  3./4., -1., -1./2.},
+		{ 1./2. ,  3./4., -1., 1./2. },
+	};
+
 	assert(angle >= 0);
 
 	angle %= 6;	
@@ -125,12 +347,30 @@ terrain_builder::terrain_constraint terrain_builder::rotate(const terrain_builde
 	ret.loc.x = rj;
 	ret.loc.y = ri + (rj >= 0 ? rj/2 : (rj-1)/2);
 
+	for (rule_imagelist::iterator itor = ret.images.begin();
+			itor != ret.images.end(); ++itor) {
+
+		if (itor->position == rule_image::HORIZONTAL)
+			continue;
+
+		double vx, vy, rx, ry;
+
+		vx = double(itor->basex) - double(rule_image::TILEWIDTH)/2;
+		vy = double(itor->basey) - double(rule_image::TILEWIDTH)/2;
+
+		rx = xyrotations[angle].xx * vx + xyrotations[angle].xy * vy;
+		ry = xyrotations[angle].yx * vx + xyrotations[angle].yy * vy;
+
+		itor->basex = int(rx + rule_image::TILEWIDTH/2);
+		itor->basey = int(ry + rule_image::TILEWIDTH/2);
+	}
+
 	return ret;
 }
 
 void terrain_builder::replace_token(std::string &s, const std::string &token, const std::string &replacement)
 {
-	int pos;
+	size_t pos;
 	
 	if(token.empty()) {
 		std::cerr << "Error: empty token in replace_token\n";
@@ -141,14 +381,29 @@ void terrain_builder::replace_token(std::string &s, const std::string &token, co
 	}
 }
 
-void terrain_builder::replace_token(terrain_builder::imagelist &list, const std::string &token, const std::string &replacement)
+void terrain_builder::replace_token(terrain_builder::rule_image_variant &variant, const std::string &token, const std::string &replacement)
 {
-	for(imagelist::iterator itor = list.begin(); itor != list.end(); ++itor) {
-		replace_token(itor->second, token, replacement);
+	replace_token(variant.image_string, token, replacement);
+}
+
+void terrain_builder::replace_token(terrain_builder::rule_image &image, const std::string &token, const std::string &replacement)
+{
+	rule_image_variantlist::iterator itor;
+
+	for(itor = image.variants.begin(); itor != image.variants.end(); ++itor) {
 		replace_token(itor->second, token, replacement);
 	}
 }
 
+void terrain_builder::replace_token(terrain_builder::rule_imagelist &list, const std::string &token, const std::string &replacement)
+{
+	rule_imagelist::iterator itor;
+
+	for(itor = list.begin(); itor != list.end(); ++itor) {
+		replace_token(*itor, token, replacement);
+	}
+}
+
 void terrain_builder::replace_token(terrain_builder::building_rule &rule, const std::string &token, const std::string& replacement)
 {
 	constraint_set::iterator cons;
@@ -169,7 +424,7 @@ void terrain_builder::replace_token(terrain_builder::building_rule &rule, const
 		replace_token(cons->second.images, token, replacement);
 	}
 
-	replace_token(rule.images, token, replacement);
+	//replace_token(rule.images, token, replacement);
 }
 
 terrain_builder::building_rule terrain_builder::rotate_rule(const terrain_builder::building_rule &rule, 
@@ -180,7 +435,6 @@ terrain_builder::building_rule terrain_builder::rotate_rule(const terrain_builde
 		std::cerr << "Error: invalid rotations\n";
 		return ret;
 	}
-	ret.images = rule.images;
 	ret.location_constraints = rule.location_constraints;
 	ret.probability = rule.probability;
 	ret.precedence = rule.precedence;
@@ -224,34 +478,55 @@ terrain_builder::building_rule terrain_builder::rotate_rule(const terrain_builde
 	return ret;
 }
 
-terrain_builder::building_rule terrain_builder::rule_from_terrain_template(const terrain_builder::building_rule&tpl, const gamemap::TERRAIN terrain)
-{
-	terrain_builder::building_rule ret = tpl;
-
-	std::string ter(1, terrain);
-	constraint_set::iterator cons;
-	for(cons = ret.constraints.begin(); cons != ret.constraints.end(); ++cons) {
-		replace_token(cons->second.terrain_types, "@", ter);
-	}
-	replace_token(ret, "@T", map_.get_terrain_info(terrain).default_image());
-
-	return ret;
-}
-
-void terrain_builder::add_images_from_config(imagelist& images, const config &cfg)
+void terrain_builder::add_images_from_config(rule_imagelist& images, const config &cfg, bool global, int dx, int dy)
 {
 	const config::child_list& cimages = cfg.get_children("image");
 
-	for(config::child_list::const_iterator itor = cimages.begin(); itor != cimages.end(); ++itor) {
-		const int z_index = atoi((**itor)["z_index"].c_str());
-		const std::string &name = (**itor)["name"];
+
+	for(config::child_list::const_iterator img = cimages.begin(); img != cimages.end(); ++img) {
+
+		const std::string &name = (**img)["name"];
+
+		if((**img)["position"].empty() || 
+				(**img)["position"] == "horizontal") { 
+
+			const int layer = atoi((**img)["layer"].c_str());
+			images.push_back(rule_image(layer, global));
+
+		} else if((**img)["position"] == "vertical") {
+
+			std::vector<std::string> base = config::split((**img)["base"]);
+			int basex = 0, basey = 0;
+
+			if(base.size() >= 2) {
+				basex = atoi(base[0].c_str());
+				basey = atoi(base[1].c_str());
+			}
+			images.push_back(rule_image(basex - dx, basey - dy, global));
 			
-		images.insert(std::pair<int, std::string>(z_index, name));
-	}	
+		}
+
+		// Adds the main (default) variant of the image, if present
+		images.back().variants.insert(std::pair<std::string, rule_image_variant>("", rule_image_variant(name,"")));
+
+		// Adds the other variants of the image
+		const config::child_list& variants = (**img).get_children("variant");
+
+		for(config::child_list::const_iterator variant = variants.begin();
+				variant != variants.end(); ++variant) {
+			const std::string &name = (**variant)["name"];
+			const std::string &tod = (**variant)["tod"];
+
+			images.back().variants.insert(std::pair<std::string, rule_image_variant>(tod, rule_image_variant(name,tod)));
+
+		}
+	}
 }
 
-void terrain_builder::add_constraints(std::map<gamemap::location, terrain_builder::terrain_constraint> & constraints, 
-				      const gamemap::location& loc, const std::string& type)
+void terrain_builder::add_constraints(
+		terrain_builder::constraint_set& constraints, 
+		const gamemap::location& loc,
+		const std::string& type, const config& global_images)
 {
 	if(constraints.find(loc) == constraints.end()) {
 		//the terrain at the current location did not exist, so create it
@@ -260,24 +535,29 @@ void terrain_builder::add_constraints(std::map<gamemap::location, terrain_builde
 
 	if(!type.empty())
 		constraints[loc].terrain_types = type;			
+	
+	int x = loc.x * rule_image::TILEWIDTH / 2;
+	int y = loc.y * rule_image::TILEWIDTH + (loc.x % 2) * 
+		rule_image::TILEWIDTH / 2;
+	add_images_from_config(constraints[loc].images, global_images, true, x, y);
+
 }
 
 void terrain_builder::add_constraint_item(std::vector<std::string> &list, const config& cfg, const std::string &item)
 {
-	if(!cfg[item].empty())
-		list.push_back(cfg[item]);
-
-	const config::child_list& items = cfg.get_children(item);
-
-	for(config::child_list::const_iterator itor = items.begin(); itor != items.end(); ++itor) {
-		if(!(**itor)["name"].empty())
-			list.push_back((**itor)["name"]);
+	if(!cfg[item].empty()) {
+		std::vector<std::string> item_string = config::split(cfg[item]);
+		
+		for(std::vector<std::string>::const_iterator itor = item_string.begin();
+				itor != item_string.end(); ++itor) {
+			list.push_back(*itor);
+		}
 	}
 }
 
-void terrain_builder::add_constraints(terrain_builder::constraint_set &constraints, const gamemap::location& loc, const config& cfg)
+void terrain_builder::add_constraints(terrain_builder::constraint_set &constraints, const gamemap::location& loc, const config& cfg, const config& global_images)
 {
-	add_constraints(constraints, loc, cfg["type"]);
+	add_constraints(constraints, loc, cfg["type"], global_images);
 
 	terrain_constraint& constraint = constraints[loc];
 	
@@ -285,17 +565,16 @@ void terrain_builder::add_constraints(terrain_builder::constraint_set &constrain
 	add_constraint_item(constraint.has_flag, cfg, "has_flag");
 	add_constraint_item(constraint.no_flag, cfg, "no_flag");
 
-	add_images_from_config(constraint.images, cfg);
+	add_images_from_config(constraint.images, cfg, false);
 }
 
-void terrain_builder::parse_mapstring(const std::string &mapstring, struct building_rule &br,
-				      anchormap& anchors)
+void terrain_builder::parse_mapstring(const std::string &mapstring,
+		struct building_rule &br, anchormap& anchors,
+		const config& global_images)
 {
 	int lineno = 0;
 	int x = 0;
 
-	// std::cerr << "Loading map \"" << mapstring << "\"\n";
-	
 	const std::vector<std::string> &lines = config::split(mapstring, '\n', 0);
 	std::vector<std::string>::const_iterator line = lines.begin();
 	
@@ -307,17 +586,15 @@ void terrain_builder::parse_mapstring(const std::string &mapstring, struct build
 	if(line == lines.end())
 		return;
 	
-	//If the strings starts with a space, the first line is an odd line, else it is an even one
+	//If the strings starts with a space, the first line is an odd line,
+	//else it is an even one
 	if((*line)[0] == ' ')
 		lineno = 1;
 
-	//std::cerr << "--- Begin map ---\n";
-	
 	for(; line != lines.end(); ++line) {
-		//cuts each line into chunks of 4 characters, ignoring the 2 first ones if the line is odd
+		//cuts each line into chunks of 4 characters, ignoring the 2
+		//first ones if the line is odd
 
-		//std::cerr << "Line is " << *line << "\n";
-		
 		x = 0;
 		std::string::size_type lpos = 0;
 		if(lineno % 2) {
@@ -329,40 +606,46 @@ void terrain_builder::parse_mapstring(const std::string &mapstring, struct build
 			std::string types = line->substr(lpos, 4);
 			config::strip(types);
 			
-			//std::cerr << types << "/";
-			
-			//If there are numbers in the types string, consider it is an anchor
+			//If there are numbers in the types string, consider it
+			//is an anchor
 			if(types[0] == '.') {
-				// Dots are simple placeholders, which do not represent actual terrains.
+				// Dots are simple placeholders, which do not
+				// represent actual terrains.
 			} else if(types.find_first_of("0123456789") != std::string::npos) {
 				int anchor = atoi(types.c_str());
 				anchors.insert(std::pair<int, gamemap::location>(anchor, gamemap::location(x, lineno / 2)));
 			} else {
 				const gamemap::location loc(x, lineno / 2);
-				add_constraints(br.constraints, loc, types);
+				add_constraints(br.constraints, loc, types, global_images);
 			}
 			lpos += 4;
 			x += 2;
 		}
-		//std::cerr << "\n";
 		lineno++;
 	}
-	//std::cerr << "--- End map ---\n";
+}
+
+void terrain_builder::add_rule(building_ruleset& rules, building_rule &rule)
+{
+	if(rule_valid(rule)) {
+		start_animation(rule);
+		rules.insert(std::pair<int, building_rule>(rule.precedence, rule));
+	}
 
 }
 
-void terrain_builder::add_rotated_rules(building_ruleset& rules, const building_rule& tpl, const std::string &rotations)
+void terrain_builder::add_rotated_rules(building_ruleset& rules, building_rule& tpl, const std::string &rotations)
 {
-
 	if(rotations.empty()) {
 		// Adds the parsed built terrain to the list
-		building_rules_.insert(std::pair<int, building_rule>(tpl.precedence, tpl));
+
+		add_rule(rules, tpl);
 	} else {
 		const std::vector<std::string>& rot = config::split(rotations, ',');
 			
-		for(int angle = 0; angle < rot.size(); angle++) {
-			building_rules_.insert(std::pair<int, building_rule>(tpl.precedence,
-						rotate_rule(tpl, angle, rot)));
+		for(size_t angle = 0; angle < rot.size(); angle++) {
+			building_rule rule = rotate_rule(tpl, angle, rot);
+			add_rule(rules, rule);
 		}
 	}
 }
@@ -377,7 +660,7 @@ void terrain_builder::parse_config(const config &cfg)
 	for(config::child_list::const_iterator br = brs.begin(); br != brs.end(); ++br) {
 		building_rule pbr; // Parsed Building rule
 
-		add_images_from_config(pbr.images, **br);
+		// add_images_from_config(pbr.images, **br);
 
 		if(!((**br)["x"].empty() || (**br)["y"].empty()))
 			pbr.location_constraints = gamemap::location(atoi((**br)["x"].c_str()), atoi((**br)["y"].c_str()));
@@ -389,7 +672,7 @@ void terrain_builder::parse_config(const config &cfg)
 		anchormap anchors;
 		
 		// Parse the map= , if there is one (and fill the anchors list)
-		parse_mapstring((**br)["map"], pbr, anchors);
+		parse_mapstring((**br)["map"], pbr, anchors, **br);
 
 		// Parses the terrain constraints (TCs)
 		config::child_list tcs((*br)->get_children("tile"));
@@ -404,8 +687,15 @@ void terrain_builder::parse_config(const config &cfg)
 			if((**tc)["y"].size()) {
 				loc.y = atoi((**tc)["y"].c_str());
 			} 
+			if(!(**tc)["loc"].empty()) {
+				std::vector<std::string> sloc = config::split((**tc)["pos"]);
+				if(sloc.size() == 2) {
+					loc.x = atoi(sloc[0].c_str());
+					loc.y = atoi(sloc[1].c_str());
+				}
+			}
 			if(loc.valid()) {
-				add_constraints(pbr.constraints, loc, **tc);
+				add_constraints(pbr.constraints, loc, **tc, **br);
 			}
 			if((**tc)["pos"].size()) {
 				int pos = atoi((**tc)["pos"].c_str());
@@ -419,7 +709,7 @@ void terrain_builder::parse_config(const config &cfg)
 				
 				for(; range.first != range.second; range.first++) {
 					loc = range.first->second;
-					add_constraints(pbr.constraints, loc, **tc);
+					add_constraints(pbr.constraints, loc, **tc, **br);
 				}					
 			}
 		}
@@ -444,18 +734,8 @@ void terrain_builder::parse_config(const config &cfg)
 
 		// Handles rotations
 		const std::string rotations = (**br)["rotations"];
-		const std::string terrains = (**br)["terrains"];
 
-		if(terrains.empty()) {
-			add_rotated_rules(building_rules_, pbr, rotations);
-		} else {
-			for(std::string::const_iterator terrain = terrains.begin();
-					terrain != terrains.end(); ++terrain) {
-
-				const building_rule r = rule_from_terrain_template(pbr, *terrain);
-				add_rotated_rules(building_rules_, r, rotations);
-			}
-		}
+		add_rotated_rules(building_rules_, pbr, rotations);
 
 	}
 
@@ -486,14 +766,8 @@ void terrain_builder::parse_config(const config &cfg)
 
 }
 
-namespace {
-int matches = 0;
-}
-
 bool terrain_builder::terrain_matches(gamemap::TERRAIN letter, const std::string &terrains)
 {
-
-	bool res = false;
 	bool negative = false;
 	std::string::const_iterator itor;
 
@@ -517,12 +791,10 @@ bool terrain_builder::terrain_matches(gamemap::TERRAIN letter, const std::string
 
 bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule, const gamemap::location &loc, int rule_index, bool check_loc)
 {
-	matches++;
-
 	if(rule.location_constraints.valid() && rule.location_constraints != loc)
 		return false;
-	
-		
+
+
 	if(check_loc) {
 		for(constraint_set::const_iterator cons = rule.constraints.begin();
 				cons != rule.constraints.end(); ++cons) {
@@ -533,8 +805,6 @@ bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule, c
 			if(!tile_map_.on_map(tloc))
 				return false;
 
-			const tile& btile = tile_map_[tloc];
-
 			if(!terrain_matches(map_.get_terrain(tloc), cons->second.terrain_types))
 				return false;
 		}
@@ -547,13 +817,13 @@ bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule, c
 		unsigned int random = a*b*c + a*b + b*c + a*c + a + b + c;
 
 		random %= 100;
-		
+
 		if(random > rule.probability)
 			return false;
 	}
 
 	for(constraint_set::const_iterator cons = rule.constraints.begin();
-	    cons != rule.constraints.end(); ++cons) {
+			cons != rule.constraints.end(); ++cons) {
 
 		const gamemap::location tloc = loc + cons->second.loc;
 		if(!tile_map_.on_map(tloc))
@@ -562,13 +832,13 @@ bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule, c
 
 		std::vector<std::string>::const_iterator itor;
 		for(itor = cons->second.no_flag.begin(); itor != cons->second.no_flag.end(); ++itor) {
-			
+
 			//If a flag listed in "no_flag" is present, the rule does not match
 			if(btile.flags.find(*itor) != btile.flags.end())
 				return false;
 		}
 		for(itor = cons->second.has_flag.begin(); itor != cons->second.has_flag.end(); ++itor) {
-			
+
 			//If a flag listed in "has_flag" is not present, this rule does not match
 			if(btile.flags.find(*itor) == btile.flags.end())
 				return false;
@@ -581,38 +851,32 @@ bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule, c
 void terrain_builder::apply_rule(const terrain_builder::building_rule &rule, const gamemap::location &loc)
 {
 	for(constraint_set::const_iterator constraint = rule.constraints.begin();
-	    constraint != rule.constraints.end(); ++constraint) {
+			constraint != rule.constraints.end(); ++constraint) {
 
+		rule_imagelist::const_iterator img;
 		const gamemap::location tloc = loc + constraint->second.loc;
 		if(!tile_map_.on_map(tloc))
 			return;
 
 		tile& btile = tile_map_[tloc];
 
-		std::multimap<int, std::string>::const_iterator img;
-		
-		for(img = rule.images.begin(); img != rule.images.end(); ++img) {
-			image::locator th(img->second, constraint->second.loc);
-			if(img->first < 0) {
-				btile.images_background.push_back(th);
-			} else {
-				btile.images_foreground.push_back(th);
-			}
-
-		}
+		//std::multimap<int, std::string>::const_iterator img;
 
 		for(img = constraint->second.images.begin(); img != constraint->second.images.end(); ++img) {
-			image::locator th(img->second);
-			if(img->first < 0) {
-				btile.images_background.push_back(th);
-			} else {
-				btile.images_foreground.push_back(th);
+			//animated<image::locator> th(img->second, locator_string_initializer());
+
+			if(img->position == rule_image::HORIZONTAL) {
+				btile.horizontal_images.insert(std::pair<int, const rule_image*>(
+							img->layer, &*img));
+			} else if(img->position == rule_image::VERTICAL) {
+				btile.vertical_images.insert(std::pair<int, const rule_image*>(
+						img->basey - rule_image::UNITPOS, &*img));
 			}
 		}
 
 		// Sets flags
 		for(std::vector<std::string>::const_iterator itor = constraint->second.set_flag.begin(); 
-		    itor != constraint->second.set_flag.end(); itor++) {
+				itor != constraint->second.set_flag.end(); itor++) {
 			btile.flags.insert(*itor);
 		}
 
@@ -689,7 +953,6 @@ int terrain_builder::get_constraint_size(const building_rule& rule, const terrai
 void terrain_builder::build_terrains()
 {
 	log_scope("terrain_builder::build_terrains");
-	matches = 0;
 
 	//builds the terrain_by_type_ cache
 	for(int x = -1; x <= map_.x(); ++x) {
@@ -723,36 +986,14 @@ void terrain_builder::build_terrains()
 	building_ruleset::const_iterator rule;
 	
 	for(rule = building_rules_.begin(); rule != building_rules_.end(); ++rule) {
-		//if the rule has no constraints, it is invalid
-		if(rule->second.constraints.empty())
-			continue;
 
-		//checks if all the images referenced by the current rule are valid.
-		//if not, this rule will not match.
-		bool absent_image = false;
-		imagelist::const_iterator image;
+		if (rule->second.location_constraints.valid()) {
+			apply_rule(rule->second, rule->second.location_constraints);
+			continue;
+		}
+
 		constraint_set::const_iterator constraint;
 
-		for(image = rule->second.images.begin(); 
-				!absent_image && (image != rule->second.images.end()); ++image) {
-
-			if(!image::exists("terrain/" + image->second + ".png"))
-				absent_image = true;
-		}
-
-		for(constraint = rule->second.constraints.begin();
-		    constraint != rule->second.constraints.end(); ++constraint) {
-			for(image = constraint->second.images.begin(); 
-					!absent_image && (image != constraint->second.images.end());
-					++image) {
-				if(!image::exists("terrain/" + image->second + ".png"))
-					absent_image = true;
-			}
-		}
-
-		if(absent_image)
-			continue;
-
 		//find the constraint that contains the less terrain of all terrain rules.
 		constraint_set::const_iterator smallest_constraint;
 		constraint_set::const_iterator constraint_most_adjacents;
@@ -846,6 +1087,4 @@ void terrain_builder::build_terrains()
 
 		rule_index++;
 	}
-
-	std::cerr << "Matches = " << matches << "\n";
 }
