@@ -1,4 +1,4 @@
-/* $Id: config.cpp,v 1.91 2004/06/20 17:13:58 gruikya Exp $ */
+/* $Id: config.cpp,v 1.109 2004/09/12 21:55:19 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -10,6 +10,12 @@
 
    See the COPYING file for more details.
 */
+
+//disable the very annoying VC++ warning 4786
+#ifdef WIN32
+#pragma warning(disable:4786)
+#endif
+
 #include <algorithm>
 #include <cassert>
 #include <cctype>
@@ -23,9 +29,11 @@
 #include "filesystem.hpp"
 #include "game_config.hpp"
 #include "game_events.hpp"
+#include "gettext.hpp"
 #include "log.hpp"
 #include "scoped_resource.hpp"
 #include "util.hpp"
+#include "wesconfig.h"
 
 bool operator<(const line_source& a, const line_source& b)
 {
@@ -34,6 +42,8 @@ bool operator<(const line_source& a, const line_source& b)
 
 namespace {
 
+const int max_recursion_levels = 100;
+
 bool isnewline(char c)
 {
 	return c == '\r' || c == '\n';
@@ -64,13 +74,6 @@ line_source get_line_source(const std::vector<line_source>& line_src, int line)
 	return res;
 }
 
-#if 0
-struct close_FILE
-{
-	void operator()(FILE* f) const { if(f != NULL) { fclose(f); } }
-};
-#endif
-
 void read_file_internal(const std::string& fname, std::string& res)
 {
 	const int size = file_size(fname);
@@ -297,15 +300,37 @@ void internal_preprocess_data(const std::string& data,
 				internal_preprocess_data(str,defines_map,depth,res,NULL,line,fname,srcline);
 			} else if(depth < 20) {
 				std::string prefix;
-				std::string fname = newfilename;
+				std::string nfname;
 
-				//if the filename begins with a '~', then look in the user's data directory
-				if(newfilename != "" && fname[0] == '~') {
-					prefix = get_user_data_dir() + "/";
-					fname.erase(fname.begin(),fname.begin()+1);
+				//if the filename begins with a '~', then look
+				//in the user's data directory. If the filename begins with
+				//a '@' then we look in the user's data directory,
+				//but default to the standard data directory if it's not found
+				//there.
+				if(newfilename != "" && (newfilename[0] == '~' || newfilename[0] == '@')) {
+					nfname = newfilename;
+					nfname.erase(nfname.begin(),nfname.begin()+1);
+					nfname = get_user_data_dir() + "/data/" + nfname;
+
+					std::cerr << "got relative name '" << newfilename << "' -> '" << nfname << "'\n";
+
+					if(newfilename[0] == '@' && file_exists(nfname) == false && is_directory(nfname) == false) {
+						nfname = "data/" + newfilename.substr(1);
+					}
+				} else if(newfilename.size() >= 2 && newfilename[0] == '.' &&
+						newfilename[1] == '/' ) {
+					//if the filename begins with a "./", then look
+					//in the same directory as the file currrently
+					//being preprocessed
+					nfname = newfilename;
+					nfname.erase(nfname.begin(),nfname.begin()+2);
+					nfname = directory_name(fname) + nfname;
+					
+				} else {
+					nfname = "data/" + newfilename;
 				}
 
-				internal_preprocess_file(prefix + "data/" + fname,
+				internal_preprocess_file(nfname,
 				                       defines_map, depth+1,res,
 				                       lines_src,line);
 			} else {
@@ -530,6 +555,8 @@ void config::read(const std::string& data,
 	std::stack<int> element_locs;
 	std::stack<config*> elements;
 	std::stack<std::map<std::string,config*> > last_element; //allows [+element] syntax
+	std::stack<std::string> textdomains;
+	std::string current_textdomain = PACKAGE;
 	elements.push(this);
 	element_names.push("");
 	element_locs.push(0);
@@ -540,7 +567,7 @@ void config::read(const std::string& data,
 	std::string var;
 	std::string value;
 
-	bool in_quotes = false, has_quotes = false, in_comment = false, escape_next = false;
+	bool in_quotes = false, has_quotes = false, in_comment = false, escape_next = false, translatable = false;;
 
 	int line = 0;
 
@@ -612,6 +639,11 @@ void config::read(const std::string& data,
 
 						last_element.top()[name] = element;
 
+						if(element->values.count("textdomain") != 0){
+							current_textdomain = textdomains.top();
+							textdomains.pop();
+						}
+
 						state = IN_ELEMENT;
 
 						break;
@@ -690,9 +722,19 @@ void config::read(const std::string& data,
 				} else if(c == '"') {
 					in_quotes = !in_quotes;
 					has_quotes = true;
+
+					//if we have an underscore outside of quotes in front, then
+					//we strip it away, since it simply indicates that this value is translatable.
+					if(value.empty() == false && std::count(value.begin(),value.end(),'_') == 1) {
+						std::string val = value;
+						if(strip(val) == "_") {
+							value = "";
+							translatable = true;
+						}
+					}
 				} else if(c == '\n' && !in_quotes) {
 
-					//see if this is a CVS list=CVS list style assignment (e.g. x,y=5,8)
+					//see if this is a CSV list=CSV list style assignment (e.g. x,y=5,8)
 					std::vector<std::string> vars, values;
 					if(std::count(var.begin(),var.end(),',') > 0) {
 						vars = config::split(var);
@@ -700,6 +742,13 @@ void config::read(const std::string& data,
 					} else {
 						vars.push_back(var);
 						values.push_back(value);
+						if (var == "textdomain") {
+							textdomains.push(current_textdomain);
+							current_textdomain = value;
+							bindtextdomain(current_textdomain.c_str(),
+								       get_intl_dir().c_str());
+							bind_textdomain_codeset (current_textdomain.c_str(), "UTF-8");
+						}
 					}
 
 					//iterate over the names and values, assigning each to its corresponding
@@ -708,8 +757,7 @@ void config::read(const std::string& data,
 					//all the last values get concatenated onto the last name
 					if(vars.empty() == false) {
 						for(size_t n = 0; n != maximum<size_t>(vars.size(),values.size()); ++n) {
-							const std::string& varname = n < vars.size() ? vars[n] : vars.back();
-							std::string value = "";
+							std::string value;
 							if(n < values.size()) {
 								value = values[n];
 							} else if(values.empty() == false) {
@@ -720,6 +768,11 @@ void config::read(const std::string& data,
 								strip(value);
 							}
 
+							if(translatable) {
+								value = dsgettext(current_textdomain.c_str(),
+										  value.c_str());
+							}
+
 							if(n < vars.size()) {
 								elements.top()->values[vars[n]] = value;
 							} else {
@@ -733,6 +786,7 @@ void config::read(const std::string& data,
 					value = "";
 					has_quotes = false;
 					escape_next = false;
+					translatable = false;
 				} else if(in_quotes || !has_quotes) {
 					push_back(value, c);
 				}
@@ -795,6 +849,9 @@ size_t config::write_size(size_t tab) const
 
 std::string::iterator config::write_internal(std::string::iterator out, size_t tab) const
 {
+	if(tab > max_recursion_levels)
+		return out;
+
 	for(std::map<std::string,std::string>::const_iterator i = values.begin();
 					i != values.end(); ++i) {
 		if(i->second.empty() == false) {
@@ -878,6 +935,7 @@ namespace {
 	                   compress_schema_item = 2, compress_literal_word = 3,
 	                   compress_first_word = 4, compress_end_words = 256;
 	const size_t compress_max_words = compress_end_words - compress_first_word;
+	const size_t max_schema_item_length = 20;
 
 	void compress_output_literal_word(const std::string& word, std::vector<char>& output)
 	{
@@ -888,6 +946,10 @@ namespace {
 
 	compression_schema::word_char_map::const_iterator add_word_to_schema(const std::string& word, compression_schema& schema)
 	{
+		if(word.size() > max_schema_item_length) {
+			throw config::error("Schema item is too long");
+		}
+
 		unsigned int c = compress_first_word + schema.word_to_char.size();
 
 		schema.char_to_word.insert(std::pair<unsigned int,std::string>(c,word));
@@ -896,6 +958,10 @@ namespace {
 
 	compression_schema::word_char_map::const_iterator get_word_in_schema(const std::string& word, compression_schema& schema, std::vector<char>& output)
 	{
+		if(word.size() > max_schema_item_length) {
+			return schema.word_to_char.end();
+		}
+
 		//see if this word is already in the schema
 		const compression_schema::word_char_map::const_iterator w = schema.word_to_char.find(word);
 		if(w != schema.word_to_char.end()) {
@@ -941,8 +1007,11 @@ namespace {
 	}
 }
 
-void config::write_compressed_internal(compression_schema& schema, std::vector<char>& res) const
+void config::write_compressed_internal(compression_schema& schema, std::vector<char>& res, int level) const
 {
+	if(level > max_recursion_levels)
+		throw config::error("Too many recursion levels in compressed config write\n");
+
 	for(std::map<std::string,std::string>::const_iterator i = values.begin();
 					i != values.end(); ++i) {
 		if(i->second.empty() == false) {
@@ -962,7 +1031,7 @@ void config::write_compressed_internal(compression_schema& schema, std::vector<c
 
 		res.push_back(compress_open_element);
 		compress_emit_word(name,schema,res);
-		cfg.write_compressed_internal(schema,res);
+		cfg.write_compressed_internal(schema,res, level+1);
 		res.push_back(compress_close_element);
 	}
 }
@@ -970,15 +1039,18 @@ void config::write_compressed_internal(compression_schema& schema, std::vector<c
 std::string config::write_compressed(compression_schema& schema) const
 {
 	std::vector<char> res;
-	write_compressed_internal(schema,res);
+	write_compressed_internal(schema,res,0);
 	std::string s;
 	s.resize(res.size());
 	std::copy(res.begin(),res.end(),s.begin());
 	return s;
 }
 
-std::string::const_iterator config::read_compressed_internal(std::string::const_iterator i1, std::string::const_iterator i2, compression_schema& schema)
+std::string::const_iterator config::read_compressed_internal(std::string::const_iterator i1, std::string::const_iterator i2, compression_schema& schema, int level)
 {
+	if(level >= max_recursion_levels)
+		throw config::error("Too many recursion levels in compressed config read\n");
+	
 	bool in_open_element = false;
 	while(i1 != i2) {
 		switch(*i1) {
@@ -1016,7 +1088,7 @@ std::string::const_iterator config::read_compressed_internal(std::string::const_
 			if(in_open_element) {
 				in_open_element = false;
 				config& cfg = add_child(word);
-				i1 = cfg.read_compressed_internal(i1+1,i2,schema);
+				i1 = cfg.read_compressed_internal(i1+1,i2,schema,level+1);
 			} else {
 				//we have a name/value pair, the value is always a literal string
 				std::string value;
@@ -1025,8 +1097,10 @@ std::string::const_iterator config::read_compressed_internal(std::string::const_
 			}
 		}
 
-		}
+		} //end switch
 
+		if(i1 == i2)
+			return i2;
 		++i1;
 	}
 
@@ -1036,7 +1110,7 @@ std::string::const_iterator config::read_compressed_internal(std::string::const_
 void config::read_compressed(const std::string& data, compression_schema& schema)
 {
 	clear();
-	read_compressed_internal(data.begin(),data.end(),schema);
+	read_compressed_internal(data.begin(),data.end(),schema,0);
 }
 
 config::child_itors config::child_range(const std::string& key)
@@ -1086,7 +1160,12 @@ config* config::child(const std::string& key)
 
 const config* config::child(const std::string& key) const
 {
-	return const_cast<config*>(this)->child(key);
+	const child_map::const_iterator i = children.find(key);
+	if(i != children.end() && i->second.empty() == false) {
+		return i->second.front();
+	} else {
+		return NULL;
+	}
 }
 
 config& config::add_child(const std::string& key)
