@@ -1,4 +1,4 @@
-/* $Id: config.hpp,v 1.43 2004/06/10 00:35:43 Sirp Exp $ */
+/* $Id: config.hpp,v 1.46 2004/08/23 06:40:59 gakusho Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,7 +14,7 @@
 #define CONFIG_HPP_INCLUDED
 
 //disable the very annoying VC++ warning 4786
-#ifdef _WIN32
+#ifdef WIN32
 #pragma warning(disable:4786)
 #endif
 
@@ -96,8 +96,9 @@ struct compression_schema
 
 //a config object defines a single node in a WML file, with access to
 //child nodes.
-struct config
+class config
 {
+public:
 	//create an empty node.
 	config() {}
 
@@ -227,8 +228,8 @@ struct config
 private:
 	size_t write_size(size_t tab=0) const;
 	std::string::iterator write_internal(std::string::iterator out, size_t tab=0) const;
-	std::string::const_iterator read_compressed_internal(std::string::const_iterator i1, std::string::const_iterator i2, compression_schema& schema);
-	void write_compressed_internal(compression_schema& schema, std::vector<char>& res) const;
+	std::string::const_iterator read_compressed_internal(std::string::const_iterator i1, std::string::const_iterator i2, compression_schema& schema, int level);
+	void write_compressed_internal(compression_schema& schema, std::vector<char>& res, int level) const;
 
 	//a list of all children of this node.
 	child_map children;
