@@ -1,4 +1,4 @@
-/* $Id: dialogs.cpp,v 1.40 2004/06/17 18:48:34 Sirp Exp $ */
+/* $Id: dialogs.cpp,v 1.58 2004/09/25 19:46:33 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -15,6 +15,7 @@
 #include "events.hpp"
 #include "font.hpp"
 #include "game_config.hpp"
+#include "help.hpp"
 #include "language.hpp"
 #include "log.hpp"
 #include "playturn.hpp"
@@ -45,38 +46,52 @@ void advance_unit(const game_data& info,
 	if(u == units.end() || u->second.advances() == false)
 		return;
 
+	std::cerr << "advance_unit: " << u->second.type().name() << "\n";
+
 	const std::vector<std::string>& options = u->second.type().advances_to();
 
 	std::vector<std::string> lang_options;
 
 	std::vector<unit> sample_units;
-	for(std::vector<std::string>::const_iterator op = options.begin();
-	    op != options.end(); ++op) {
+	for(std::vector<std::string>::const_iterator op = options.begin(); op != options.end(); ++op) {
 		sample_units.push_back(::get_advanced_unit(info,units,loc,*op));
 		const unit_type& type = sample_units.back().type();
 		lang_options.push_back("&" + type.image() + "," + type.language_name());
 	}
 
+	const config::child_list& mod_options = u->second.get_modification_advances();
+
+	for(config::child_list::const_iterator mod = mod_options.begin(); mod != mod_options.end(); ++mod) {
+		sample_units.push_back(::get_advanced_unit(info,units,loc,u->second.type().name()));
+		sample_units.back().add_modification("advance",**mod);
+		const unit_type& type = sample_units.back().type();
+		// lang_options.push_back("&" + type.image() + "," + translate_string_default("advance_" + (**mod)["id"],(**mod)["description"]));
+		lang_options.push_back("&" + type.image() + "," + (**mod)["description"]);
+	}
+
+	std::cerr << "options: " << options.size() << "\n";
+
 	int res = 0;
 
-	if(options.empty()) {
+	if(lang_options.empty()) {
 		return;
 	} else if(random_choice) {
-		res = rand()%options.size();
-	} else if(options.size() > 1) {
+		res = rand()%lang_options.size();
+	} else if(lang_options.size() > 1) {
 
 		const events::event_context dialog_events_context;
 		unit_preview_pane unit_preview(gui,&map,sample_units);
 		std::vector<gui::preview_pane*> preview_panes;
 		preview_panes.push_back(&unit_preview);
 
-		res = gui::show_dialog(gui,NULL,string_table["advance_unit_heading"],
-		                       string_table["advance_unit_message"],
+		res = gui::show_dialog(gui,NULL,_("Advance Unit"),
+		                       _("What should our victorious unit become?"),
 		                       gui::OK_ONLY, &lang_options, &preview_panes);
 	}
 
 	recorder.choose_option(res);
 
+	std::cerr << "animating advancement...\n";
 	animate_unit_advancement(info,units,loc,gui,size_t(res));
 }
 
@@ -90,7 +105,9 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 	}
 
 	const std::vector<std::string>& options = u->second.type().advances_to();
-	if(choice >= options.size()) {
+	const config::child_list& mod_options = u->second.get_modification_advances();
+
+	if(choice >= options.size() + mod_options.size()) {
 		return false;
 	}
 	
@@ -107,7 +124,13 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 		}
 	}
 
-	::advance_unit(info,units,loc,options[choice]);
+	const std::string& chosen_unit = choice < options.size() ? options[choice] : u->second.type().name();
+	::advance_unit(info,units,loc,chosen_unit);
+
+	u = units.find(loc);
+	if(u != units.end() && choice >= options.size()) {
+		u->second.add_modification("advance",*mod_options[choice - options.size()]);
+	}
 
 	gui.invalidate_unit();
 
@@ -131,12 +154,9 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 
 void show_objectives(display& disp, config& level_info)
 {
-	static const std::string no_objectives(string_table["no_objectives"]);
-	const std::string& id = level_info["id"];
-	const std::string& lang_name = string_table[id];
-	const std::string& name = lang_name.empty() ? level_info["name"] :
-	                                              lang_name;
-	const std::string& lang_objectives = string_table[id + "_objectives"];
+	static const std::string no_objectives(_("No objectives available"));
+	const std::string& name = level_info["name"];
+	const std::string& lang_objectives = level_info["objectives"];
 
 	const std::string& objectives = lang_objectives.empty() ?
 	        level_info["objectives"] : lang_objectives;
@@ -151,10 +171,10 @@ int get_save_name(display & disp,const std::string& caption, const std::string&
     int overwrite=0;
     int res=0;
     do {
-        res = gui::show_dialog(disp,NULL,"",caption,dialog_type,NULL,NULL,message,name);
+        res = gui::show_dialog(disp,NULL,_("Save Game"),caption,dialog_type,NULL,NULL,message,name);
             if (res == 0 && save_game_exists(*name))
-                overwrite = gui::show_dialog(disp,NULL,"",
-                    string_table["save_confirm_overwrite"],gui::YES_NO);
+                overwrite = gui::show_dialog(disp,NULL,_("Overwrite?"),
+                    _("Save already exists. Do you want to overwrite it ?"),gui::YES_NO);
         else overwrite = 0;
     } while ((res==0)&&(overwrite!=0));
 	return res;
@@ -183,9 +203,9 @@ gui::dialog_button_action::RESULT delete_save::button_pressed(int menu_selection
 		//see if we should ask the user for deletion confirmation
 		if(preferences::ask_delete_saves()) {
 			std::vector<gui::check_item> options;
-			options.push_back(gui::check_item(string_table["dont_ask_again"],false));
+			options.push_back(gui::check_item(_("Don't ask me again!"),false));
 
-			const int res = gui::show_dialog(disp_,NULL,"",string_table["really_delete_save"],gui::YES_NO,
+			const int res = gui::show_dialog(disp_,NULL,"",_("Do you really want to delete this game?"),gui::YES_NO,
 			                                 NULL,NULL,"",NULL,NULL,&options);
 
 			//see if the user doesn't want to be asked this again
@@ -243,7 +263,7 @@ private:
 	const std::vector<save_info>* info_;
 	const std::vector<config*>* summaries_;
 	int index_;
-	std::map<std::string,shared_sdl_surface> map_cache_;
+	std::map<std::string,surface> map_cache_;
 };
 
 void save_preview_pane::draw()
@@ -260,11 +280,9 @@ void save_preview_pane::draw()
 
 	set_dirty(false);
 
-	static int n = 0;
-
 	const config& summary = *(*summaries_)[index_];
 
-	SDL_Surface* const screen = disp().video().getSurface();
+	surface const screen = disp().video().getSurface();
 
 	const SDL_Rect area = { location().x+save_preview_border, location().y+save_preview_border,
 	                        location().w-save_preview_border*2, location().h-save_preview_border*2 };
@@ -275,7 +293,7 @@ void save_preview_pane::draw()
 
 	const game_data::unit_type_map::const_iterator leader = data_->unit_types.find(summary["leader"]);
 	if(leader != data_->unit_types.end()) {
-		const scoped_sdl_surface image(image::get_image(leader->second.image(),image::UNSCALED));
+		const surface image(image::get_image(leader->second.image(),image::UNSCALED));
 		if(image != NULL) {
 			SDL_Rect image_rect = {area.x,area.y,image->w,image->h};
 			ypos += image_rect.h + save_preview_border;
@@ -300,10 +318,10 @@ void save_preview_pane::draw()
 		}
 	}
 
-	SDL_Surface* map_surf = NULL;
+	surface map_surf(NULL);
 
 	if(map_data.empty() == false) {
-		const std::map<std::string,shared_sdl_surface>::const_iterator itor = map_cache_.find(map_data);
+		const std::map<std::string,surface>::const_iterator itor = map_cache_.find(map_data);
 		if(itor != map_cache_.end()) {
 			map_surf = itor->second;
 		} else if(map_ != NULL) {
@@ -312,7 +330,7 @@ void save_preview_pane::draw()
 
 				map_surf = image::getMinimap(100,100,*map_,0,NULL);
 				if(map_surf != NULL) {
-					map_cache_.insert(std::pair<std::string,shared_sdl_surface>(map_data,shared_sdl_surface(map_surf)));
+					map_cache_.insert(std::pair<std::string,surface>(map_data,surface(map_surf)));
 				}
 			} catch(gamemap::incorrect_format_exception&) {
 			}
@@ -326,7 +344,7 @@ void save_preview_pane::draw()
 	}
 
 	char time_buf[256];
-	const size_t res = strftime(time_buf,sizeof(time_buf),string_table["date_format"].c_str(),localtime(&((*info_)[index_].time_modified)));
+	const size_t res = strftime(time_buf,sizeof(time_buf),_("%a %b %d %H:%M %Y"),localtime(&((*info_)[index_].time_modified)));
 	if(res == 0) {
 		time_buf[0] = 0;
 	}
@@ -337,36 +355,36 @@ void save_preview_pane::draw()
 	std::string name = (*info_)[index_].name;
 	str << font::BOLD_TEXT << config::escape(name) << "\n" << time_buf;
 
+	const std::string& campaign_type = summary["campaign_type"];
 	if(summary["corrupt"] == "yes") {
-		str << "\n" << string_table["save_invalid"];
-	} else if(summary["campaign_type"] != "") {
+		str << "\n" << _("#(Invalid)");
+	} else if (!campaign_type.empty()) {
 		str << "\n";
 			
-		const std::string& campaign_type = summary["campaign_type"];
 		if(campaign_type == "scenario") {
-			str << translate_string("campaign_button");
+			str << _("Campaign");
 		} else if(campaign_type == "multiplayer") {
-			str << translate_string("multiplayer_button");
+			str << _("Multiplayer");
 		} else if(campaign_type == "tutorial") {
-			str << translate_string("tutorial_button");
+			str << _("Tutorial");
 		} else {
-			str << translate_string(campaign_type);
+			str << campaign_type;
 		}
 
 		str << "\n";
 			
 		if(summary["snapshot"] == "no" && summary["replay"] == "yes") {
-			str << translate_string("replay");
-		} else if(summary["turn"] != "") {
-			str << translate_string("turn") << " " << summary["turn"];
+			str << _("replay");
+		} else if (!summary["turn"].empty()) {
+			str << _("Turn") << " " << summary["turn"];
 		} else {
-			str << string_table["scenario_start"];
+			str << _("Scenario Start");
 		}
 
-		str << "\n" << translate_string("difficulty") << ": " << translate_string(summary["difficulty"]);
+		str << "\n" << _("Difficulty") << ": " << string_table[summary["difficulty"]];
 	}
 
-	font::draw_text(&disp(),area,12,font::NORMAL_COLOUR,str.str(),area.x,ypos,NULL,true);
+	font::draw_text(&disp(),area,font::SIZE_SMALL,font::NORMAL_COLOUR,str.str(),area.x,ypos,NULL,true);
 }
 
 } //end anon namespace
@@ -377,8 +395,9 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 
 	if(games.empty()) {
 		gui::show_dialog(disp,NULL,
-		                 string_table["no_saves_heading"],
-						 string_table["no_saves_message"],
+		                 _("No Saved Games"),
+						 _("There are no saved games to load.\n\
+(Games are saved automatically when you complete a scenario)"),
 		                 gui::OK_ONLY);
 		return "";
 	}
@@ -396,7 +415,7 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	}
 
 	delete_save save_deleter(disp,games,summaries);
-	gui::dialog_button delete_button(&save_deleter,string_table["delete_save"]);
+	gui::dialog_button delete_button(&save_deleter,_("Delete Save"));
 	std::vector<gui::dialog_button> buttons;
 	buttons.push_back(delete_button);
 
@@ -411,17 +430,18 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 		} else if(preferences::cache_saves() == preferences::CACHE_SAVES_ALWAYS) {
 			generate_summaries = true;
 		} else {
-			std::string caption = "import_saves_caption", message = "import_old_saves";
+			char* caption = N_("Import Saved Games");
+			char* message = N_("Your saves directory contains some files from an old version of Battle for Wesnoth. Would you like to update these to the latest version? This may take some time.");
 
 			//if there are already some cached games, then we assume that the user is importing new
 			//games, and it's not a total import, so tailor the message accordingly
 			if(no_summary.size() < games.size()) {
-				message = "import_saves";
+				message = N_("Your saves directory contains some files that don't appear to have been generated by this version of Battle for Wesnoth. Would you like to register these files with the game?");
 			}
 
 			std::vector<gui::check_item> options;
-			options.push_back(gui::check_item(string_table["dont_ask_again"],false));
-			const int res = gui::show_dialog(disp,NULL,string_table[caption],string_table[message],
+			options.push_back(gui::check_item(_("Don't ask me again!"),false));
+			const int res = gui::show_dialog(disp,NULL,_(caption),_(message),
 			                                 gui::YES_NO,NULL,NULL,"",NULL,NULL,&options);
 
 			generate_summaries = res == 0;
@@ -486,11 +506,11 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	std::vector<gui::check_item> options;
 
 	if(show_replay != NULL)
-		options.push_back(gui::check_item(string_table["show_replay"],false));
+		options.push_back(gui::check_item(_("Show replay"),false));
 
 	const int res = gui::show_dialog(disp,NULL,
-					 string_table["load_game_heading"],
-					 string_table["load_game_message"],
+					 _("Load Game"),
+					 _("Choose the game to load"),
 			         gui::OK_CANCEL,&items,&preview_panes,"",NULL,NULL,&options,-1,-1,NULL,&buttons);
 
 	if(res == -1)
@@ -513,80 +533,13 @@ void unit_speak(const config& message_info, display& disp, const unit_map& units
 	for(unit_map::const_iterator i = units.begin(); i != units.end(); ++i) {
 		if(i->second.matches_filter(message_info)) {
 			disp.scroll_to_tile(i->first.x,i->first.y);
-			const scoped_sdl_surface surface(image::get_image(i->second.type().image_profile(),image::UNSCALED));
+			const surface surface(image::get_image(i->second.type().image_profile(),image::UNSCALED));
 			gui::show_dialog(disp,surface,i->second.underlying_description(),message_info["message"],gui::MESSAGE);
 		}
 	}
 }
 
 
-int show_file_chooser_dialog(display &disp, std::string &filename,
-							 const std::string title,
-							 int xloc, int yloc) {
-	const events::event_context dialog_events_context;
-	const gui::dialog_manager manager;
-	const events::resize_lock prevent_resizing;
-
-	CVideo& screen = disp.video();
-	SDL_Surface* const scr = screen.getSurface();
-
-	const int width = 400;
-	const int height = 400;
-	const int left_padding = 10;
-	const int right_padding = 10;
-	const int top_padding = 10;
-	const int bot_padding = 10;
-
-	// If not both locations were supplied, put the dialog in the middle
-	// of the screen.
-	if (yloc <= -1 || xloc <= -1) {
-		xloc = scr->w / 2 - width / 2;
-		yloc = scr->h / 2 - height / 2;
-	}
-	std::vector<gui::button*> buttons_ptr;
-	gui::button ok_button_(disp, string_table["ok_button"]);
-	gui::button cancel_button_(disp, string_table["cancel_button"]);
-	buttons_ptr.push_back(&ok_button_);
-	buttons_ptr.push_back(&cancel_button_);
-	surface_restorer restorer;
-	gui::draw_dialog(xloc, yloc, width, height, disp, title, NULL, &buttons_ptr, &restorer);
-
-	gui::file_chooser fc(disp, filename);
-	fc.set_location(xloc + left_padding, yloc + top_padding);
-	fc.set_width(width - left_padding - right_padding);
-	fc.set_height(height - top_padding - bot_padding);
-	fc.set_dirty(true);
-	
-	events::raise_draw_event();
-	screen.flip();
-	disp.invalidate_all();
-
-	CKey key;
-	for (;;) {
-		events::pump();
-		events::raise_process_event();
-		events::raise_draw_event();
-		if (fc.choice_made()) {
-			filename = fc.get_choice();
-			return 0; // We know that the OK button is on index 0.
-		}
-		if (key[SDLK_ESCAPE]) {
-			// Escape quits from the dialog.
-			return -1;
-		}
-		for (std::vector<gui::button*>::iterator button_it = buttons_ptr.begin();
-			 button_it != buttons_ptr.end(); button_it++) {
-			if ((*button_it)->pressed()) {
-				// Return the index of the pressed button.
-				filename = fc.get_choice();
-				return button_it - buttons_ptr.begin();
-			}
-		}
-		screen.flip();
-		SDL_Delay(10);
-	}
-}
-
 namespace {
 	static const SDL_Rect unit_preview_size = {-200,-370,200,370};
 	static const SDL_Rect weaponless_unit_preview_size = {-190,-140,190,140};
@@ -594,7 +547,7 @@ namespace {
 }
 
 unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const unit& u, TYPE type, bool on_left_side)
-                                        : gui::preview_pane(disp), details_button_(disp,translate_string("profile"),gui::button::TYPE_PRESS,"lite",gui::button::MINIMUM_SPACE),
+                                        : gui::preview_pane(disp), details_button_(disp,_("Profile"),gui::button::TYPE_PRESS,"lite_small",gui::button::MINIMUM_SPACE),
 										  map_(map), units_(&unit_store_), index_(0), left_(on_left_side),
 										  weapons_(type == SHOW_ALL)
 {
@@ -603,7 +556,7 @@ unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const un
 }
 
 unit_preview_pane::unit_preview_pane(display& disp, const gamemap* map, const std::vector<unit>& units, TYPE type, bool on_left_side)
-                                        : gui::preview_pane(disp), details_button_(disp,translate_string("profile"),gui::button::TYPE_PRESS,"lite",gui::button::MINIMUM_SPACE),
+                                        : gui::preview_pane(disp), details_button_(disp,_("Profile"),gui::button::TYPE_PRESS,"lite_small",gui::button::MINIMUM_SPACE),
 										  map_(map), units_(&units), index_(0), left_(on_left_side),
 										  weapons_(type == SHOW_ALL)
 {
@@ -650,14 +603,14 @@ void unit_preview_pane::draw()
 
 	const bool right_align = left_side();
 
-	SDL_Surface* const screen = disp().video().getSurface();
+	surface const screen = disp().video().getSurface();
 
 	const SDL_Rect area = { location().x+unit_preview_border, location().y+unit_preview_border,
 	                        location().w-unit_preview_border*2, location().h-unit_preview_border*2 };
 	SDL_Rect clip_area = area;
 	const clip_rect_setter clipper(screen,clip_area);
 
-	scoped_sdl_surface unit_image(image::get_image(u.type().image(),image::UNSCALED));
+	surface unit_image(image::get_image(u.type().image(),image::UNSCALED));
 	if(left_side() == false && unit_image != NULL) {
 		unit_image.assign(image::reverse_image(unit_image));
 	}
@@ -670,34 +623,34 @@ void unit_preview_pane::draw()
 		image_rect = rect;
 	}
 
-	SDL_Rect description_rect = {image_rect.x,image_rect.y+image_rect.h,0,0};
-
-	if(u.description().empty() == false) {
-		std::stringstream desc;
-		desc << font::BOLD_TEXT << u.description();
-		const std::string description = desc.str();
-		description_rect = font::text_area(description,12);
-		description_rect = font::draw_text(&disp(),area,12,font::NORMAL_COLOUR,desc.str(),right_align ? image_rect.x : image_rect.x + image_rect.w - description_rect.w,image_rect.y+image_rect.h);
-	}
-
 	//place the 'unit profile' button
 	if(map_ != NULL) {
 		const SDL_Rect button_loc = {right_align ? area.x : area.x + area.w - details_button_.location().w,
-		                             image_rect.y + image_rect.h + description_rect.h,
+		                             image_rect.y + image_rect.h,
 		                             details_button_.location().w,details_button_.location().h};
 		details_button_.set_location(button_loc);
 	}
 
+	SDL_Rect description_rect = {image_rect.x,image_rect.y+image_rect.h+details_button_.location().h,0,0};
+
+	if(u.description().empty() == false) {
+		std::stringstream desc;
+		desc << font::NORMAL_TEXT << u.description();
+		const std::string description = desc.str();
+		description_rect = font::text_area(description,font::SIZE_NORMAL);
+		description_rect = font::draw_text(&disp(),area,font::SIZE_NORMAL,font::NORMAL_COLOUR,desc.str(),right_align ? image_rect.x : image_rect.x + image_rect.w - description_rect.w,image_rect.y+image_rect.h+details_button_.location().h);
+	}
+
 	std::stringstream details;
-	details << font::BOLD_TEXT << u.type().language_name()
-			<< "\n" << font::SMALL_TEXT << "(" << string_table["level"] << " "
-			<< u.type().level() << ")\n"
-			<< translate_string(unit_type::alignment_description(u.type().alignment())) << "\n"
+	details << u.type().language_name()
+			<< "\n" << _("level") << " "
+			<< u.type().level() << "\n"
+			<< string_table[unit_type::alignment_description(u.type().alignment())] << "\n"
 			<< u.traits_description() << " \n";
 
 	const std::vector<std::string>& abilities = u.type().abilities();
 	for(std::vector<std::string>::const_iterator a = abilities.begin(); a != abilities.end(); ++a) {
-		details << translate_string_default("ability_" + *a, *a);
+		details << *a;
 		if(a+1 != abilities.end()) {
 			details << ",";
 		}
@@ -711,11 +664,11 @@ void unit_preview_pane::draw()
 	else if(u.hitpoints() > 2*(u.max_hitpoints()/3))
 		details << font::GOOD_TEXT;
 
-	details << string_table["hp"] << ": " << u.hitpoints()
+	details << _("HP") << ": " << u.hitpoints()
 			<< "/" << u.max_hitpoints() << "\n";
 	
-	if(u.type().advances_to().empty()) {
-		details << string_table["xp"] << ": " << u.experience() << "/-";
+	if(u.can_advance() == false) {
+		details << _("XP") << ": " << u.experience() << "/-";
 	} else {
 		//if killing a unit the same level as us would level us up,
 		//then display in green
@@ -723,12 +676,12 @@ void unit_preview_pane::draw()
 			details << font::GOOD_TEXT;
 		}
 
-		details << string_table["xp"] << ": " << u.experience() << "/" << u.max_experience();
+		details << _("XP") << ": " << u.experience() << "/" << u.max_experience();
 	}
 	
 	if(weapons_) {
 		details << "\n"
-				<< string_table["moves"] << ": " << u.movement_left() << "/"
+				<< _("Moves") << ": " << u.movement_left() << "/"
 				<< u.total_movement()
 				<< "\n";
 
@@ -736,17 +689,13 @@ void unit_preview_pane::draw()
 		for(std::vector<attack_type>::const_iterator at_it = attacks.begin();
 		    at_it != attacks.end(); ++at_it) {
 
-			const std::string& lang_weapon = string_table["weapon_name_" + at_it->name()];
-			const std::string& lang_type = string_table["weapon_type_" + at_it->type()];
-			const std::string& lang_special = string_table["weapon_special_" + at_it->special()];
 			details << "\n"
-					<< (lang_weapon.empty() ? at_it->name():lang_weapon) << " ("
-					<< (lang_type.empty() ? at_it->type():lang_type) << ")\n"
-					<< (lang_special.empty() ? at_it->special():lang_special)<<"\n"
-					<< at_it->damage() << "-" << at_it->num_attacks() << " -- "
+				<< at_it->name() << " (" << at_it->type() << ")\n"
+				<< at_it->special() << "\n"
+				<< at_it->damage() << "-" << at_it->num_attacks() << " -- "
 			        << (at_it->range() == attack_type::SHORT_RANGE ?
-			            string_table["short_range"] :
-						string_table["long_range"]);
+			            _("melee") :
+						_("ranged"));
 	
 			if(at_it->hexes() > 1) {
 				details << " (" << at_it->hexes() << ")";
@@ -755,7 +704,6 @@ void unit_preview_pane::draw()
 	}
 	
 	const std::string text = details.str();
-	const SDL_Rect& text_area = font::text_area(text,12);
 	
 	const std::vector<std::string> lines = config::split(text,'\n');
 
@@ -768,11 +716,11 @@ void unit_preview_pane::draw()
 	for(std::vector<std::string>::const_iterator line = lines.begin(); line != lines.end(); ++line) {
 		int xpos = cur_area.x;
 		if(right_align && !weapons_) {
-			const SDL_Rect& line_area = font::text_area(*line,12);
+			const SDL_Rect& line_area = font::text_area(*line,font::SIZE_SMALL);
 			xpos = cur_area.x + cur_area.w - line_area.w;
 		}
 
-		cur_area = font::draw_text(&disp(),location(),12,font::NORMAL_COLOUR,*line,xpos,cur_area.y);
+		cur_area = font::draw_text(&disp(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,xpos,cur_area.y);
 		cur_area.y += cur_area.h;
 	}
 }
@@ -787,102 +735,86 @@ void unit_preview_pane::process()
 
 void show_unit_description(display& disp, const gamemap& map, const unit& u)
 {
-	const std::string description = u.unit_description()
-	                                + "\n\n" + string_table["see_also"];
-
-	std::vector<std::string> options;
-
-	options.push_back(string_table["terrain_info"]);
-	options.push_back(string_table["attack_resistance"]);
-	options.push_back(string_table["close_window"]);
-
-	const scoped_sdl_surface profile(image::get_image(u.type().image_profile(),image::SCALED));
-
-	const int res = gui::show_dialog(disp, profile, u.type().language_name(),
-	                                 description,gui::MESSAGE, &options);
-	if(res == 0) {
-		show_unit_terrain_table(disp,map,u);
-	} else if(res == 1) {
-		show_unit_resistance(disp,u);
-	}
+	help::show_help(disp,"unit_" + u.type().id());
 }
 
-void show_unit_resistance(display& disp, const unit& u)
+
+namespace {
+	static const SDL_Rect campaign_preview_size = {-350,-400,350,400};
+	static const int campaign_preview_border = 10;
+}
+
+campaign_preview_pane::campaign_preview_pane(display& disp,std::vector<std::pair<std::string,std::string> >* desc) : gui::preview_pane(disp),descriptions_(desc),index_(0)
 {
-	std::vector<std::string> items;
-	items.push_back(string_table["attack_type"] + "," + string_table["attack_resistance"]);
-	const std::map<std::string,std::string>& table = u.type().movement_type().damage_table();
-	for(std::map<std::string,std::string>::const_iterator i = table.begin(); i != table.end(); ++i) {
-		int resistance = 100 - atoi(i->second.c_str());
-
-		//if resistance is less than 0, display in red
-		const char prefix = resistance < 0 ? font::BAD_TEXT : font::NULL_MARKUP;
-
-		const std::string& lang_weapon = string_table["weapon_type_" + i->first];
-		const std::string& weap = lang_weapon.empty() ? i->first : lang_weapon;
-
-		std::stringstream str;
-		str << weap << "," << prefix << resistance << "%";
-		items.push_back(str.str());
-	}
-
-	const events::event_context dialog_events_context;
-	dialogs::unit_preview_pane unit_preview(disp,NULL,u);
-	std::vector<gui::preview_pane*> preview_panes;
-	preview_panes.push_back(&unit_preview);
-
-	gui::show_dialog(disp,NULL,
-	                 u.type().language_name(),
-					 string_table["unit_resistance_table"],
-					 gui::MESSAGE,&items,&preview_panes);
+	set_location(campaign_preview_size);
 }
 
-void show_unit_terrain_table(display& disp, const gamemap& map, const unit& u)
+bool campaign_preview_pane::show_above() const { return false; }
+bool campaign_preview_pane::left_side() const { return false; }
+
+void campaign_preview_pane::set_selection(int index)
 {
-	std::vector<std::string> items;
-	items.push_back(string_table["terrain"] + "," +
-	                string_table["movement"] + "," +
-					string_table["defense"]);
-
-	const unit_type& type = u.type();
-	const unit_movement_type& move_type = type.movement_type();
-	const std::vector<gamemap::TERRAIN>& terrains = map.get_terrain_precedence();
-	for(std::vector<gamemap::TERRAIN>::const_iterator t =
-	    terrains.begin(); t != terrains.end(); ++t) {
-
-		//exclude fog and shroud
-		if(*t == gamemap::FOGGED || *t == gamemap::VOID_TERRAIN) {
-			continue;
-		}
-
-		const terrain_type& info = map.get_terrain_info(*t);
-		if(!info.is_alias()) {
-			const std::string& name = map.terrain_name(*t);
-			const std::string& lang_name = string_table[name];
-			const int moves = move_type.movement_cost(map,*t);
-
-			std::stringstream str;
-			str << lang_name << ",";
-			if(moves < 100)
-				str << moves;
-			else
-				str << "--";
-
-			const int defense = 100 - move_type.defense_modifier(map,*t);
-			str << "," << defense << "%";
-
-			items.push_back(str.str());
-		}
+	index = minimum<int>(descriptions_->size()-1,index);
+	if(index != index_ && index >= 0) {
+		index_ = index;
+		set_dirty();
 	}
-
-	const events::event_context dialog_events_context;
-	dialogs::unit_preview_pane unit_preview(disp,NULL,u);
-	std::vector<gui::preview_pane*> preview_panes;
-	preview_panes.push_back(&unit_preview);
-
-	gui::show_dialog(disp,NULL,u.type().language_name(),
-					 string_table["terrain_info"],
-					 gui::MESSAGE,&items,&preview_panes);
 }
 
-} //end namespace dialogs
\ No newline at end of file
+void campaign_preview_pane::draw()
+{
+	if(!dirty()) {
+		return;
+	}
+
+	bg_restore();
+
+	if(index_ < 0 || index_ >= descriptions_->size()) {
+		return;
+	}
+
+	set_dirty(false);
+
+	const SDL_Rect area = {
+		location().x+campaign_preview_border,
+		location().y+campaign_preview_border*5,
+		location().w-campaign_preview_border*2,
+		location().h-campaign_preview_border*6 };
+
+	/* background frame */
+	static const std::string default_style("mainmenu");
+	const std::string* style = &default_style;
+	gui::draw_dialog_frame(area.x,area.y,area.w,area.h,disp(),style);
+
+	/* description text */
+	const std::string& desc_text = font::word_wrap_text((*descriptions_)[index_].first,font::SIZE_SMALL,area.w-2*campaign_preview_border);
+	const std::vector<std::string> lines = config::split(desc_text,'\n');
+	SDL_Rect txt_area = { area.x+campaign_preview_border,area.y,0,0 };
+
+	for(std::vector<std::string>::const_iterator line = lines.begin(); line != lines.end(); ++line) {
+		txt_area = font::draw_text(&disp(),location(),font::SIZE_SMALL,font::NORMAL_COLOUR,*line,txt_area.x,txt_area.y);
+		txt_area.y += txt_area.h;
+	}
+
+	/* description image */
+	surface img(NULL);
+	const std::string desc_img_name = (*descriptions_)[index_].second;
+	if(!desc_img_name.empty()) {
+		img.assign(image::get_image(desc_img_name,image::UNSCALED));
+	}
+	if (!img.null()) {
+		SDL_Rect src_rect,dst_rect;
+		int max_height = area.h-(txt_area.h+txt_area.y-area.y);
+
+		src_rect.x = src_rect.y = 0;
+		src_rect.w = minimum<int>(area.w,img->w);
+		src_rect.h = minimum<int>(max_height,img->h);
+		dst_rect.x = area.x+(area.w-src_rect.w)/2;
+		dst_rect.y = txt_area.y+(max_height-src_rect.h)/2;
+
+		SDL_BlitSurface(img,&src_rect,disp().video().getSurface(),&dst_rect);
+
+	}
+}
+
+} //end namespace dialogs
