@@ -42,8 +42,8 @@ namespace map_editor {
 
 bool confirm_modification_disposal(display& disp) {
 	const int res = gui::show_dialog(disp, NULL, "",
-									 translate_string("edit_modifications_lost_confirm"),
-									 gui::OK_CANCEL);
+					 _("Your modifications to the map will be lost.  Continue?"),
+					 gui::OK_CANCEL);
 	return res == 0;
 }
 
@@ -69,10 +69,10 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 	gui::draw_dialog_frame(xpos,ypos,width,height,disp);
 
 	SDL_Rect title_rect = font::draw_text(NULL,disp.screen_area(),24,font::NORMAL_COLOUR,
-					      translate_string("edit_create_new_map_title"),0,0);
+					      _("Create New Map"),0,0);
 
-	const std::string& width_label = string_table["map_width"] + ":";
-	const std::string& height_label = string_table["map_height"] + ":";
+	const std::string& width_label = _("Width:");
+	const std::string& height_label = _("Height:");
 
 	SDL_Rect width_rect = font::draw_text(NULL, disp.screen_area(), 14, font::NORMAL_COLOUR,
 										  width_label, 0, 0);
@@ -88,10 +88,10 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 	width_rect.y = ypos + title_rect.h + vertical_margin*2;
 	height_rect.y = width_rect.y + width_rect.h + vertical_margin;
 
-	gui::button new_map_button(disp, translate_string("edit_generate_new_map"));
-	gui::button random_map_button(disp, translate_string("edit_generate_random_map"));
-	gui::button random_map_setting_button(disp, translate_string("edit_random_generator_settings"));
-	gui::button cancel_button(disp, translate_string("cancel_button"));
+	gui::button new_map_button(disp, _("Generate New Map"));
+	gui::button random_map_button(disp, _("Generate Random Map"));
+	gui::button random_map_setting_button(disp, _("Random Generator Settings"));
+	gui::button cancel_button(disp, _("Cancel"));
 
 	new_map_button.set_location(xpos + horz_margin,height_rect.y + height_rect.h + vertical_margin);
 	random_map_button.set_location(xpos + horz_margin,ypos + height - random_map_button.height()-14*2-vertical_margin);
@@ -174,7 +174,7 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 					random_map_generator.get()->create_map(std::vector<std::string>());
 				if (map == "") {
 					gui::show_dialog(disp, NULL, "",
-									 translate_string("edit_map_creation_failed"), gui::OK_ONLY);
+									 _("Map creation failed."), gui::OK_ONLY);
 				}
 				return map;
 			}
@@ -188,7 +188,7 @@ std::string new_map_dialog(display& disp, gamemap::TERRAIN fill_terrain,
 			map_height = height_slider.value();
 			gui::draw_dialog_frame(xpos,ypos,width,height,disp);
 			title_rect = font::draw_text(&disp,disp.screen_area(),24,font::NORMAL_COLOUR,
-										 translate_string("edit_create_new_map_title"),
+										 _("Create New Map"),
 										 xpos+(width-title_rect.w)/2,ypos+10);
 
 			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,
@@ -240,15 +240,15 @@ void preferences_dialog(display &disp, config &prefs) {
 
 	SDL_Rect clip_rect = {0,0,disp.x(),disp.y()};
 
-	gui::button close_button(disp,string_table["close_window"]);
+	gui::button close_button(disp,_("Close Window"));
 
 	std::vector<gui::button*> buttons;
 	buttons.push_back(&close_button);
 
 	surface_restorer restorer;
-	gui::draw_dialog(xpos,ypos,width,height,disp,string_table["preferences"],NULL,&buttons,&restorer);
+	gui::draw_dialog(xpos,ypos,width,height,disp,_("Preferences"),NULL,&buttons,&restorer);
 
-	const std::string& scroll_label = string_table["scroll_speed"];
+	const std::string& scroll_label = _("Scroll Speed:");
 
 	SDL_Rect scroll_rect = {0,0,0,0};
 	scroll_rect = font::draw_text(NULL,clip_rect,14,font::NORMAL_COLOUR,
@@ -274,24 +274,24 @@ void preferences_dialog(display &disp, config &prefs) {
 	scroll_slider.set_max(100);
 	scroll_slider.set_value(preferences::scroll_speed());
 
-	gui::button fullscreen_button(disp,string_table["full_screen"],
+	gui::button fullscreen_button(disp,_("Full Screen"),
 	                              gui::button::TYPE_CHECK);
 
 	fullscreen_button.set_check(preferences::fullscreen());
 
 	fullscreen_button.set_location(slider_left,scroll_pos + 80);
 
-	gui::button grid_button(disp,string_table["grid_button"],
+	gui::button grid_button(disp,_("Show Grid"),
 	                        gui::button::TYPE_CHECK);
 	grid_button.set_check(preferences::grid());
 
 	grid_button.set_location(slider_left + fullscreen_button.width() + 100,
 							 scroll_pos + 80);
 
-	gui::button resolution_button(disp,string_table["video_mode"]);
+	gui::button resolution_button(disp,_("Video Mode"));
 	resolution_button.set_location(slider_left,scroll_pos + 80 + 50);
 
-	gui::button hotkeys_button (disp,string_table["hotkeys_button"]);
+	gui::button hotkeys_button (disp,_("Hotkeys"));
 	hotkeys_button.set_location(slider_left + fullscreen_button.width() + 100,
 								scroll_pos + 80 + 50);
 
@@ -312,7 +312,7 @@ void preferences_dialog(display &disp, config &prefs) {
 
 		if(redraw_all) {
 			restorer.cancel();
-			gui::draw_dialog(xpos,ypos,width,height,disp,string_table["preferences"],NULL,&buttons,&restorer);
+			gui::draw_dialog(xpos,ypos,width,height,disp,_("Preferences"),NULL,&buttons,&restorer);
 			fullscreen_button.set_dirty();
 			close_button.set_dirty();
 			resolution_button.set_dirty();
@@ -380,10 +380,10 @@ resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h) {
 	gui::draw_dialog_frame(xpos,ypos,width,height,disp);
 
 	SDL_Rect title_rect = font::draw_text(NULL,disp.screen_area(),24,font::NORMAL_COLOUR,
-					      translate_string("edit_resize_map_title"),0,0);
+					      _("Resize Map"),0,0);
 
-	const std::string& width_label = string_table["map_width"] + ":";
-	const std::string& height_label = string_table["map_height"] + ":";
+	const std::string& width_label = _("Width:");
+	const std::string& height_label = _("Height:");
 
 	SDL_Rect width_rect = font::draw_text(NULL, disp.screen_area(), 14, font::NORMAL_COLOUR,
 										  width_label, 0, 0);
@@ -399,8 +399,8 @@ resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h) {
 	width_rect.y = ypos + title_rect.h + vertical_margin*2;
 	height_rect.y = width_rect.y + width_rect.h + vertical_margin;
 
-	gui::button cancel_button(disp, translate_string("cancel_button"));
-	gui::button ok_button(disp, translate_string("ok_button"));
+	gui::button cancel_button(disp, _("Cancel"));
+	gui::button ok_button(disp, _("Ok"));
 
 	cancel_button.set_location(xpos + width - cancel_button.width() - horz_margin,
 	                           ypos + height - cancel_button.height()-14);
@@ -438,7 +438,7 @@ resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h) {
 			map_height = height_slider.value();
 			gui::draw_dialog_frame(xpos,ypos,width,height,disp);
 			title_rect = font::draw_text(&disp,disp.screen_area(),24,font::NORMAL_COLOUR,
-										 translate_string("edit_resize_map_title"),
+										 _("Resize Map"),
 										 xpos+(width-title_rect.w)/2,ypos+10);
 
 			font::draw_text(&disp,disp.screen_area(),14,font::NORMAL_COLOUR,
@@ -481,9 +481,9 @@ resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h) {
 
 FLIP_AXIS flip_dialog(display &disp) {
 	std::vector<std::string> items;
-	items.push_back(translate_string("edit_x_axis"));
-	items.push_back(translate_string("edit_y_axis"));
-	const std::string msg = translate_string("edit_flip_around");
+	items.push_back(_("X-Axis"));
+	items.push_back(_("Y-Axis"));
+	const std::string msg = _("Flip around (this may change the dimensions of the map):");
 	const int res =
 		gui::show_dialog(disp, NULL, "",
 						 font::word_wrap_text(msg, 12, 180),
