@@ -255,16 +255,17 @@ void terrain_palette::draw(bool force) {
 		const gamemap::TERRAIN terrain = terrains_[counter];
 		const std::string filename = "terrain/" +
 			map_.get_terrain_info(terrain).default_image() + ".png";
-		scoped_sdl_surface image(image::get_image(filename, image::UNSCALED));
+		surface image(image::get_image(filename, image::UNSCALED));
+		if(image == NULL) {
+			std::cerr << "image for terrain " << counter << ": '" << filename << "' not found\n";
+			return;
+		}
+
 		if((unsigned)image->w != size_specs_.terrain_size
 		   || (unsigned)image->h != size_specs_.terrain_size) {
 			image.assign(scale_surface(image, size_specs_.terrain_size,
 									   size_specs_.terrain_size));
 		}
-		if(image == NULL) {
-			std::cerr << "image for terrain '" << counter << "' not found\n";
-			return;
-		}
 		const int counter_from_zero = counter - starting;
 		SDL_Rect dstrect;
 		dstrect.x = loc.x + (counter_from_zero % 2 != 0 ? size_specs_.terrain_space : 0);
@@ -310,9 +311,9 @@ int terrain_palette::tile_selected(const int x, const int y) const {
 }
 
 void terrain_palette::update_report() {
-	const std::string msg = translate_string("edit_foreground_short") + ": "
+	const std::string msg = std::string(_("FG")) + ": "
 		+ get_terrain_string(selected_fg_terrain()) + "\n"
-		+ translate_string("edit_background_short") +
+		+ std::string(_("BG")) +
 		": " + get_terrain_string(selected_bg_terrain());
 	reports::set_report_content(reports::SELECTED_TERRAIN, msg);
 }
@@ -402,7 +403,7 @@ void brush_bar::draw(bool force) {
 	for (int i = 1; i <= total_brush_; i++) {
 		std::stringstream filename;
 		filename << "editor/brush-" << i << ".png";
-		scoped_sdl_surface image(image::get_image(filename.str(), image::UNSCALED));
+		surface image(image::get_image(filename.str(), image::UNSCALED));
 		if (image == NULL) {
 			std::cerr << "Image " << filename.str() << " not found." << std::endl;
 			continue;
