@@ -1,3 +1,9 @@
+//disable the very annoying VC++ warning 4786
+#ifdef WIN32
+#pragma warning(disable:4786)
+#endif
+
+#include "clipboard.hpp"
 #include "cursor.hpp"
 #include "events.hpp"
 #include "mouse.hpp"
@@ -216,7 +222,7 @@ void pump()
 
 			case SDL_APPMOUSEFOCUS: {
 				SDL_ActiveEvent& ae = reinterpret_cast<SDL_ActiveEvent&>(event);
-				if(ae.state == SDL_APPMOUSEFOCUS) {
+				if(ae.state == SDL_APPMOUSEFOCUS || ae.state == SDL_APPINPUTFOCUS) {
 					cursor::set_focus(ae.gain == 1);
 				}
 				break;
@@ -243,12 +249,17 @@ void pump()
 			}
 		
 			case SDL_MOUSEMOTION: {
+				//always make sure a cursor is displayed if the
+				//mouse moves or if the user clicks
+				cursor::set_focus(true);
 				raise_help_string_event(event.motion.x,event.motion.y);
 				break;
 			}
 
 			case SDL_MOUSEBUTTONDOWN: {
-				
+				//always make sure a cursor is displayed if the
+				//mouse moves or if the user clicks
+				cursor::set_focus(true);
 				if(event.button.button == SDL_BUTTON_LEFT) {
 					static const int DoubleClickTime = 500;
 
@@ -279,6 +290,14 @@ void pump()
 
 				break;
 			}
+                        
+#if defined(_X11) && !defined(__APPLE__)
+			case SDL_SYSWMEVENT: {
+				//clipboard support for X11
+				handle_system_event(event);
+				break;
+			}
+#endif
 
 			case SDL_QUIT: {
 				throw CVideo::quit();
