@@ -1,4 +1,4 @@
-/* $Id: filesystem.hpp,v 1.21 2004/06/20 17:13:58 gruikya Exp $ */
+/* $Id: filesystem.hpp,v 1.29 2004/09/18 21:42:13 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -37,8 +37,13 @@ std::string get_prefs_file();
 std::string get_save_index_file();
 std::string get_saves_dir();
 std::string get_cache_dir();
+std::string get_intl_dir();
 std::string get_user_data_dir();
 
+std::string get_cwd();
+
+void make_directory(const std::string& dirname);
+
 std::string read_map(const std::string& name);
 
 //function which returns true iff the given file is a directory
@@ -50,12 +55,25 @@ bool file_exists(const std::string& name);
 //function to get the creation time of a file
 time_t file_create_time(const std::string& fname);
 
+struct file_tree_checksum
+{
+	file_tree_checksum();
+	explicit file_tree_checksum(const class config& cfg);
+	void write(class config& cfg) const;
+	size_t nfiles, sum_size;
+	time_t modified;
+};
+
+bool operator==(const file_tree_checksum& lhs, const file_tree_checksum& rhs);
+bool operator!=(const file_tree_checksum& lhs, const file_tree_checksum& rhs);
+
+
 //function to get the time at which the most recently modified file
 //in a directory tree was modified at
-time_t file_tree_modified_time(const std::string& path, time_t tm=0);
+file_tree_checksum get_file_tree_checksum(const std::string& path);
 
 //function to get the time at which the data/ tree was last modified at
-time_t data_tree_modified_time();
+const file_tree_checksum& data_tree_checksum();
 
 //returns the size of a file, or -1 if the file doesn't exist
 int file_size(const std::string& fname);
@@ -64,6 +82,10 @@ int file_size(const std::string& fname);
 //to a portable basename() function
 std::string file_name(const std::string& file);
 
+//returns the directory name of a file, with filename stripped. Equivalent to a
+//portable dirname()
+std::string directory_name(const std::string& file);
+
 ///the paths manager is responsible for recording the various paths that
 ///binary files may be located at. It should be passed a config object
 ///which holds binary path information. This is in the format
@@ -73,10 +95,18 @@ std::string file_name(const std::string& file);
 ///Binaries will be searched for in [wesnoth-path]/data/<path>/images/
 struct binary_paths_manager
 {
-	binary_paths_manager(const struct config& cfg);
+	binary_paths_manager();
+	binary_paths_manager(const class config& cfg);
 	~binary_paths_manager();
 
+	void set_paths(const class config& cfg);
+
 private:
+	binary_paths_manager(const binary_paths_manager& o);
+	binary_paths_manager& operator=(const binary_paths_manager& o);
+
+	void cleanup();
+
 	std::vector<std::string> paths_;
 };
 
