@@ -1,4 +1,4 @@
-/* $Id: font.cpp,v 1.73 2004/06/16 14:40:15 Sirp Exp $ */
+/* $Id: font.cpp,v 1.86 2004/09/18 21:24:59 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,8 +11,6 @@
    See the COPYING file for more details.
 */
 
-#include "SDL_ttf.h"
-
 #include "config.hpp"
 #include "font.hpp"
 #include "game_config.hpp"
@@ -23,13 +21,19 @@
 #include "tooltips.hpp"
 #include "util.hpp"
 
+#include <algorithm>
 #include <cstdio>
 #include <iostream>
+#include <list>
 #include <map>
 #include <sstream>
 #include <stack>
 #include <string>
 
+#define LOG_FT lg::info(lg::display)
+#define WRN_FT lg::warn(lg::display)
+#define ERR_FT lg::err(lg::display)
+
 namespace {
 
 std::map<int,TTF_Font*> font_table;
@@ -41,36 +45,38 @@ TTF_Font* open_font(const std::string& fname, int size)
 {
 	std::string name;
 
+	LOG_FT << "Opening font '" << fname << "' ...\n";
 	if(game_config::path.empty() == false) {
 		name = game_config::path + "/fonts/" + fname;
-		std::cerr << "Opening font file: " << name << " ...\n";
+		LOG_FT << "Trying file '" << name << "' ...\n";
 
 		if(read_file(name).empty()) {
 			name = "fonts/" + fname;
-			std::cerr << "Failed, now trying: " << name << " ...\n";
+			WRN_FT << "Failed opening '" << name << "'; now trying '" << name << "' ...\n";
 			if(read_file(name).empty()) {
-				std::cerr << "Failed :(\n";
+				ERR_FT << "Failed opening font: " << fname << "\n";
 				return NULL;
 			}
 		}
 	} else {
 		name = "fonts/" + fname;
-		std::cerr << "Opening font file: " << name << " ...\n";
+		LOG_FT << "Trying file '" << name << "' ...\n";
 
 		if(read_file(name).empty()) {
-			std::cerr << "Failed :(\n";
+			ERR_FT << "Failed opening font '" << fname << "'\n";
 			return NULL;
 		}
 	}
 
-	std::cerr << "opening font '" << name << "' " << size << "\n";
+	LOG_FT << "Opening font file '" << name << "', font size is " << size << "\n";
 
 	TTF_Font* font = TTF_OpenFont(name.c_str(),size);
 	if(font == NULL) {
-		std::cerr << "Could not open font file: " << name << '\n';
+		ERR_FT << "Failed opening font file '" << name << "'\n";
+		return NULL;
 	}
 
-	std::cerr << "opened font okay\n";
+	LOG_FT << "Opened font okay\n";
 
 	return font;
 }
@@ -88,7 +94,7 @@ TTF_Font* get_font(int size)
 
 	TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
 
-	std::cerr << "Inserting font...\n";
+	LOG_FT << "Inserting font...\n";
 	font_table.insert(std::pair<int,TTF_Font*>(size,font));
 	return font;
 }
@@ -116,7 +122,7 @@ struct font_style_setter
 		//SDL_ttf to segfault. We work around this here by disallowing combinations
 		//of styles
 		if((style&TTF_STYLE_UNDERLINE) != 0) {
-			style = TTF_STYLE_UNDERLINE;
+			style = TTF_STYLE_NORMAL; //TTF_STYLE_UNDERLINE;
 		} else if((style&TTF_STYLE_BOLD) != 0) {
 			style = TTF_STYLE_BOLD;
 		} else if((style&TTF_STYLE_ITALIC) != 0) {
@@ -144,10 +150,10 @@ manager::manager()
 {
 	const int res = TTF_Init();
 	if(res == -1) {
-		std::cerr << "Could not initialize true type fonts\n";
+		ERR_FT << "Could not initialize true type fonts\n";
 		throw error();
 	} else {
-		std::cerr << "Initialized true type fonts\n";
+		LOG_FT << "Initialized true type fonts\n";
 	}
 }
 
@@ -157,13 +163,10 @@ manager::~manager()
 	TTF_Quit();
 }
 
-void set_font(const std::string& name)
+void set_font()
 {
 	clear_fonts();
-	font_name = name;
-	if(font_name == "") {
-		font_name = "Vera.ttf";
-	}
+	font_name = _("Vera.ttf");
 }
 
 const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
@@ -172,47 +175,160 @@ const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
                 BLACK_COLOUR  = {0x00,0x00,0x00,0},
                 DARK_COLOUR   = {0x00,0x00,0x66,0},
                 YELLOW_COLOUR = {0xFF,0xFF,0x00,0},
-                BUTTON_COLOUR = {0xBC,0xB0,0x88,0};
+                BUTTON_COLOUR = {0xBC,0xB0,0x88,0},
+                TITLE_COLOUR  = {0xBC,0xB0,0x88,0};
 
 const char LARGE_TEXT='*', SMALL_TEXT='`', GOOD_TEXT='@', BAD_TEXT='#',
            NORMAL_TEXT='{', BLACK_TEXT='}', BOLD_TEXT='~', IMAGE='&', NULL_MARKUP='^';
 
 namespace {
 
-SDL_Rect text_size(TTF_Font* font, const std::string& str, const SDL_Color& colour, int style)
+static const size_t max_text_line_width = 4096;
+
+class text_surface
 {
-	const font_style_setter style_setter(font,style);
-	SDL_Rect res = {0,0,0,0};
-	int w = 0, h = 0;
+public:
+	text_surface(TTF_Font *font, std::string const &str, SDL_Color color, int style);
+	text_surface(TTF_Font *font, SDL_Color color, int style);
+	void set_text(std::string const &str);
+	bool operator==(text_surface const &t) const { return hash_ == t.hash_ && equal(t); }
+	bool operator!=(text_surface const &t) const { return hash_ != t.hash_ || !equal(t); }
+	size_t width() const;
+	size_t height() const;
+	surface const &get_surface() const;
+private:
+	int hash_;
+	TTF_Font *font_; // no font should ever be closed
+	std::string str_;
+	SDL_Color color_;
+	int style_;
+	mutable int w_, h_;
+	mutable surface surf_;
+	void hash();
+	void measure() const;
+	bool equal(text_surface const &t) const;
+};
 
-	TTF_SizeUTF8(font,str.c_str(),&w,&h);
-
-	res.w = size_t(w);
-	res.h = size_t(h);
-
-	return res;
+text_surface::text_surface(TTF_Font *font, std::string const &str, SDL_Color color, int style)
+  : font_(font), str_(str), color_(color), style_(style), w_(-1), h_(-1)
+{
+	hash();
 }
 
-SDL_Surface* render_text_internal(TTF_Font* font,const std::string& str,
-						 const SDL_Color& colour, int style)
+text_surface::text_surface(TTF_Font *font, SDL_Color color, int style)
+  : hash_(0), font_(font), color_(color), style_(style), w_(-1), h_(-1)
+{}
+
+void text_surface::set_text(std::string const &str)
 {
-	const font_style_setter style_setter(font,style);
-
-	return TTF_RenderUTF8_Blended(font,str.c_str(),colour);
-
+	str_ = str;
+	hash();
 }
 
-SDL_Surface* render_text(TTF_Font* font,const std::string& text, const SDL_Color& colour, int style)
+void text_surface::hash()
 {
+	int h = 0;
+	for(std::string::const_iterator it = str_.begin(), it_end = str_.end(); it != it_end; ++it)
+		h = ((h << 9) | (h >> (sizeof(int) * 8 - 9))) ^ (*it);
+	hash_ = h;
+}
+
+bool text_surface::equal(text_surface const &t) const
+{
+	return font_ == t.font_
+		&& color_.r == t.color_.r && color_.g == t.color_.g && color_.b == t.color_.b
+		&& style_ == t.style_
+		&& str_ == t.str_;
+}
+
+void text_surface::measure() const
+{
+	font_style_setter const style_setter(font_, style_);
+	TTF_SizeUTF8(font_, str_.c_str(), &w_, &h_);
+}
+
+size_t text_surface::width() const
+{
+	if (w_ == -1)
+		measure();
+	return w_;
+}
+
+size_t text_surface::height() const
+{
+	if (h_ == -1)
+		measure();
+	return h_;
+}
+
+surface const &text_surface::get_surface() const
+{
+	if (!surf_.null())
+		return surf_;
+
+	// Impose a maximal number of characters for a text line. Do now draw
+	// any text longer that that, to prevent a SDL buffer overflow
+	if (width() > max_text_line_width)
+		return surf_;
+
+	// Validate the UTF-8 string: workaround a SDL_TTF bug that makes it
+	// crash when used with an invalid UTF-8 string
+	std::string fixed_str = wstring_to_string(string_to_wstring(str_));
+
+	font_style_setter const style_setter(font_, style_);
+	surf_ = surface(TTF_RenderUTF8_Blended(font_, fixed_str.c_str(), color_));
+	return surf_;
+}
+
+class text_cache
+{
+public:
+	static text_surface &find(text_surface const &t);
+private:
+	typedef std::list< text_surface > text_list;
+	static text_list cache_;
+};
+
+text_cache::text_list text_cache::cache_;
+
+text_surface &text_cache::find(text_surface const &t)
+{
+	static size_t lookup_ = 0, hit_ = 0;
+	text_list::iterator it_bgn = cache_.begin(), it_end = cache_.end();
+	text_list::iterator it = std::find(it_bgn, it_end, t);
+	if (it != it_end) {
+		cache_.splice(it_bgn, cache_, it);
+		++hit_;
+	} else {
+		if (cache_.size() >= 50)
+			cache_.pop_back();
+		cache_.push_front(t);
+	}
+	if (++lookup_ % 1000 == 0) {
+		LOG_FT << "Text cache: " << lookup_ << " lookups, " << (hit_ / 10) << "% hits\n";
+		hit_ = 0;
+	}
+	return cache_.front();
+}
+
+surface render_text(TTF_Font* font,const std::string& text, const SDL_Color& colour, int style)
+{
+	if (font == NULL)
+		return surface();
+
 	// XXX Changed by erl, to not strip when rendering text. Works everywhere?
 	const std::vector<std::string> lines = config::split(text,'\n', config::REMOVE_EMPTY);
-	std::vector<shared_sdl_surface> surfaces;
+	std::vector<surface> surfaces;
+	surfaces.reserve(lines.size());
 	size_t width = 0, height = 0;
-	for(std::vector<std::string>::const_iterator ln = lines.begin(); ln != lines.end(); ++ln) {
-		if(*ln != "" && font != NULL) {
-			shared_sdl_surface res(render_text_internal(font,*ln,colour,style));
+	text_surface txt_surf(font, colour, style);
 
-			if(res != NULL) {
+	for(std::vector< std::string >::const_iterator ln = lines.begin(), ln_end = lines.end(); ln != ln_end; ++ln) {
+		if (!ln->empty()) {
+			txt_surf.set_text(*ln);
+			surface const &res = text_cache::find(txt_surf).get_surface();
+
+			if (!res.null()) {
 				surfaces.push_back(res);
 				width = maximum<size_t>(res->w,width);
 				height += res->h;
@@ -220,27 +336,25 @@ SDL_Surface* render_text(TTF_Font* font,const std::string& text, const SDL_Color
 		}
 	}
 
-	if(surfaces.empty()) {
-		return NULL;
-	} else if(surfaces.size() == 1) {
-		sdl_add_ref(surfaces.front());
+	if (surfaces.empty()) {
+		return surface();
+	} else if (surfaces.size() == 1) {
 		return surfaces.front();
 	} else {
 
-		shared_sdl_surface res(create_compatible_surface(surfaces.front(),width,height));
-		if(res == NULL) {
-			return NULL;
-		}
+		surface res(create_compatible_surface(surfaces.front(),width,height));
+		if (res.null())
+			return res;
 
 		size_t ypos = 0;
-		for(std::vector<shared_sdl_surface>::const_iterator i = surfaces.begin(); i != surfaces.end(); ++i) {
-			SDL_SetAlpha(*i,0,0);
-			SDL_Rect dstrect = {0,ypos,(*i)->w,(*i)->h};
-			SDL_BlitSurface(*i,NULL,res,&dstrect);
+		for(std::vector< surface >::const_iterator i = surfaces.begin(),
+		    i_end = surfaces.end(); i != i_end; ++i) {
+			SDL_SetAlpha(*i, 0, 0); // direct blit without alpha blending
+			SDL_Rect dstrect = {0, ypos};
+			SDL_BlitSurface(*i, NULL, res, &dstrect);
 			ypos += (*i)->h;
 		}
 
-		sdl_add_ref(res);
 		return res;
 	}
 }
@@ -300,41 +414,39 @@ std::string::const_iterator parse_markup(std::string::const_iterator i1, std::st
 }
 
 
-SDL_Surface* get_rendered_text(const std::string& str, int size, const SDL_Color& colour, int style)
+surface get_rendered_text(const std::string& str, int size, const SDL_Color& colour, int style)
 {
 	TTF_Font* const font = get_font(size);
 	if(font == NULL) {
-		std::cerr << "Could not get font\n";
+		ERR_FT << "Could not get font for size " << size << "\n";
 		return NULL;
 	}
 
-	SDL_Surface *res = render_text(font,str,colour,style);
-	if(res == NULL) {
-		return NULL;
-	}
-	return res;
+	return render_text(font, str, colour, style);
 }
 
 
-SDL_Rect draw_text_line(SDL_Surface *gui_surface, const SDL_Rect& area, int size,
+SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 		   const SDL_Color& colour, const std::string& text,
-		   int x, int y, SDL_Surface* bg, bool use_tooltips, int style)
+		   int x, int y, surface bg, bool use_tooltips, int style)
 {
 	
 	TTF_Font* const font = get_font(size);
 	if(font == NULL) {
-		std::cerr << "Could not get font\n";
+		ERR_FT << "Could not get font for size " << size << "\n";
 		SDL_Rect res = {0,0,0,0};
 		return res;
 	}
 
 	const std::string etext = make_text_ellipsis(text, size, area.w);
 
-	if(gui_surface == NULL) {
-		return text_size(font,etext,colour,style);
+	if (gui_surface.null()) {
+		text_surface const &u = text_cache::find(text_surface(font, text, colour, style));
+		SDL_Rect res = {0, 0, u.width(), u.height()};
+		return res;
 	}
 
-	scoped_sdl_surface surface(render_text(font,etext,colour,style));
+	surface surface(render_text(font,etext,colour,style));
 	if(surface == NULL) {
 		SDL_Rect res = {0,0,0,0};
 		return res;
@@ -368,7 +480,7 @@ SDL_Rect draw_text_line(SDL_Surface *gui_surface, const SDL_Rect& area, int size
 		SDL_Rect src = dest;
 		src.x = 0;
 		src.y = 0;
-		sdl_safe_blit(surface,&src,gui_surface,&dest);
+		SDL_BlitSurface(surface,&src,gui_surface,&dest);
 	}
 
 	if(use_tooltips) {
@@ -380,9 +492,9 @@ SDL_Rect draw_text_line(SDL_Surface *gui_surface, const SDL_Rect& area, int size
 
 SDL_Rect draw_text_line(display* gui, const SDL_Rect& area, int size,
                         const SDL_Color& colour, const std::string& text,
-                        int x, int y, SDL_Surface* bg, bool use_tooltips, int style)
+                        int x, int y, surface bg, bool use_tooltips, int style)
 {
-	SDL_Surface* surface;
+	surface surface;
 	
 	if(gui == NULL) {
 		surface = NULL;
@@ -401,7 +513,7 @@ SDL_Rect text_area(const std::string& text, int size, int style)
 
 SDL_Rect draw_text(display* gui, const SDL_Rect& area, int size,
                    const SDL_Color& colour, const std::string& txt,
-                   int x, int y, SDL_Surface* bg, bool use_tooltips,
+                   int x, int y, surface bg, bool use_tooltips,
                    MARKUP use_markup, int style)
 {
 	//make sure there's always at least a space, so we can ensure
@@ -481,17 +593,18 @@ std::string remove_first_space(const std::string& text)
 
 namespace font {
 
-	int line_width(const std::string line, int font_size)
+	int line_width(const std::string line, int font_size, int style)
 	{
     
 		TTF_Font* const font = get_font(font_size);
 		if(font == NULL) {
-			std::cerr << "Could not get font\n";
+			ERR_FT << "Could not get font for size " << font_size << "\n";
 			return 0;
 		}
 		int w = 0;
 		int h = 0;
   
+		font_style_setter style_setter(font, style);
 		TTF_SizeUTF8(font, line.c_str(), &w, &h);
 
 		return w;
@@ -500,7 +613,7 @@ namespace font {
   
 	std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width)
 	{
-		std::cerr << "Wrapping word " << unwrapped_text << "\n";
+		//std::cerr << "Wrapping word " << unwrapped_text << "\n";
 		
 		std::string wrapped_text; // the final result
   
@@ -508,7 +621,7 @@ namespace font {
 		std::string cur_word; // including start-whitespace
 		std::string cur_line; // the whole line so far
   
-		for(int c = 0; c < unwrapped_text.length(); c++) {
+		for(size_t c = 0; c < unwrapped_text.length(); c++) {
 
 			// Find the next word
 			bool forced_line_break = false;
@@ -593,7 +706,7 @@ namespace font {
 	
 	SDL_Rect draw_wrapped_text(display* gui, const SDL_Rect& area, int font_size,
 			     const SDL_Color& colour, const std::string& text,
-			     int x, int y, int max_width, SDL_Surface* bg)
+			     int x, int y, int max_width, surface bg)
 	{
 		std::string wrapped_text = word_wrap_text(text, font_size, max_width);
 		return font::draw_text(gui, area, font_size, colour, wrapped_text, x, y, bg, false, NO_MARKUP);
@@ -617,10 +730,10 @@ public:
 
 	void move(double xmove, double ymove);
 
-	void draw(SDL_Surface* screen);
-	void undraw(SDL_Surface* screen);
+	void draw(surface screen);
+	void undraw(surface screen);
 
-	SDL_Surface* create_surface();
+	surface create_surface();
 
 	bool expired() const;
 
@@ -634,7 +747,7 @@ private:
 
 	int xpos(size_t width) const;
 
-	shared_sdl_surface surf_, buf_, foreground_;
+	surface surf_, buf_, foreground_;
 	std::string text_;
 	int font_size_;
 	SDL_Color colour_, bgcolour_;
@@ -673,49 +786,10 @@ int floating_label::xpos(size_t width) const
 	return xpos;
 }
 
-SDL_Surface* floating_label::create_surface()
+surface floating_label::create_surface()
 {
-	if(surf_ == NULL) {
-
-		const std::vector<std::string> lines = config::split(text_,'\n');
-		std::vector<shared_sdl_surface> surfaces;
-		for(std::vector<std::string>::const_iterator ln = lines.begin(); ln != lines.end(); ++ln) {
-			SDL_Color colour = colour_;
-			int size = font_size_;
-			int style = 0;
-			const std::string::const_iterator i1 = font::parse_markup(ln->begin(),ln->end(),&size,&colour,&style);
-			const std::string str(i1,ln->end());
-
-			TTF_Font* const font = get_font(size);
-
-			if(str != "" && font != NULL) {
-				surfaces.push_back(shared_sdl_surface(font::render_text(font,str,colour,style)));
-			}
-		}
-
-		if(surfaces.empty()) {
-			return NULL;
-		} else if(surfaces.size() == 1) {
-			surf_.assign(surfaces.front());
-		} else {
-			size_t width = 0, height = 0;
-			std::vector<shared_sdl_surface>::const_iterator i;
-			for(i = surfaces.begin(); i != surfaces.end(); ++i) {
-				width = maximum<size_t>((*i)->w,width);
-				height += (*i)->h;
-			}
-
-			const SDL_PixelFormat* const format = surfaces.front()->format;
-			surf_.assign(create_compatible_surface(surfaces.front(),width,height));
-
-			size_t ypos = 0;
-			for(i = surfaces.begin(); i != surfaces.end(); ++i) {
-				SDL_SetAlpha(*i,0,0);
-				SDL_Rect dstrect = {0,ypos,(*i)->w,(*i)->h};
-				SDL_BlitSurface(*i,NULL,surf_,&dstrect);
-				ypos += (*i)->h;
-			}
-		}
+	if (surf_.null()) {
+		surf_ = font::render_text(get_font(font_size_), text_, colour_, 0);
 
 		if(surf_ == NULL) {
 			return NULL;
@@ -723,14 +797,14 @@ SDL_Surface* floating_label::create_surface()
 
 		//if the surface has to be created onto some kind of background, then do that here
 		if(bgalpha_ != 0) {
-			shared_sdl_surface tmp(create_compatible_surface(surf_,surf_->w+border_*2,surf_->h+border_*2));
+			surface tmp(create_compatible_surface(surf_,surf_->w+border_*2,surf_->h+border_*2));
 			if(tmp == NULL) {
 				return NULL;
 			}
 
-			SDL_FillRect(tmp,NULL,SDL_MapRGB(tmp.get()->format,bgcolour_.r,bgcolour_.g,bgcolour_.b));
+			SDL_FillRect(tmp,NULL,SDL_MapRGB(tmp->format,bgcolour_.r,bgcolour_.g,bgcolour_.b));
 			if(bgalpha_ != 255) {
-				tmp.assign(adjust_surface_alpha_add(tmp.get(),bgalpha_ - 255));
+				tmp.assign(adjust_surface_alpha_add(tmp,bgalpha_ - 255));
 				if(tmp == NULL) {
 					return NULL;
 				}
@@ -744,7 +818,7 @@ SDL_Surface* floating_label::create_surface()
 	return surf_;
 }
 
-void floating_label::draw(SDL_Surface* screen)
+void floating_label::draw(surface screen)
 {
 	if(!visible_) {
 		buf_.assign(NULL);
@@ -780,7 +854,7 @@ void floating_label::draw(SDL_Surface* screen)
 	update_rect(rect);
 }
 
-void floating_label::undraw(SDL_Surface* screen)
+void floating_label::undraw(surface screen)
 {
 	if(screen == NULL || buf_ == NULL) {
 		return;
@@ -886,7 +960,7 @@ SDL_Rect get_floating_label_rect(int handle)
 	static const SDL_Rect empty_rect = {0,0,0,0};
 	const label_map::iterator i = labels.find(handle);
 	if(i != labels.end()) {
-		const SDL_Surface* const surf = i->second.create_surface();
+		const surface surf = i->second.create_surface();
 		if(surf != NULL) {
 			SDL_Rect rect = {0,0,surf->w,surf->h};
 			return rect;
@@ -898,7 +972,7 @@ SDL_Rect get_floating_label_rect(int handle)
 
 floating_label_context::floating_label_context()
 {
-	SDL_Surface* const screen = SDL_GetVideoSurface();
+	surface const screen = SDL_GetVideoSurface();
 	if(screen != NULL) {
 		draw_floating_labels(screen);
 	}
@@ -915,13 +989,13 @@ floating_label_context::~floating_label_context()
 
 	label_contexts.pop();
 
-	SDL_Surface* const screen = SDL_GetVideoSurface();
+	surface const screen = SDL_GetVideoSurface();
 	if(screen != NULL) {
 		undraw_floating_labels(screen);
 	}
 }
 
-void draw_floating_labels(SDL_Surface* screen)
+void draw_floating_labels(surface screen)
 {
 	if(label_contexts.empty()) {
 		return;
@@ -938,7 +1012,7 @@ void draw_floating_labels(SDL_Surface* screen)
 	}
 }
 
-void undraw_floating_labels(SDL_Surface* screen)
+void undraw_floating_labels(surface screen)
 {
 	if(label_contexts.empty()) {
 		return;
