@@ -1,4 +1,4 @@
-/* $Id: font.hpp,v 1.35 2004/06/13 21:44:14 Sirp Exp $ */
+/* $Id: font.hpp,v 1.44 2004/10/04 23:34:51 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,6 +14,7 @@
 #define FONT_HPP_INCLUDED
 
 #include "SDL.h"
+#include "SDL_ttf.h"
 
 #include "display.hpp"
 #include "video.hpp"
@@ -30,17 +31,34 @@ struct manager {
 };
 
 //function which sets the name of the font to use
-void set_font(const std::string& name);
+void set_font();
 
 //various standard colours
 extern const SDL_Color NORMAL_COLOUR, GOOD_COLOUR, BAD_COLOUR, BLACK_COLOUR,
-                       DARK_COLOUR, YELLOW_COLOUR, BUTTON_COLOUR;
+                       DARK_COLOUR, YELLOW_COLOUR, BUTTON_COLOUR, TITLE_COLOUR;
 
 enum MARKUP { USE_MARKUP, NO_MARKUP };
 
 //standard markups
 extern const char LARGE_TEXT, SMALL_TEXT, GOOD_TEXT, BAD_TEXT, NORMAL_TEXT, BLACK_TEXT, BOLD_TEXT, IMAGE, NULL_MARKUP;
 
+// font sizes, to be made theme parameters
+#ifdef USE_TINY_GUI
+// this is not meant for normal play, just for checking other dimensions get adapted accordingly
+const int SIZE_NORMAL = 8;
+#else
+const int SIZE_NORMAL = 14;
+#endif
+// automatic computation of other font sizes, to be made a default for theme-provided values
+const int
+  SIZE_TINY	= SIZE_NORMAL * 10 / 14,
+  SIZE_SMALL	= SIZE_NORMAL * 12 / 14,
+
+  SIZE_15	= SIZE_NORMAL * 15 / 14,
+  SIZE_PLUS	= SIZE_NORMAL * 16 / 14,
+  SIZE_LARGE	= SIZE_NORMAL * 18 / 14,
+  SIZE_XLARGE	= SIZE_NORMAL * 24 / 14
+  ;  
 //function to draw text on the screen. The text will be clipped to area.
 //If the text runs outside of area horizontally, an ellipsis will be displayed
 //at the end of it. If use_tooltips is true, then text with an ellipsis will
@@ -61,14 +79,14 @@ extern const char LARGE_TEXT, SMALL_TEXT, GOOD_TEXT, BAD_TEXT, NORMAL_TEXT, BLAC
 
 SDL_Rect draw_text(display* gui, const SDL_Rect& area, int size,
                    const SDL_Color& colour, const std::string& text,
-                   int x, int y, SDL_Surface* bg=NULL,
+                   int x, int y, surface bg=NULL,
                    bool use_tooltips=false, MARKUP use_markup=USE_MARKUP, int style=0);
 
 //function which returns the size of text if it were to be drawn.
 SDL_Rect text_area(const std::string& text, int size, int style=0);
 
 // Returns a SDL surface containing the text rendered in a given colour.
-SDL_Surface* get_rendered_text(const std::string& text, int size, const SDL_Color& colour, int style=0);
+surface get_rendered_text(const std::string& text, int size, const SDL_Color& colour, int style=0);
 	
 // Returns the maximum height of a font, in pixels
 int get_max_height(int size);
@@ -79,7 +97,7 @@ bool is_format_char(char c);
 /// Determine the width of a line of text given a certain font size.
 /// The font type used is the default wesnoth font type.
 ///
-int line_width(const std::string line, int font_size);
+int line_width(const std::string line, int font_size, int style=TTF_STYLE_NORMAL);
 
 ///
 /// If the text exceedes the specified max width, wrap it one a word basis.
@@ -105,7 +123,7 @@ std::string make_text_ellipsis(const std::string& text, int font_size, int max_w
 /// 
 SDL_Rect draw_wrapped_text(display* gui, const SDL_Rect& area, int font_size,
 			     const SDL_Color& colour, const std::string& text,
-			     int x, int y, int max_width, SDL_Surface* bg = NULL);
+			     int x, int y, int max_width, surface bg = NULL);
 
 
 /// structure which will hide all current floating labels, and cause floating labels
@@ -150,8 +168,8 @@ const std::string& get_floating_label_text(int handle);
 
 SDL_Rect get_floating_label_rect(int handle);
 
-void draw_floating_labels(SDL_Surface* screen);
-void undraw_floating_labels(SDL_Surface* screen);
+void draw_floating_labels(surface screen);
+void undraw_floating_labels(surface screen);
 
 }
 
