@@ -1,4 +1,4 @@
-/* $Id: game_config.cpp,v 1.72 2004/06/20 22:45:11 gakusho Exp $ */
+/* $Id: game_config.cpp,v 1.90 2004/09/22 00:05:33 cedricd Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -11,6 +11,7 @@
    See the COPYING file for more details.
 */
 #include "game_config.hpp"
+#include "wesconfig.h"
 
 #include <cstdlib>
 
@@ -26,7 +27,7 @@ namespace game_config
 	int recall_cost = 20;
 	int kill_experience = 8;
 	int leadership_bonus = 25;
-	const std::string version = "0.7.10";
+	const std::string version = VERSION;
 	bool debug = false, editor = false;
 
 	std::string game_icon = "wesnoth-icon.png", game_title, game_logo, title_music;
@@ -37,15 +38,16 @@ namespace game_config
 
 	std::string terrain_mask_image = "terrain/alphamask.png";
 
-	std::string map_image = "misc/map.png";
+	std::string map_image = "maps/wesnoth.png";
 	std::string rightside_image = "misc/rightside.png";
 	std::string rightside_image_bot = "misc/rightside-bottom.png";
 
-	std::string moved_energy_image = "moved-energy.png";
-	std::string unmoved_energy_image = "unmoved-energy.png";
-	std::string partmoved_energy_image = "partmoved-energy.png";
-	std::string enemy_energy_image = "enemy-energy.png";
-	std::string ally_energy_image = "ally-energy.png";
+	std::string moved_energy_image = "misc/bar-energy-moved.png";
+	std::string unmoved_energy_image = "misc/bar-energy-unmoved.png";
+	std::string partmoved_energy_image = "misc/bar-energy-partmoved.png";
+	std::string enemy_energy_image = "misc/bar-energy-enemy.png";
+	std::string ally_energy_image = "misc/bar-energy-ally.png";
+	std::string flag_image = "terrain/flag-team%d-1.png:150,terrain/flag-team%d-2.png:150";
 
 	std::string dot_image = "misc/dot.png";
 	std::string cross_image = "misc/cross.png";
@@ -57,6 +59,8 @@ namespace game_config
 	std::string unchecked_menu_image = "buttons/checkbox.png";
 	std::string checked_menu_image = "buttons/checkbox-pressed.png";
 
+	std::string download_campaign_image;
+
 	namespace sounds {
 		const std::string turn_bell = "bell.wav",
 		                  receive_message = "receive.wav",
@@ -113,6 +117,7 @@ namespace game_config
 		partmoved_energy_image = v["partmoved_energy_image"];
 		enemy_energy_image = v["enemy_energy_image"];
 		ally_energy_image = v["ally_energy_image"];
+		flag_image = v["flag_image"];
 
 		cross_image = v["cross_image"];
 		dot_image = v["dot_image"];
@@ -128,5 +133,7 @@ namespace game_config
 		terrain_mask_image = v["terrain_mask_image"];
 		
 		observer_image = v["observer_image"];
+
+		download_campaign_image = v["download_campaign_image"];
 	}
 }
