@@ -1,4 +1,4 @@
-/* $Id: game_events.cpp,v 1.81 2004/06/17 18:48:35 Sirp Exp $ */
+/* $Id: game_events.cpp,v 1.109 2004/10/03 19:28:43 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -16,6 +16,7 @@
 #include "language.hpp"
 #include "log.hpp"
 #include "playlevel.hpp"
+#include "preferences.hpp"
 #include "replay.hpp"
 #include "show_dialog.hpp"
 #include "sound.hpp"
@@ -29,18 +30,21 @@
 #include <set>
 #include <string>
 
+#define LOG_NG lg::info(lg::engine)
+#define WRN_NG lg::warn(lg::engine)
+#define ERR_NG lg::err(lg::engine)
+#define LOG_DP lg::info(lg::display)
+
 namespace game_events {
 
-bool conditional_passed(game_state& state_of_game,
-                        const std::map<gamemap::location,unit>* units,
+bool conditional_passed(const std::map<gamemap::location,unit>* units,
                         const config& cond)
 {
 	//an 'or' statement means that if the contained statements are true,
 	//then it automatically passes
 	const config::child_list& or_statements = cond.get_children("or");
-	for(config::child_list::const_iterator or_it = or_statements.begin();
-	    or_it != or_statements.end(); ++or_it) {
-		if(conditional_passed(state_of_game,units,**or_it)) {
+	for(config::child_list::const_iterator or_it = or_statements.begin(); or_it != or_statements.end(); ++or_it) {
+		if(conditional_passed(units,**or_it)) {
 			return true;
 		}
 	}
@@ -49,8 +53,7 @@ bool conditional_passed(game_state& state_of_game,
 	//check for that.
 	const config::child_list& have_unit = cond.get_children("have_unit");
 
-	for(config::child_list::const_iterator u = have_unit.begin();
-	    u != have_unit.end(); ++u) {
+	for(config::child_list::const_iterator u = have_unit.begin(); u != have_unit.end(); ++u) {
 
 		if(units == NULL)
 			return false;
@@ -70,8 +73,7 @@ bool conditional_passed(game_state& state_of_game,
 	//check against each variable statement to see if the variable
 	//matches the conditions or not
 	const config::child_list& variables = cond.get_children("variable");
-	for(config::child_list::const_iterator var = variables.begin();
-	    var != variables.end(); ++var) {
+	for(config::child_list::const_iterator var = variables.begin(); var != variables.end(); ++var) {
 		const config& values = **var;
 
 		const std::string& name = values["name"];
@@ -205,8 +207,8 @@ std::multimap<std::string,event_handler> events_map;
 //by an event.
 bool event_handler::handle_event_command(const queued_event& event_info, const std::string& cmd, const config& cfg)
 {
-	log_scope("handle_event_command");
-	std::cerr << "handling command: '" << cmd << "'\n";
+	log_scope2(engine, "handle_event_command");
+	LOG_NG << "handling command: '" << cmd << "'\n";
 
 	bool rval = true;
 	//sub commands that need to be handled in a guaranteed ordering
@@ -277,7 +279,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 	else if(cmd == "unstone") {
 		const config* const filter = cfg.child("filter");
 		for(unit_map::iterator i = units->begin(); i != units->end(); ++i) {
-			if(i->second.stone() && (filter == NULL || i->second.matches_filter(*filter))) {
+			if(i->second.stone() && (filter == NULL || game_events::unit_matches_filter(i,*filter))) {
 				i->second.remove_flag("stone");
 			}
 		}
@@ -295,8 +297,11 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		const std::vector<std::string>& types = config::split(type);
 		for(std::vector<std::string>::const_iterator i = types.begin(); i != types.end(); ++i) {
 			(*teams)[index].recruits().insert(*i);
-			if(index == 0) {
-				state_of_game->can_recruit.insert(*i);
+			preferences::encountered_units().insert(*i);
+
+                        player_info *player=state_of_game->get_player((*teams)[index].save_id());
+                        if(player) {
+                                player->can_recruit.insert(*i);
 			}
 		}
 	}
@@ -312,8 +317,10 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		const std::vector<std::string>& types = config::split(type);
 		for(std::vector<std::string>::const_iterator i = types.begin(); i != types.end(); ++i) {
 			(*teams)[index].recruits().erase(*i);
-			if(index == 0) {
-				state_of_game->can_recruit.erase(*i);
+
+                        player_info *player=state_of_game->get_player((*teams)[index].save_id());
+                        if(player) {
+                                player->can_recruit.erase(*i);
 			}
 		}
 	}
@@ -331,8 +338,10 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		std::set<std::string>& can_recruit = (*teams)[index].recruits();
 		can_recruit.clear();
 		std::copy(recruit.begin(),recruit.end(),std::inserter(can_recruit,can_recruit.end()));
-		if(index == 0) {
-			state_of_game->can_recruit = can_recruit;
+
+                player_info *player=state_of_game->get_player((*teams)[index].save_id());
+                if(player) {
+                        player->can_recruit = can_recruit;
 		}
 	}
 
@@ -391,6 +400,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 
 	//modifications of some attributes of a side: gold, income, team name
 	else if(cmd == "modify_side") {
+		std::cerr << "modifying side...\n";
 		const std::string& side = cfg["side"];
 		const std::string& income = cfg["income"];
 		const std::string& team_name = cfg["team_name"];
@@ -399,7 +409,9 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		const size_t team_index = side_num-1;
 
 		if(team_index < teams->size()) {
+			std::cerr << "modifying team: " << team_index << "\n";
 			if(!team_name.empty()) {
+				std::cerr << "change team to team_name '" << team_name << "'\n";
 				(*teams)[team_index].change_team(team_name);
 			}
 
@@ -434,18 +446,41 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 	//the visual effect
 	else if(cmd == "move_unit_fake") {
 		const std::string& type = cfg["type"];
+		const std::string& side = cfg["side"];
+		int side_num = atoi(side.c_str())-1;
+		if(side_num < 0 || side_num >= teams->size()) side_num = 0;
+
+		const unit_race::GENDER gender = cfg["gender"] == "female" ? unit_race::FEMALE : unit_race::MALE;
 		const game_data::unit_type_map::const_iterator itor = game_data_ptr->unit_types.find(type);
 		if(itor != game_data_ptr->unit_types.end()) {
-			unit dummy_unit(&itor->second,0,false,true);
+			unit dummy_unit(&itor->second,0,false,true,gender);
 			const std::vector<std::string> xvals = config::split(cfg["x"]);
 			const std::vector<std::string> yvals = config::split(cfg["y"]);
 			std::vector<gamemap::location> path;
+			gamemap::location src;
+			gamemap::location dst;
 			for(size_t i = 0; i != minimum(xvals.size(),yvals.size()); ++i) {
-				path.push_back(gamemap::location(atoi(xvals[i].c_str())-1,
-				                                 atoi(yvals[i].c_str())-1));
-			}
+				if(i==0){
+					src.x = atoi(xvals[i].c_str())-1;
+					src.y = atoi(yvals[i].c_str())-1;
+					continue;
+				}
+				shortest_path_calculator calc(dummy_unit,
+						(*teams)[side_num],
+						*units,
+						*teams,
+						*game_map,
+						*status_ptr);
 
-			unit_display::move_unit(*screen,*game_map,path,dummy_unit);
+				dst.x = atoi(xvals[i].c_str())-1;
+				dst.y = atoi(yvals[i].c_str())-1;
+
+				paths::route route=a_star_search(src,dst,10000,calc,0);
+				unit_display::move_unit(*screen, *game_map, route.steps,
+						dummy_unit,status_ptr->get_time_of_day(), *units, *teams);
+
+				src = dst;
+			}
 		}
 	}
 
@@ -538,9 +573,9 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 				}
 			}
 
-			int choice = rand() % num_choices;
+			int choice = get_random() % num_choices;
 			tmp = 0;	
-			for (int i = 0; i < ranges.size(); i++) {
+			for(size_t i = 0; i < ranges.size(); i++) {
 				tmp += (ranges[i].second - ranges[i].first) + 1;
 				if (tmp > choice) {
 					if (ranges[i].first == 0 && ranges[i].second == 0) {
@@ -567,7 +602,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		const std::string fail = (cmd == "if" ? "else" : "");
 		for(size_t i = 0; i != max_iterations; ++i) {
 			const std::string type = game_events::conditional_passed(
-			                              *state_of_game,units,cfg) ? pass : fail;
+			                              units,cfg) ? pass : fail;
 
 			if(type == "") {
 				break;
@@ -588,12 +623,16 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		//get a list of the types this unit can be
 		std::vector<std::string> types = config::split(cfg["type"]);
 
+                std::vector<std::string> sides = config::split(cfg["side"]);
+
 		//iterate over all the types, and for each type, try to find
 		//a unit that matches
 		std::vector<std::string>::iterator ti;
 		for(ti = types.begin(); ti != types.end(); ++ti) {
 			config item = cfg;
 			item["type"] = *ti;
+                        item["side"] = cfg["side"];
+			item["role"] = "";
 
 			std::map<gamemap::location,unit>::iterator itor;
 			for(itor = units->begin(); itor != units->end(); ++itor) {
@@ -606,19 +645,49 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 			if(itor != units->end())
 				break;
 
-			std::vector<unit>::iterator ui;
-			//iterate over the units, and try to find one that matches
-			for(ui = state_of_game->available_units.begin();
-			    ui != state_of_game->available_units.end(); ++ui) {
-				if(ui->matches_filter(item)) {
-					ui->assign_role(cfg["role"]);
-					break;
-				}
+                        bool found = false;
+
+                        if(sides.empty() == false) {
+                          std::vector<std::string>::const_iterator si;
+                          for(si = sides.begin(); si != sides.end(); ++si) {
+                            int side_num = lexical_cast_default<int>(*si,1);
+
+                            player_info* player=state_of_game->get_player((*teams)[side_num-1].save_id());
+
+                            if(!player)
+                              continue;
+
+                            //iterate over the units, and try to find one that matches
+                            std::vector<unit>::iterator ui;
+                            for(ui = player->available_units.begin();
+                                ui != player->available_units.end(); ++ui) {
+                              if(game_events::unit_matches_filter(*ui, item)) {
+                                ui->assign_role(cfg["role"]);
+                                found=true;
+                                break;
+                              }
+                            }
+                          }
+                        } else {
+                          std::map<std::string, player_info>::iterator pi;
+                          for(pi=state_of_game->players.begin();
+                              pi!=state_of_game->players.end(); ++pi) {
+                            std::vector<unit>::iterator ui;
+                            //iterate over the units, and try to find one that matches
+                            for(ui = pi->second.available_units.begin();
+                                ui != pi->second.available_units.end(); ++ui) {
+                              if(game_events::unit_matches_filter(*ui, item)) {
+                                ui->assign_role(cfg["role"]);
+                                found=true;
+                                break;
+                              }
+                            }
+                          }
 			}
 
 			//if we found a unit, we don't have to keep going.
-			if(ui != state_of_game->available_units.end())
-				break;
+                        if(found)
+                          break;
 		}
 	}
 
@@ -678,6 +747,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 
 		for(std::vector<gamemap::location>::const_iterator loc = locs.begin(); loc != locs.end(); ++loc) {
 			const std::string& terrain_type = cfg["letter"];
+			preferences::encountered_terrains().insert(terrain_type);
 			if(terrain_type.size() > 0) {
 				game_map->set_terrain(*loc,terrain_type[0]);
 			}
@@ -690,6 +760,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 	//if we should spawn a new unit on the map somewhere
 	else if(cmd == "unit") {
 		unit new_unit(*game_data_ptr,cfg);
+		preferences::encountered_units().insert(new_unit.type().name());
 		gamemap::location loc(cfg);
 
 		if(game_map->on_board(loc)) {
@@ -701,24 +772,42 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 
 			screen->invalidate(loc);
 		} else {
-			state_of_game->available_units.push_back(new_unit);
+			player_info* const player = state_of_game->get_player((*teams)[new_unit.side()-1].save_id());
+
+			if(player != NULL) {
+				player->available_units.push_back(new_unit);
+			} else {
+				ERR_NG << "Cannot create unit: location is not on the map, and player "
+					<< new_unit.side() << " has no recall list.\n";
+			}
 		}
 	}
 
 	//if we should recall units that match a certain description
 	else if(cmd == "recall") {
-		std::vector<unit>& avail = state_of_game->available_units;
-		for(std::vector<unit>::iterator u = avail.begin(); u != avail.end(); ++u) {
-			if(u->matches_filter(cfg)) {
-				gamemap::location loc(cfg);
-				recruit_unit(*game_map,1,*units,*u,loc,cfg["show"] == "no" ? NULL : screen,false,true);
-				avail.erase(u);
-				break;
+		LOG_NG << "recalling unit...\n";
+		for(int index = 0; index < int(teams->size()); ++index) {
+			LOG_NG << "for side " << index << "...\n";
+			player_info* const player = state_of_game->get_player((*teams)[index].save_id());
+
+			if(player == NULL) {
+				ERR_NG << "player not found!\n";
+				continue;
+			}
+
+			std::vector<unit>& avail = player->available_units;
+
+			for(std::vector<unit>::iterator u = avail.begin(); u != avail.end(); ++u) {
+				LOG_NG << "checking unit against filter...\n";
+				if(game_events::unit_matches_filter(*u,cfg)) {
+					gamemap::location loc(cfg);
+					recruit_unit(*game_map,index+1,*units,*u,loc,cfg["show"] == "no" ? NULL : screen,false,true);
+					avail.erase(u);
+					break;
+				}
 			}
 		}
-	}
-
-	else if(cmd == "object") {
+	} else if(cmd == "object") {
 		const config* filter = cfg.child("filter");
 
 		const std::string& id = cfg["id"];
@@ -737,22 +826,24 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		std::string text;
 
 		gamemap::location loc;
-		bool remove_overlay = false;
 		if(filter != NULL) {
-			loc = gamemap::location(*filter);
+			for(unit_map::const_iterator u = units->begin(); u != units->end(); ++u) {
+				if(game_events::unit_matches_filter(u,*filter)) {
+					loc = u->first;
+					break;
+				}
+			}
 		}
 
 		if(loc.valid() == false) {
-			remove_overlay = true;
 			loc = event_info.loc1;
 		}
 
 		const unit_map::iterator u = units->find(loc);
 
-		if(u == units->end())
-			return rval;
+		std::string command_type = "then";
 
-		if(filter == NULL || u->second.matches_filter(*filter)) {
+		if(u != units->end() && (filter == NULL || game_events::unit_matches_filter(u,*filter))) {
 			const std::string& lang = string_table[id];
 			if(!lang.empty())
 				text = lang;
@@ -761,10 +852,6 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 
 			u->second.add_modification("object",cfg);
 
-			if(remove_overlay) {
-				screen->remove_overlay(event_info.loc1);
-			}
-
 			screen->select_hex(event_info.loc1);
 			screen->invalidate_unit();
 
@@ -776,10 +863,12 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 				text = lang;
 			else
 				text = cfg["cannot_use_message"];
+
+			command_type = "else";
 		}
 
 		if(cfg["silent"] != "yes") {
-			scoped_sdl_surface surface(NULL);
+			surface surface(NULL);
 
 			if(image.empty() == false) {
 				surface.assign(image::get_image(image,image::UNSCALED));
@@ -790,21 +879,26 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 
 			gui::show_dialog(*screen,surface,caption,text);
 		}
+
+		const config::child_list& commands = cfg.get_children(command_type);
+		for(config::child_list::const_iterator cmd = commands.begin();
+		    cmd != commands.end(); ++cmd) {
+			handle_event(event_info,*cmd);
+		}
 	}
 
 	//displaying a message on-screen
 	else if(cmd == "print") {
 		const std::string& text = cfg["text"];
-		const std::string& id = cfg["id"];
-		const int size = lexical_cast_default<int>(cfg["size"],12);
-		const int lifetime = lexical_cast_default<int>(cfg["duration"],20);
+		const int size = lexical_cast_default<int>(cfg["size"],font::SIZE_SMALL);
+		const int lifetime = lexical_cast_default<int>(cfg["duration"],50);
 		const int red = lexical_cast_default<int>(cfg["red"],0);
 		const int green = lexical_cast_default<int>(cfg["green"],0);
 		const int blue = lexical_cast_default<int>(cfg["blue"],0);
 
 		SDL_Color colour = {red,green,blue,255};
 
-		const std::string& msg = translate_string_default(id,text);
+		const std::string& msg = text;
 		if(msg != "") {
 			const SDL_Rect rect = screen->map_area();
 			font::add_floating_label(msg,size,colour,rect.w/2,rect.h/2,
@@ -824,12 +918,19 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 				if(game_events::unit_matches_filter(speaker,cfg))
 					break;
 			}
+		}
+	
+		if(speaker == units->end() && cfg["speaker"] != "narrator") {
+			//no matching unit found, so the dialog can't come up
+			//continue onto the next message
+			WRN_NG << "cannot show message\n";
+			return rval;
+		}
 
-			if(speaker == units->end()) {
-				//no matching unit found, so the dialog can't come up
-				//continue onto the next message
-				return rval;
-			}
+		if(speaker != units->end()) {
+			LOG_NG << "set speaker to '" << speaker->second.description() << "'\n";
+		} else {
+			LOG_NG << "no speaker\n";
 		}
 
 		const std::string& sfx = cfg["sound"];
@@ -849,6 +950,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 			caption = cfg["caption"];
 
 		if(speaker != units->end()) {
+			LOG_DP << "scrolling to speaker..\n";
 			screen->highlight_hex(speaker->first);
 			screen->scroll_to_tile(speaker->first.x,speaker->first.y);
 
@@ -862,6 +964,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 					caption = speaker->second.type().language_name();
 				}
 			}
+			LOG_DP << "done scrolling to speaker...\n";
 		}
 
 		std::vector<std::string> options;
@@ -870,18 +973,20 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		const config::child_list& menu_items = cfg.get_children("option");
 		for(config::child_list::const_iterator mi = menu_items.begin();
 		    mi != menu_items.end(); ++mi) {
-			const std::string& msg = translate_string_default((**mi)["id"],(**mi)["message"]);
+			const std::string& msg = (**mi)["message"];
 			options.push_back(msg);
 			option_events.push_back((*mi)->child_range("command"));
 		}
 
-		scoped_sdl_surface surface(NULL);
+		surface surface(NULL);
 		if(image.empty() == false) {
 			surface.assign(image::get_image(image,image::UNSCALED));
 		}
 
 		const std::string& lang_message = string_table[id];
 		int option_chosen = -1;
+
+		LOG_DP << "showing dialog...\n";
 		
 		//if we're not replaying, or if we are replaying and there is no choice
 		//to be made, show the dialog.
@@ -891,6 +996,8 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		                        options.empty() ? gui::MESSAGE : gui::OK_ONLY,
 		                        options.empty() ? NULL : &options);
 
+			LOG_DP << "showed dialog...\n";
+
 			if (option_chosen == gui::ESCAPE_DIALOG){
 				rval = false;
 			}
@@ -904,7 +1011,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		else {
 			const config* const action = get_replay_source().get_next_action();
 			if(action == NULL || action->get_children("choose").empty()) {
-				std::cerr << "choice expected but none found\n";
+				ERR_NG << "choice expected but none found\n";
 				throw replay::error();
 			}
 
@@ -928,8 +1035,8 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 	}
 
 	else if(cmd == "kill") {
-
-		for(unit_map::iterator un = units->begin(); un != units->end();) {
+		unit_map::iterator un = units->begin();
+		while(un != units->end()) {
 			if(game_events::unit_matches_filter(un,cfg)) {
 				if(cfg["animate"] == "yes") {
 					screen->scroll_to_tile(un->first.x,un->first.y,display::WARP);
@@ -939,6 +1046,11 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 				if(cfg["fire_event"] == "yes") {
 					gamemap::location loc = un->first;
 					game_events::fire("die",loc,un->first);
+					un = units->find(loc);
+					if(un == units->end()) {
+						un = units->begin();
+						continue;
+					}
 				}
 				units->erase(un++);
 			} else {
@@ -947,16 +1059,21 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		}
 
 		//if the filter doesn't contain positional information, then it may match
-		//units on the recall list.
+                //units on all recall lists.
 		if(cfg["x"].empty() && cfg["y"].empty()) {
-			std::vector<unit>& avail_units = state_of_game->available_units;
+                  std::map<std::string, player_info>& players=state_of_game->players;
+
+                  for(std::map<std::string, player_info>::iterator pi = players.begin();
+                      pi!=players.end(); ++pi) {
+                        std::vector<unit>& avail_units = pi->second.available_units;
 			for(std::vector<unit>::iterator j = avail_units.begin(); j != avail_units.end();) {
-				if(j->matches_filter(cfg)) {
-					j = avail_units.erase(j);
-				} else {
-					++j;
-				}
+                          if(game_events::unit_matches_filter(*j, cfg)) {
+                            j = avail_units.erase(j);
+                          } else {
+                            ++j;
+                          }
 			}
+                  }
 		}
 	}
 
@@ -997,24 +1114,29 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		}
 
 		if(filter["x"].empty() && filter["y"].empty()) {
-			std::vector<unit>& avail_units = state_of_game->available_units;
-			for(std::vector<unit>::iterator j = avail_units.begin(); j != avail_units.end();) {
-				if(j->matches_filter(filter) == false) {
-					++j;
-					continue;
-				}
+                  std::map<std::string, player_info>& players = state_of_game->players;
+
+                  for(std::map<std::string, player_info>::iterator pi = players.begin();
+                      pi!=players.end(); ++pi) {
+                    std::vector<unit>& avail_units = pi->second.available_units;
+                    for(std::vector<unit>::iterator j = avail_units.begin(); j != avail_units.end();) {
+                      if(game_events::unit_matches_filter(*j, filter) == false) {
+                        ++j;
+                        continue;
+                      }
 	
-				config& data = vars.add_child(variable);
-				j->write(data);
-				data["x"] = "recall";
-				data["y"] = "recall";
+                      config& data = vars.add_child(variable);
+                      j->write(data);
+                      data["x"] = "recall";
+                      data["y"] = "recall";
 	
-				if(kill_units) {
-					j = avail_units.erase(j);
-				} else {
-					++j;
-				}
-			}
+                      if(kill_units) {
+                        j = avail_units.erase(j);
+                      } else {
+                        ++j;
+                      }
+                    }
+                  }
 		}
 	}
 
@@ -1032,10 +1154,17 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 				units->erase(loc);
 				units->insert(std::pair<gamemap::location,unit>(loc,u));
 			} else {
-				state_of_game->available_units.push_back(u);
+				player_info *player=state_of_game->get_player((*teams)[u.side()-1].save_id());
+
+				if(player) {
+					player->available_units.push_back(u);
+				} else {
+					ERR_NG << "Cannot unstore unit: no recall list for player " << u.side()
+						<< " and the map location is invalid.\n";
+				}
 			}
 		} catch(gamestatus::load_game_failed& e) {
-			std::cerr << "could not de-serialize unit: '" << e.message << "'\n";
+			ERR_NG << "could not de-serialize unit: '" << e.message << "'\n";
 		}
 	}
 
@@ -1062,30 +1191,22 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 
 		const size_t radius = minimum<size_t>(MaxLoop,lexical_cast_default<size_t>(cfg["radius"]));
 		std::set<gamemap::location> res;
-		for(std::vector<gamemap::location>::const_iterator i = locs.begin(); i != locs.end(); ++i) {
-			get_tiles_radius(*i,radius,res);
-		}
+		get_tiles_radius(*game_map, locs, radius, res);
 
 		size_t added = 0;
 		for(std::set<gamemap::location>::const_iterator j = res.begin(); j != res.end() && added != MaxLoop; ++j) {
-			if(game_map->on_board(*j)) {
-				if(terrain.empty() == false) {
-					const gamemap::TERRAIN c = game_map->get_terrain(*j);
-					if(std::find(terrain.begin(),terrain.end(),c) == terrain.end()) {
-						continue;
-					}
-				}
-
-				if(unit_filter != NULL) {
-					const unit_map::const_iterator u = units->find(*j);
-					if(u == units->end() || game_events::unit_matches_filter(u,*unit_filter) == false) {
-						continue;
-					}
-				}
-
-				j->write(state_of_game->variables.add_child(variable));
-				++added;
+			if (terrain.empty() == false) {
+				const gamemap::TERRAIN c = game_map->get_terrain(*j);
+				if(std::find(terrain.begin(), terrain.end(), c) == terrain.end())
+					continue;
 			}
+			if (unit_filter != NULL) {
+				const unit_map::const_iterator u = units->find(*j);
+				if (u == units->end() || !game_events::unit_matches_filter(u, *unit_filter))
+					continue;
+			}
+			j->write(state_of_game->variables.add_child(variable));
+			++added;
 		}
 	}
 
@@ -1125,7 +1246,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		} else if(result == "continue_no_save") {
 			throw end_level_exception(LEVEL_CONTINUE_NO_SAVE);
 		} else {
-			std::cerr << "throwing event defeat...\n";
+			LOG_NG << "throwing event defeat...\n";
 			throw end_level_exception(DEFEAT);
 		}
 	}
@@ -1135,7 +1256,7 @@ bool event_handler::handle_event_command(const queued_event& event_info, const s
 		screen->draw(true);
 	}
 
-	std::cerr << "done handling command...\n";
+	LOG_NG << "done handling command...\n";
 
 	return rval;
 }
@@ -1228,14 +1349,6 @@ bool filter_loc_impl(const gamemap::location& loc, const std::string& xloc,
 
 bool filter_loc(const gamemap::location& loc, const config& cfg)
 {
-	//iterate over any [not] tags, and if any match, then the filter does not match
-	const config::child_list& negatives = cfg.get_children("not");
-	for(config::child_list::const_iterator i = negatives.begin(); i != negatives.end(); ++i) {
-		if(filter_loc(loc,**i)) {
-			return false;
-		}
-	}
-
 	const std::string& xloc = cfg["x"];
 	const std::string& yloc = cfg["y"];
 
@@ -1296,7 +1409,7 @@ void get_variable_internal(const std::string& key, config& cfg,
 			const std::string index_str(index_start+1,index_end);
 			index = size_t(atoi(index_str.c_str()));
 			if(index > MaxLoop) {
-				std::cerr << "ERROR: index greater than 1024: truncated\n";
+				LOG_NG << "get_variable_internal: index greater than 1024, truncated\n";
 				index = MaxLoop;
 			}
 
@@ -1345,9 +1458,34 @@ void get_variable_internal(const std::string& key, config& cfg,
 
 namespace game_events {
 
+bool unit_matches_filter(const unit& u, const config& filter)
+{
+	const bool res = u.matches_filter(filter);
+	if(res == true) {
+		const config::child_list& nots = filter.get_children("not");
+		for(config::child_list::const_iterator i = nots.begin(); i != nots.end(); ++i) {
+			if(unit_matches_filter(u,**i)) {
+				return false;
+			}
+		}
+	}
+
+	return res;
+}
+
 bool unit_matches_filter(unit_map::const_iterator itor, const config& filter)
 {
-	return filter_loc(itor->first,filter) && itor->second.matches_filter(filter);
+	const bool res = filter_loc(itor->first,filter) && itor->second.matches_filter(filter);
+	if(res == true) {
+		const config::child_list& nots = filter.get_children("not");
+		for(config::child_list::const_iterator i = nots.begin(); i != nots.end(); ++i) {
+			if(unit_matches_filter(itor,**i)) {
+				return false;
+			}
+		}
+	}
+
+	return res;
 }
 
 std::string& get_variable(const std::string& key)
@@ -1503,7 +1641,7 @@ bool pump()
 		}
 
 		while(i.first != i.second) {
-			std::cerr << "processing event '" << event_name << "'\n";
+			LOG_NG << "processing event '" << event_name << "'\n";
 			event_handler& handler = i.first->second;
 			if(process_event(handler, ev))
 				result = true;
