@@ -15,6 +15,7 @@
 #include "config.hpp"
 #include "display.hpp"
 #include "font.hpp"
+#include "map.hpp"
 #include "sdl_utils.hpp"
 #include "unit_types.hpp"
 #include "widgets/button.hpp"
@@ -31,10 +32,14 @@
 namespace help {
 
 struct help_manager {
-	help_manager(const config *game_config, game_data *game_info);
+	help_manager(const config *game_config, game_data *game_info, gamemap *map);
 	~help_manager();
 };
 
+/// Generate the help contents from the configurations given to the
+/// manager.
+void generate_contents();
+
 struct section;
 
 typedef std::vector<section *> section_list;
@@ -83,10 +88,17 @@ public:
 	has_id(const std::string &id) : id_(id) {}
 	bool operator()(const topic &t) { return t.id == id_; }
 	bool operator()(const section &s) { return s.id == id_; }
+	bool operator()(const section *s) { return s != NULL && s->id == id_; }
 private:
 	const std::string id_;
 };
 
+/// To be used as a function object when sorting topic lists on the title.
+class title_less {
+public:
+	bool operator()(const topic &t1, const topic &t2) { return t1.title < t2.title; }
+};
+
 struct delete_section {
 	void operator()(section *s) { delete s; }
 };
@@ -177,7 +189,7 @@ private:
 	topic const *chosen_topic_;
 	int internal_width_;
 	visible_item selected_item_;
-	bool clicked_;
+	bool selected_;
 };
 
 /// Thrown when the help system fails to parse something.
@@ -222,16 +234,17 @@ private:
 	/// that should be blitted along with some other information.
 	struct item {
 
-		item(shared_sdl_surface surface, int x, int y, const std::string text="",
+		item(surface surface, int x, int y, const std::string text="",
 			 const std::string reference_to="", bool floating=false,
-			 ALIGNMENT alignment=HERE);
+			 bool box=false, ALIGNMENT alignment=HERE);
 
-		item(shared_sdl_surface surface, int x, int y,
-			 bool floating, ALIGNMENT=HERE);
+		item(surface surface, int x, int y,
+			 bool floating, bool box=false, ALIGNMENT=HERE);
 
 		/// Relative coordinates of this item.
 		SDL_Rect rect;
-		shared_sdl_surface surf;
+
+		surface surf;
 
 		// If this item contains text, this will contain that text.
 		std::string text; 
@@ -243,6 +256,7 @@ private:
 		// If this item is floating, that is, if things should be filled
 		// around it.
 		bool floating;
+		bool box;
 		ALIGNMENT align;
 	};
 	
@@ -293,7 +307,8 @@ private:
 					   SDL_Color color=font::NORMAL_COLOUR);
 
 	/// Add an image item with the specified attributes.
-	void add_img_item(const std::string path, const std::string alignment, const bool floating);
+	void add_img_item(const std::string path, const std::string alignment, const bool floating,
+					  const bool box);
 
 	/// Move the current input point to the next line.
 	void down_one_line();
@@ -345,7 +360,7 @@ private:
 /// A help browser widget.
 class help_browser : public gui::widget {
 public:
-	help_browser(display &disp, section &toplevel);
+	help_browser(display &disp, const section &toplevel);
 
 	// Overloaded from widget so that the layout may be adjusted to fit
 	// the new dimensions.
@@ -402,8 +417,7 @@ enum UNIT_DESCRIPTION_TYPE {FULL_DESCRIPTION, NO_DESCRIPTION, NON_REVEALING_DESC
 UNIT_DESCRIPTION_TYPE description_type(const unit_type &type);
 std::vector<topic> generate_ability_topics();
 std::vector<topic> generate_weapon_special_topics();
-
-
+std::vector<topic> generate_terrains_topics();
 
 /// Parse a help config, return the top level section. Return an empty
 /// section if cfg is NULL.
@@ -412,11 +426,23 @@ section parse_config(const config *cfg);
 void parse_config_internal(const config *help_cfg, const config *section_cfg,
 						   section &sec, int level=0);
 
+/// Return true if the section with id section_id is referenced from
+/// another section in the config, or the toplevel.
+bool section_is_referenced(const std::string &section_id, const config &cfg);
+/// Return true if the topic with id topic_id is referenced from
+/// another section in the config, or the toplevel.
+bool topic_is_referenced(const std::string &topic_id, const config &cfg);
+
 /// Search for the topic with the specified identifier in the section
-/// and it's subsections. Return the found topic, or NULL if none could
+/// and its subsections. Return the found topic, or NULL if none could
 /// be found.
 const topic *find_topic(const section &sec, const std::string &id);
 
+/// Search for the section with the specified identifier in the section
+/// and its subsections. Return the found section or NULL if none could
+/// be found.
+const section *find_section(const section &sec, const std::string &id);
+
 /// Parse a text string. Return a vector with the different parts of the
 /// text. Each markup item is a separate part while the text between
 /// markups are separate parts.
@@ -446,12 +472,29 @@ std::string to_lower(const std::string &s);
 /// Return a copy of s with the first letter capitalized.
 std::string cap(const std::string &s);
 
+/// Prepend all chars with meaning inside attributes with a backslash.
+std::string escape(const std::string &s);
+
 /// Return the first word in s, not removing any spaces in the start of
 /// it.
 std::string get_first_word(const std::string &s);
 
-/// Open a help dialog. The help topic will have the topic with id
-/// show_topic open if it is not the empty string.
+/// Open a help dialog showing the topics with ids topics_to_show and
+/// the sections with ids sections_to_show. Subsections and subtopics of
+/// the sections will be added recursively according to the help config.
+void show_help(display &disp, const std::vector<std::string> &topics_to_show,
+			   const std::vector<std::string> &sections_to_show, const std::string show_topic="",
+			   int xloc=-1, int yloc=-1);
+
+/// Open a help dialog using a toplevel other than the default. This
+/// allows for complete customization of the contents, although not in a
+/// very easy way.
+void show_help(display &disp, const section &toplevel, const std::string show_topic="",
+			   int xloc=-1, int yloc=-1);
+
+/// Open the help browser. The help browser will have the topic with id
+/// show_topic open if it is not the empty string. The default topic
+/// will be shown if show_topic sis the empty string.
 void show_help(display &disp, const std::string show_topic="", int xloc=-1, int yloc=-1);
 
 } // End namespace help.
