@@ -1,4 +1,4 @@
-/* $Id: language.cpp,v 1.31 2004/05/29 05:58:40 jbm Exp $ */
+/* $Id: language.cpp,v 1.52 2004/10/03 13:04:07 cedricd Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -12,21 +12,62 @@
 */
 #include "config.hpp"
 #include "font.hpp"
-#include "hotkeys.hpp"
 #include "language.hpp"
 #include "preferences.hpp"
 #include "util.hpp"
 
 #include <cctype>
+#include <cerrno>
 #include <cstdlib>
 #include <cstring>
 #include <iostream>
 
 namespace {
-	std::string current_language;
+	language_def current_language;
 	string_map strings_;
 }
 
+language_def known_languages[] = {
+ 	language_def("", N_("System default language")),
+	language_def("es_ES", "Castellano"),
+	language_def("ca_ES", "Català"),
+	language_def("cs_CZ", "Čeština"),
+	language_def("da_DK", "Dansk"),
+	language_def("de_DE", "Deutsch"),
+ 	language_def("C", "English"),
+	language_def("eu_ES", "Euskera"),
+ 	language_def("fr_FR", "Français"),
+ 	language_def("el_GR", "Greek"),
+	language_def("hu_HU", "Hungarian"),
+	language_def("it_IT", "Italiano"),
+	language_def("nl_NL", "Nederlands"),
+	language_def("no_NO", "Norsk"),
+	language_def("pl_PL", "Polski"),
+	language_def("pt_BR", "Português do Brasil"),
+	language_def("ru_RU", "Русский (Russian)"),
+	language_def("sk_SK", "Slovenčina"),
+	language_def("fi_FI", "Suomi"),
+	language_def("sv_SE", "Svenska"),
+
+	// end of list marker, do not remove
+	language_def("", "")
+};
+
+std::string languagedef_name (const language_def& def)
+{
+  return def.language;
+}
+
+bool languagedef_lessthan_p (const language_def& def1, const language_def& def2)
+{
+  return (def1.language < def2.language);
+}
+
+bool language_def::operator== (const language_def& a)
+{
+  return ((language == a.language) /* && (localename == a.localename) */ );
+}
+
 symbol_table string_table;
 
 const std::string& symbol_table::operator[](const std::string& key) const
@@ -47,115 +88,104 @@ const std::string& symbol_table::operator[](const char* key) const
 
 const std::string& translate_string(const std::string& str)
 {
-	return translate_string_default(str,str);
+	return str;
 }
 
 const std::string& translate_string_default(const std::string& str, const std::string& default_val)
 {
-	const string_map::const_iterator i = strings_.find(str);
-	if(i != strings_.end() && i->second != "") {
-		return i->second;
-	} else {
-		//if there are spaces, try searching for a no-space version
-		if(std::find(str.begin(),str.end(),' ') != str.end()) {
-			std::string str_copy = str;
-			str_copy.erase(std::remove(str_copy.begin(),str_copy.end(),' '),str_copy.end());
-			return translate_string_default(str_copy,default_val);
-		}
-
-		return default_val;
-	}
+	return str;
 }
 
-std::vector<std::string> get_languages()
+std::vector<language_def> get_languages()
 {
-	std::vector<std::string> res;
+	std::vector<language_def> res;
 
-	config cfg;
-
-	try {
-		cfg.read(preprocess_file("data/translations/",NULL,NULL));
-	} catch(config::error& e) {
-		std::cerr << "could not open translations: '" << e.message << "' -- defaulting to English only\n";
-		res.push_back("English");
-		return res;
-	}
-
-	const config::child_list& lang = cfg.get_children("language");
-	for(config::child_list::const_iterator i = lang.begin(); i != lang.end(); ++i) {
-		res.push_back((**i)["language"]);
+	for(int i = 0; known_languages[i].language[0] != '\0'; i++) {
+		res.push_back(known_languages[i]);
 	}
 
 	return res;
 }
 
-namespace {
-bool internal_set_language(const std::string& locale, config& cfg)
+char* wesnoth_setlocale(int category, const char *locale)
 {
-	const config::child_list& lang = cfg.get_children("language");
-	for(config::child_list::const_iterator i = lang.begin(); i != lang.end(); ++i) {
-		if((**i)["id"] == locale || (**i)["language"] == locale) {
+#ifdef __BEOS__
+	if(setenv ("LANG", locale, 1) == -1)
+		std::cerr << "setenv LANG failed: " << strerror(errno);
+	if(setenv ("LC_ALL", locale, 1) == -1)
+		std::cerr << "setenv LC_ALL failed: " << strerror(errno);
+#endif
+#ifdef __APPLE__
+	if(setenv ("LANGUAGE", locale, 1) == -1)
+		std::cerr << "setenv LANGUAGE failed: " << strerror(errno);
+	if(setenv ("LC_ALL", locale, 1) == -1)
+		std::cerr << "setenv LC_ALL failed: " << strerror(errno);
+#endif 
 
-			current_language = (**i)["language"];
-
-			for(string_map::const_iterator j = (*i)->values.begin(); j != (*i)->values.end(); ++j) {
-				strings_[j->first] = j->second;
-			}
-
-			hotkey::add_hotkeys(**i,false);
-
-			font::set_font((**i)["font"]);
-
-			return true;
-		}
-	}
-
-	return false;
-}
+	char* res = setlocale (category, locale);
+	if (res == NULL)
+		std::cerr << "WARNING: setlocale() failed for "
+			  << locale << ".\n";
+	return res;
 }
 
-bool set_language(const std::string& locale)
+bool set_language(const language_def& locale)
 {
 	strings_.clear();
 
 	std::string locale_lc;
-	locale_lc.resize(locale.size());
-	std::transform(locale.begin(),locale.end(),locale_lc.begin(),tolower);
+	locale_lc.resize(locale.localename.size());
+	std::transform(locale.localename.begin(),locale.localename.end(),locale_lc.begin(),tolower);
 
 	config cfg;
-	if(locale_lc == "en" || locale_lc == "english") {
-		try {
-			cfg.read(preprocess_file("data/translations/english.cfg"));
-		} catch(config::error& e) {
-			std::cerr << "Could not read english.cfg\n";
-			throw e;
-		}
-	} else {
-		try {
-			cfg.read(preprocess_file("data/translations/"));
-			
-			//default to English locale first, then set desired locale
-			internal_set_language("en",cfg);
-		} catch(config::error& e) {
-			std::cerr << "error opening translations: '" << e.message << "' Defaulting to English\n";
-			return set_language("english");
-		}
+
+	current_language = locale;
+	wesnoth_setlocale (LC_MESSAGES, locale.localename.c_str());
+	font::set_font();
+
+	// fill string_table (should be moved somwhere else some day)
+	try {
+		cfg.read(preprocess_file("data/translations/english.cfg"));
+	} catch(config::error& e) {
+		std::cerr << "Could not read english.cfg\n";
+		throw e;
 	}
 
-	return internal_set_language(locale,cfg);
+	config* langp = cfg.child("language");
+	if (langp == NULL) {
+	 	std::cerr << "No [language] block found in english.cfg";	
+		return false;
+	}
+
+	for(string_map::const_iterator j = langp->values.begin(); j != langp->values.end(); ++j) {
+		strings_[j->first] = j->second;
+	}
+	// end of string_table fill
+
+	return true;
 }
 
-const std::string& get_language() { return current_language; }
+const language_def& get_language() { return current_language; }
 
-std::string get_locale()
+const language_def& get_locale()
 {
 	//TODO: Add in support for querying the locale on Windows
 
 	const std::string& prefs_locale = preferences::locale();
 	if(prefs_locale.empty() == false) {
-		return prefs_locale;
+		char* setlocaleres = wesnoth_setlocale (LC_MESSAGES, prefs_locale.c_str());
+		if(setlocaleres == NULL)
+			std::cerr << "call to setlocale() failed for " << prefs_locale.c_str() << "\n";
+		for(int i = 0; known_languages[i].language[0] != '\0'; i++) {
+		  	if (prefs_locale == known_languages[i].localename)
+			  	return known_languages[i];
+		}
+		
+		std::cerr << "setlocale succeeded but locale not found in known array; defaulting to system locale\n";
+		return known_languages[0];
 	}
 
+#if 0
 	const char* const locale = getenv("LANG");
 	if(locale != NULL && strlen(locale) >= 2) {
 		//we can't pass pointers into the string to the std::string
@@ -166,9 +196,10 @@ std::string get_locale()
 		res[1] = tolower(locale[1]);
 		return res;
 	}
+#endif
 
-	std::cerr << "locale could not be determined; defaulting to locale 'en'\n";
-	return "en";
+	std::cerr << "locale could not be determined; defaulting to system locale\n";
+	return known_languages[0];
 }
 
 class invalid_utf8_exception : public std::exception {
@@ -252,31 +283,30 @@ int byte_size_from_utf8_first(unsigned char ch)
 wide_string utf8_to_wstring(const std::string &src)
 {
 	wide_string ret;	
-	wchar_t ch;
-	std::string::const_iterator i;
 	
 	try {
-		for(i = src.begin(); i != src.end(); ++i ) {
-			ch = (unsigned char)*i;
+		for(size_t i = 0, l = src.size(); i != l;) {
+			wchar_t ch = (unsigned char)src[i];
 			const int count = byte_size_from_utf8_first(ch);
-					
-			if(i + count - 1 == src.end())
+
+			if (i + count > l)
 				throw invalid_utf8_exception();
 
 			/* Convert the first character */
 			if (count != 1) {
 				ch &= 0xFF >> (count + 1);
 			}
-			
+
 			/* Convert the continuation bytes */
-			for(std::string::const_iterator j = i+1; j != i+count; ++j) {
-				if((*j & 0xC0) != 0x80)
+			for(size_t j = i + 1; j != i + count; ++j) {
+				unsigned char ch2 = src[j];
+				if ((ch2 & 0xC0) != 0x80)
 					throw invalid_utf8_exception();
-				
-				ch = (ch << 6) | (*j & 0x3F);
+
+				ch = (ch << 6) | (ch2 & 0x3F);
 			}
-			i += (count - 1);
-			
+			i += count;
+
 			push_back(ret,ch);
 		}
 	}
@@ -328,5 +358,3 @@ wide_string string_to_wstring(const std::string &src)
 {
 	return utf8_to_wstring(src);
 }
-
-
