@@ -1,4 +1,4 @@
-/* $Id: map.cpp,v 1.35 2004/06/10 23:43:25 Sirp Exp $ */
+/* $Id: map.cpp,v 1.40 2004/08/05 21:53:31 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -132,21 +132,6 @@ void gamemap::location::write(config& cfg) const
 	cfg["y"] = buf;
 }
 
-bool gamemap::location::operator==(const gamemap::location& a) const
-{
-	return x == a.x && y == a.y;
-}
-
-bool gamemap::location::operator!=(const gamemap::location& a) const
-{
-	return !operator==(a);
-}
-
-bool gamemap::location::operator<(const gamemap::location& a) const
-{
-	return x < a.x || x == a.x && y < a.y;
-}
-
 gamemap::location gamemap::location::operator-() const
 {
 	location ret;
@@ -287,7 +272,7 @@ std::string gamemap::write() const
 }
 
 int gamemap::x() const { return tiles_.size(); }
-int gamemap::y() const { return tiles_[0].size(); }
+int gamemap::y() const { return tiles_.empty() ? 0 : tiles_.front().size(); }
 
 const std::vector<gamemap::TERRAIN>& gamemap::operator[](int index) const
 {
@@ -407,6 +392,12 @@ void gamemap::set_terrain(const gamemap::location& loc, gamemap::TERRAIN ter)
 		return;
 
 	tiles_[loc.x][loc.y] = ter;
+
+	location adj[6];
+	get_adjacent_tiles(loc,adj);
+
+	for(int i = 0; i < 6; ++i) 
+		remove_from_border_cache(adj[i]);
 }
 
 std::vector<gamemap::location> parse_location_range(const std::string& x, const std::string& y)
