@@ -1,4 +1,4 @@
-/* $Id: map.hpp,v 1.25 2004/06/10 00:35:44 Sirp Exp $ */
+/* $Id: map.hpp,v 1.27 2004/08/07 17:21:58 Sirp Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -65,9 +65,11 @@ public:
 
 		int x, y;
 
-		bool operator<(const location& a) const;
-		bool operator==(const location& a) const;
-		bool operator!=(const location& a) const;
+		// Inlining those for performance reasons
+		bool operator<(const location& a) const { return x < a.x || x == a.x && y < a.y; }
+		bool operator==(const location& a) const { return x == a.x && y == a.y; }
+		bool operator!=(const location& a) const { return !operator==(a); }
+
 		// Adds an absolute location to a "delta" location
 		location operator-() const;
 		location operator+(const location &a) const;
@@ -80,6 +82,9 @@ public:
 		static location null_location;
 	};
 
+	const std::string& underlying_terrain(const location& loc) const
+	{ return underlying_terrain(get_terrain(loc)); }
+
 	bool is_village(TERRAIN terrain) const;
 	bool gives_healing(TERRAIN terrain) const;
 	bool is_castle(TERRAIN terrain) const;
