@@ -33,9 +33,8 @@ void map_labels::read(const config& cfg)
 	const config::child_list& items = cfg.get_children("label");
 	for(config::child_list::const_iterator i = items.begin(); i != items.end(); ++i) {
 		const gamemap::location loc(**i);
-		const std::string& id = (**i)["id"];
 		const std::string& text = (**i)["text"];
-		set_label(loc,translate_string_default(id,text));
+		set_label(loc,text);
 	}
 }
 
@@ -54,8 +53,14 @@ const std::string& map_labels::get_label(const gamemap::location& loc) const
 	}
 }
 
-void map_labels::set_label(const gamemap::location& loc, const std::string& text)
+void map_labels::set_label(const gamemap::location& loc, const std::string& str)
 {
+	const size_t max_label_size = 32;
+	std::string text = str;
+	if(text.size() > max_label_size) {
+		text.resize(max_label_size);
+	}
+
 	const label_map::iterator current_label = labels_.find(loc);
 	if(current_label != labels_.end()) {
 		font::remove_floating_label(current_label->second);
@@ -77,8 +82,8 @@ void map_labels::set_label(const gamemap::location& loc, const std::string& text
 	const gamemap::location loc_nextx(loc.x+1,loc.y);
 	const gamemap::location loc_nexty(loc.x,loc.y+1);
 	const int xloc = (disp_.get_location_x(loc) + disp_.get_location_x(loc_nextx)*2)/3;
-	const int yloc = disp_.get_location_y(loc_nexty) - 14;
-	const int handle = font::add_floating_label(text,14,colour,xloc,yloc,0,0,-1,disp_.map_area());
+	const int yloc = disp_.get_location_y(loc_nexty) - font::SIZE_NORMAL;
+	const int handle = font::add_floating_label(text,font::SIZE_NORMAL,colour,xloc,yloc,0,0,-1,disp_.map_area());
 
 	labels_.insert(std::pair<gamemap::location,int>(loc,handle));
 
