@@ -1,4 +1,4 @@
-/* $Id: multiplayer.cpp,v 1.107 2004/06/08 22:36:36 Sirp Exp $ */
+/* $Id: multiplayer.cpp,v 1.120 2004/09/25 19:46:33 ydirson Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -58,15 +58,18 @@ multiplayer_game_setup_dialog::multiplayer_game_setup_dialog(
                               const config& cfg, game_state& state, bool server)
         : disp_(disp), units_data_(units_data), cfg_(cfg), state_(state), server_(server), level_(NULL), map_selection_(-1),
 		  maps_menu_(NULL), turns_slider_(NULL), village_gold_slider_(NULL), xp_modifier_slider_(NULL),
-		  fog_game_(NULL), shroud_game_(NULL), observers_game_(NULL), vision_combo_(NULL),
+		  fog_game_(NULL), shroud_game_(NULL), observers_game_(NULL),
           cancel_game_(NULL), launch_game_(NULL), regenerate_map_(NULL), generator_settings_(NULL),
-		  era_combo_(NULL), name_entry_(NULL), generator_(NULL)
+		  era_combo_(NULL), vision_combo_(NULL), name_entry_(NULL), generator_(NULL)
 {
 	std::cerr << "setup dialog ctor\n";
 
-	state_.available_units.clear();
+        for(std::map<std::string, player_info>::iterator i=state_.players.begin();
+            i!=state_.players.end(); ++i) {
+          i->second.available_units.clear();
+          i->second.can_recruit.clear();
+        }
 	state_.variables.clear();
-	state_.can_recruit.clear();
 
 	//build the list of scenarios to play
 	get_files_in_dir(get_user_data_dir() + "/editor/maps",&user_maps_,NULL,FILE_NAME_ONLY);
@@ -75,17 +78,11 @@ multiplayer_game_setup_dialog::multiplayer_game_setup_dialog(
 
 	const config::child_list& levels = cfg.get_children("multiplayer");
 	for(config::child_list::const_iterator i = levels.begin(); i != levels.end(); ++i){
-		const std::string& id = (**i)["id"];
-
-		const std::string& lang_name = string_table[id];
-		if(lang_name.empty() == false)
-			options.push_back(lang_name);
-		else
-			options.push_back((**i)["name"]);
+		options.push_back((**i)["name"]);
 	}
 
 	//add the 'load game' option
-	options.push_back(string_table["load_game"] + "...");
+	options.push_back(_("Load Game") + std::string("..."));
 
 	//create the scenarios menu
 	maps_menu_.assign(new gui::menu(disp_,options));
@@ -97,55 +94,55 @@ multiplayer_game_setup_dialog::multiplayer_game_setup_dialog(
 	turns_slider_->set_min(20);
 	turns_slider_->set_max(100);
 	turns_slider_->set_value(50);
-	turns_slider_->set_help_string(string_table["help_string_turns_slider"]);
+	turns_slider_->set_help_string(_("The maximum turns the game will go for"));
 
 	village_gold_slider_.assign(new gui::slider(disp_,rect));
 	village_gold_slider_->set_min(1);
 	village_gold_slider_->set_max(5);
 	village_gold_slider_->set_value(1);
-	village_gold_slider_->set_help_string(string_table["help_string_village_gold_slider"]);
+	village_gold_slider_->set_help_string(_("The amount of income each village yields per turn"));
 
 	xp_modifier_slider_.assign(new gui::slider(disp_,rect));
 	xp_modifier_slider_->set_min(25);
 	xp_modifier_slider_->set_max(200);
 	xp_modifier_slider_->set_value(100);
-	xp_modifier_slider_->set_help_string(string_table["help_string_xp_modifier_slider"]);
+	xp_modifier_slider_->set_help_string(_("The amount of experience a unit needs to advance"));
 
-	fog_game_.assign(new gui::button(disp_,string_table["fog_of_war"],gui::button::TYPE_CHECK));
+	fog_game_.assign(new gui::button(disp_,_("Fog Of War"),gui::button::TYPE_CHECK));
 	fog_game_->set_check(false);
-	fog_game_->set_help_string(string_table["help_string_fog_of_war"]);
+	fog_game_->set_help_string(_("Enemy units cannot be seen unless they are in range of your units"));
 
-	shroud_game_.assign(new gui::button(disp_,string_table["shroud"],gui::button::TYPE_CHECK));
+	shroud_game_.assign(new gui::button(disp_,_("Shroud"),gui::button::TYPE_CHECK));
 	shroud_game_->set_check(false);
-	shroud_game_->set_help_string(string_table["help_string_shroud"]);
+	shroud_game_->set_help_string(_("The map is unknown until your units explore it"));
 
-	observers_game_.assign(new gui::button(disp_,string_table["observers"],gui::button::TYPE_CHECK));
+	observers_game_.assign(new gui::button(disp_,_("Observers"),gui::button::TYPE_CHECK));
 	observers_game_->set_check(true);
-	observers_game_->set_help_string(string_table["help_string_observers"]);
+	observers_game_->set_help_string(_("Allow users who are not playing to watch the game"));
 
-	cancel_game_.assign(new gui::button(disp_,string_table["cancel_button"]));
-	launch_game_.assign(new gui::button(disp_,string_table["ok_button"]));
+	cancel_game_.assign(new gui::button(disp_,_("Cancel")));
+	launch_game_.assign(new gui::button(disp_,_("Ok")));
 
-	regenerate_map_.assign(new gui::button(disp_,string_table["regenerate_map"]));
+	regenerate_map_.assign(new gui::button(disp_,_("Regenerate")));
 
-	generator_settings_.assign(new gui::button(disp_,string_table["generator_settings"]));
+	generator_settings_.assign(new gui::button(disp_,_("Settings...")));
 
 	//The possible vision settings
 	std::vector<std::string> vision_types;
-	vision_types.push_back(string_table["share_view"]);
-	vision_types.push_back(string_table["share_maps"]);
-	vision_types.push_back(string_table["share_none"]);
+	vision_types.push_back(_("Share View"));
+	vision_types.push_back(_("Share Maps"));
+	vision_types.push_back(_("Share None"));
 	vision_combo_.assign(new gui::combo(disp_,vision_types));
 
 	//the possible eras to play
 	const config::child_list& era_list = cfg.get_children("era");
 	std::vector<std::string> eras;
 	for(config::child_list::const_iterator er = era_list.begin(); er != era_list.end(); ++er) {
-		eras.push_back(translate_string_default((**er)["id"],(**er)["name"]));
+		eras.push_back((**er)["name"]);
 	}
 
 	if(eras.empty()) {
-		gui::show_dialog(disp_,NULL,"",string_table["error_no_mp_sides"],gui::OK_ONLY);
+		gui::show_dialog(disp_,NULL,"",_("No multiplayer sides."),gui::OK_ONLY);
 		std::cerr << "ERROR: no eras found\n";
 		throw config::error("no eras found");
 	}
@@ -168,17 +165,20 @@ void multiplayer_game_setup_dialog::set_area(const SDL_Rect& area)
 	int height = int(area.h);
 	const int left = area.x;
 	const int top = area.y;
-	const int border_size = 5;
+	const int border_size = 6;
 	const int right = left + width;
 	const int bottom = top + height;
 
 	int xpos = left + border_size;
-	int ypos = top + gui::draw_dialog_title(left,top,&disp_,string_table["create_new_game"]).h + border_size;
+	int ypos = top + gui::draw_dialog_title(left,top,&disp_,_("Create Game")).h + border_size;
 
 	//Name Entry
-	ypos += font::draw_text(&disp_,disp_.screen_area(),12,font::GOOD_COLOUR,
-	                        string_table["name_of_game"] + ":",xpos,ypos).h + border_size;
-	name_entry_.assign(new gui::textbox(disp_,width-20,string_table["game_prefix"] + preferences::login() + string_table["game_postfix"]));
+	ypos += font::draw_text(&disp_,disp_.screen_area(),font::SIZE_SMALL,font::GOOD_COLOUR,
+	                        _("Name of game") + std::string(":"),xpos,ypos).h + border_size;
+	string_map i18n_symbols;
+	i18n_symbols["login"] = preferences::login();
+	name_entry_.assign(new gui::textbox(disp_,width-20,
+		vgettext("$login's game", i18n_symbols)));
 	name_entry_->set_location(xpos,ypos);
 	name_entry_->set_dirty();
 
@@ -188,8 +188,8 @@ void multiplayer_game_setup_dialog::set_area(const SDL_Rect& area)
 
 	//the map selection menu goes near the middle of the dialog, to the right of
 	//the minimap
-	const int map_label_height = font::draw_text(&disp_,disp_.screen_area(),12,font::GOOD_COLOUR,
-	                                             string_table["map_to_play"] + ":",xpos + minimap_width + border_size,ypos).h;
+	const int map_label_height = font::draw_text(&disp_,disp_.screen_area(),font::SIZE_SMALL,font::GOOD_COLOUR,
+	                                             _("Map to play") + std::string(":"),xpos + minimap_width + border_size,ypos).h;
 
 	maps_menu_->set_loc(xpos + minimap_width + border_size,ypos + map_label_height + border_size);
 	maps_menu_->set_dirty();
@@ -197,46 +197,46 @@ void multiplayer_game_setup_dialog::set_area(const SDL_Rect& area)
 	SDL_Rect rect;
 
 	//the sliders and other options on the right side of the dialog
-	rect.x = xpos + minimap_width + maps_menu_->width() + border_size*2;
+	rect.x = xpos + minimap_width + maps_menu_->width() + border_size;
 	rect.y = ypos;
 	rect.w = maximum<int>(0,right - border_size - rect.x);
 	//a font sized "12" isn't necessarily 12 pixel high.
-	rect.h = font::get_max_height(12);
+	rect.h = font::get_max_height(font::SIZE_SMALL);
 
 	turns_restorer_ = surface_restorer(&disp_.video(),rect);
 
-	rect.y += rect.h + border_size*2;
+	rect.y += rect.h + border_size + 1;
 
 	turns_slider_->set_location(rect);
 	turns_slider_->set_dirty();
 
 	//Village Gold
-	rect.y += rect.h + border_size*2;
+	rect.y += rect.h + border_size + 1;
 
 	village_gold_restorer_ = surface_restorer(&disp_.video(),rect);
 
-	rect.y += rect.h + border_size*2;
+	rect.y += rect.h + border_size + 1;
 
 	village_gold_slider_->set_location(rect);
 	village_gold_slider_->set_dirty();
 
 	//Experience Modifier
-	rect.y += rect.h + border_size*2;
+	rect.y += rect.h + border_size + 1;
 
 	xp_restorer_ = surface_restorer(&disp_.video(),rect);
 
-	rect.y += rect.h + border_size*2;
+	rect.y += rect.h + border_size + 1;
 
 	xp_modifier_slider_->set_location(rect);
 	xp_modifier_slider_->set_dirty();
 
 	//FOG of war
-	rect.y += rect.h + border_size*2;
+	rect.y += rect.h + border_size + 1;
 
 	fog_game_->set_location(rect.x,rect.y);
 	fog_game_->set_dirty();
 
-	rect.y += fog_game_->location().h + border_size;
+	rect.y += fog_game_->location().h + border_size + 1;
 
 	//Shroud
 	shroud_game_->set_location(rect.x,rect.y);
@@ -256,11 +256,16 @@ void multiplayer_game_setup_dialog::set_area(const SDL_Rect& area)
 
 	rect.y += vision_combo_->height() + border_size;
 
+	gui::button* left_button = launch_game_;
+	gui::button* right_button = cancel_game_;
+
+#ifdef OK_BUTTON_ON_RIGHT
+	std::swap(left_button,right_button);
+#endif
+
 	//Buttons
-	cancel_game_->set_location(right - cancel_game_->width() - gui::ButtonHPadding,
-	                     bottom - cancel_game_->height() - gui::ButtonVPadding);
-	launch_game_->set_location(right - cancel_game_->width() - launch_game_->width() - gui::ButtonHPadding*2,
-	                     bottom - launch_game_->height() - gui::ButtonVPadding);
+	right_button->set_location(right - right_button->width() - gui::ButtonHPadding,bottom - right_button->height() - gui::ButtonVPadding);
+	left_button->set_location(right - right_button->width() - left_button->width() - gui::ButtonHPadding*2,bottom - left_button->height() - gui::ButtonVPadding);
 
 	cancel_game_->set_dirty();
 	launch_game_->set_dirty();
@@ -281,7 +286,7 @@ void multiplayer_game_setup_dialog::set_area(const SDL_Rect& area)
 	playernum_restorer_ = surface_restorer(&disp_.video(),player_num_rect);
 
 	SDL_Rect era_rect = {xpos,player_num_rect.y+player_num_rect.h + border_size,50,20};
-	era_rect = font::draw_text(&disp_,era_rect,12,font::GOOD_COLOUR,translate_string("Era") + ":",
+	era_rect = font::draw_text(&disp_,era_rect,font::SIZE_SMALL,font::GOOD_COLOUR,_("Era") + std::string(":"),
 	                           era_rect.x,era_rect.y);
 	
 	era_combo_->set_location(era_rect.x+era_rect.w+border_size,era_rect.y);
@@ -343,19 +348,19 @@ lobby::RESULT multiplayer_game_setup_dialog::process()
 		sprintf(buf,"%d", cur_turns);
 		turns_str = buf;
 	} else {
-		turns_str = string_table["unlimited"];
+		turns_str = _("Unlimited");
 	}
 
-	font::draw_text(&disp_,disp_.screen_area(),12,font::GOOD_COLOUR,
-	                string_table["turns"] + ": " + turns_str,
+	font::draw_text(&disp_,disp_.screen_area(),font::SIZE_SMALL,font::GOOD_COLOUR,
+	                _("Turns") + std::string(": ") + turns_str,
 					turns_restorer_.area().x,turns_restorer_.area().y);
 
 	//Villages can produce between 1 and 10 gold a turn
 	const int village_gold = village_gold_slider_->value();
 	village_gold_restorer_.restore();
 	sprintf(buf,": %d", village_gold);
-	font::draw_text(&disp_,disp_.screen_area(),12,font::GOOD_COLOUR,
-	                string_table["village_gold"] + buf,
+	font::draw_text(&disp_,disp_.screen_area(),font::SIZE_SMALL,font::GOOD_COLOUR,
+	                _("Village Gold") + std::string(buf),
 					village_gold_restorer_.area().x,village_gold_restorer_.area().y);
 
 
@@ -366,8 +371,8 @@ lobby::RESULT multiplayer_game_setup_dialog::process()
 	sprintf(buf,": %d%%", xpmod);
 
 	const SDL_Rect& xp_rect = xp_restorer_.area();
-	font::draw_text(&disp_,disp_.screen_area(),12,font::GOOD_COLOUR,
-	                string_table["xp_modifier"] + buf,xp_rect.x,xp_rect.y);
+	font::draw_text(&disp_,disp_.screen_area(),font::SIZE_SMALL,font::GOOD_COLOUR,
+	                _("Experience Requirements") + std::string(buf),xp_rect.x,xp_rect.y);
 
 	bool map_changed = map_selection_ != maps_menu_->selection();
 	map_selection_ = maps_menu_->selection();
@@ -411,7 +416,7 @@ lobby::RESULT multiplayer_game_setup_dialog::process()
 			playernum_restorer_.restore();
 			minimap_restorer_.restore();
 			const SDL_Rect& player_num_rect = playernum_restorer_.area();
-			font::draw_text(&disp_,disp_.screen_area(),12,font::GOOD_COLOUR,
+			font::draw_text(&disp_,disp_.screen_area(),font::SIZE_SMALL,font::GOOD_COLOUR,
 				                " Load Map ",player_num_rect.x,player_num_rect.y);
 		}
 	}
@@ -462,7 +467,7 @@ lobby::RESULT multiplayer_game_setup_dialog::process()
 		}
 
 		SDL_Rect rect = minimap_restorer_.area();
-		const scoped_sdl_surface mini(image::getMinimap(rect.w,rect.h,map,0));
+		const surface mini(image::getMinimap(rect.w,rect.h,map,0));
 
 		if(mini != NULL) {
 			SDL_BlitSurface(mini, NULL, disp_.video().getSurface(), &rect);
@@ -476,8 +481,8 @@ lobby::RESULT multiplayer_game_setup_dialog::process()
 			const int nsides = level_->get_children("side").size();
 
 			std::stringstream players;
-			players << string_table["num_players"] << ": " << nsides;
-			font::draw_text(&disp_,disp_.screen_area(),12,font::GOOD_COLOUR,
+			players << _("Players") << ": " << nsides;
+			font::draw_text(&disp_,disp_.screen_area(),font::SIZE_SMALL,font::GOOD_COLOUR,
 			                players.str(),players_rect.x,players_rect.y);
 		}
 	}
@@ -487,15 +492,27 @@ lobby::RESULT multiplayer_game_setup_dialog::process()
 
 void multiplayer_game_setup_dialog::start_game()
 {
+	std::cerr << "calling start_game()\n";
+	const network::manager net_manager;
+	const network::server_manager server_man(15000,server_ ? network::server_manager::TRY_CREATE_SERVER : network::server_manager::NO_SERVER);
+
+	if(server_ && server_man.is_running() == false) {
+		gui::show_dialog(disp_,NULL,_("Warning"),_("The game was unable to bind to the port needed to host games over the network. Network players will be unable to connect to this game"),
+		                 gui::OK_ONLY);
+	}
+
 	turns_restorer_ = surface_restorer();
 	village_gold_restorer_ = surface_restorer();
 	xp_restorer_ = surface_restorer();
 	playernum_restorer_ = surface_restorer();
 	minimap_restorer_ = surface_restorer();
 
+	std::cerr << "loading connector...\n";
 	//Connector
 	mp_connect connector(disp_, name_entry_->text(), cfg_, units_data_, state_);
 
+	std::cerr << "done loading connector...\n";
+
 	const int turns = turns_slider_->value() < turns_slider_->max_value() ?
 		turns_slider_->value() : -1;
 
@@ -511,8 +528,6 @@ void multiplayer_game_setup_dialog::start_game()
 		return;
 	}
 
-	const network::manager net_manager;
-	const network::server_manager server_man(15000,server_);
 	name_entry_->set_focus(false);
 
 	//free up widget resources so they're not consumed while the game is on
