@@ -39,7 +39,7 @@ private:
 void check_response(network::connection res, const config& data)
 {
 	if(!res) {
-		throw network::error(string_table["connection_timeout"]);
+		throw network::error(_("Connection timed out"));
 	}
 
 	const config* err = data.child("error");
@@ -56,7 +56,7 @@ void receive_gamelist(display& disp, config& data)
 			break;
 		}
 
-		const network::connection res = gui::network_data_dialog(disp,string_table["receive_game_list"],data);
+		const network::connection res = gui::network_data_dialog(disp,_("Receiving game list..."),data);
 		check_response(res,data);
 	}
 }
@@ -64,13 +64,17 @@ void receive_gamelist(display& disp, config& data)
 class wait_for_start : public lobby::dialog
 {
 public:
-	wait_for_start(display& disp, config& cfg, int team_num, const std::string& team_name) : got_side(false), team(team_num), name(team_name), status(START_GAME), disp_(disp), cancel_button_(NULL), menu_(NULL), sides_(cfg)
+	wait_for_start(display& disp, config& cfg, const game_data& data, int team_num,
+			const std::string& team_name, const std::string& team_leader)
+		: status(START_GAME), got_side(false), team(team_num),
+		  name(team_name), leader(team_leader), disp_(disp), sides_(cfg),
+		  units_data_(data), cancel_button_(NULL), menu_(NULL)
 	{
 		SDL_Rect empty_rect = {0,0,0,0};
 		area_ = empty_rect;
 	}
 
-	void generate_menu()
+	void generate_menu(bool first)
 	{
 		if(area_.h == 0) {
 			return;
@@ -84,13 +88,37 @@ public:
 
 			std::string description = sd["description"];
 			std::string side_name = sd["name"];
-			if(s - sides.begin() == size_t(team-1)) {
+			std::string leader_type = sd["type"];
+
+			if(first && (s - sides.begin() == size_t(team-1))) {
 				description = preferences::login();
 				side_name = name;
+				leader_type = leader;
+			}
+
+			std::string leader_name;
+			std::string leader_image;
+			const game_data::unit_type_map& utypes = units_data_.unit_types;
+			if (utypes.find(leader_type) != utypes.end()) {
+				leader_name = utypes.find(leader_type)->second.language_name();
+				leader_image = utypes.find(leader_type)->second.image();
+			}
+			if (!leader_image.empty()) {
+				// Dumps the "image" part of the faction name, if any,
+				// to replace it by a picture of the actual leader
+				if(side_name[0] == font::IMAGE) {
+					int p = side_name.find_first_of(",");
+					if(p != std::string::npos && p < side_name.size()) {
+						side_name = "&" + leader_image + "," + side_name.substr(p+1);
+					}
+				}
 			}
 
 			std::stringstream str;
-			str << description << "," << side_name << "," << sd["gold"] << " " << translate_string("gold") << "," << sd["team_name"];
+			str << description << "," << side_name << ",";
+			if(!leader_name.empty())
+				str << "(" << leader_name << ")";
+			str << "," << sd["gold"] << " " << _("Gold") << "," << sd["team_name"];
 			details.push_back(str.str());
 		}
 
@@ -106,15 +134,15 @@ public:
 	void set_area(const SDL_Rect& area) {
 
 		area_ = area;
-		generate_menu();
+		generate_menu(true);
 
-		const std::string text = string_table["waiting_start"];
-		SDL_Rect rect = font::draw_text(NULL,disp_.screen_area(),14,font::NORMAL_COLOUR,text,0,0);
+		const std::string text = _("Waiting for game to start...");
+		SDL_Rect rect = font::draw_text(NULL,disp_.screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,text,0,0);
 		rect.x = area.x + area.w/2 - rect.w/2;
 		rect.y = area.y + (area.h*3)/4 - rect.h/2;
-		font::draw_text(&disp_,rect,14,font::NORMAL_COLOUR,text,rect.x,rect.y);
+		font::draw_text(&disp_,rect,font::SIZE_NORMAL,font::NORMAL_COLOUR,text,rect.x,rect.y);
 
-		cancel_button_.assign(new gui::button(disp_,string_table["cancel"]));
+		cancel_button_.assign(new gui::button(disp_,_("Cancel")));
 		cancel_button_->set_location(area.x+area.w - cancel_button_->width() - gui::ButtonHPadding,
 			                         area.y+area.h - cancel_button_->height() - gui::ButtonVPadding);
 		cancel_button_->draw();
@@ -144,7 +172,7 @@ public:
 			for(config::child_list::const_iterator a = assigns.begin(); a != assigns.end(); ++a) {
 				if(lexical_cast_default<int>((**a)["from"]) == team) {
 					team = lexical_cast_default<int>((**a)["to"]);
-					generate_menu();
+					generate_menu(false);
 				}
 			}
 
@@ -165,7 +193,7 @@ public:
 			} else if(reply.child("scenario_diff")) {
 				std::cerr << "received diff for scenario....applying...\n";
 				sides_.apply_diff(*reply.child("scenario_diff"));
-				generate_menu();
+				generate_menu(false);
 			} else if(reply.child("side")) {
 				sides_ = reply;
 				std::cerr << "got some sides. Current number of sides = " << sides_.get_children("side").size() << "," << reply.get_children("side").size() << "\n";
@@ -177,14 +205,17 @@ public:
 		return lobby::CONTINUE;
 	}
 
-	bool got_side;
-	int team;
-	std::string name;
 	enum { START_GAME, GAME_CANCELLED, SIDE_UNAVAILABLE } status;
 
+	bool got_side;
+	int team;
+
 private:
+	std::string name;
+	std::string leader;
 	display& disp_;
 	config& sides_;
+	const game_data& units_data_;
 
 	util::scoped_ptr<gui::button> cancel_button_;
 	util::scoped_ptr<gui::menu> menu_;
@@ -216,7 +247,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 		host = preferences::network_host();
 		const int res = gui::show_dialog(disp,NULL,"","",
 		                                 gui::OK_CANCEL,NULL,NULL,
-		                                 string_table["remote_host"] + ": ",&host);
+		                                 _("Choose host to connect to") + std::string(": "),&host);
 		if(res != 0 || host.empty()) {
 			return;
 		}
@@ -234,7 +265,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
  
 	config sides, data;
 
-	network::connection data_res = gui::network_data_dialog(disp,string_table["connecting_remote"],data);
+	network::connection data_res = gui::network_data_dialog(disp,_("Connecting to remote host..."),data);
 	check_response(data_res,data);
 
 	preferences::set_network_host(host);
@@ -263,8 +294,8 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 
 			if(!first_time) {	
 				const int res = gui::show_dialog(disp,NULL,"",
-				                    string_table["must_login"],gui::OK_CANCEL,
-									NULL,NULL,string_table["login"] + ": ",&login);
+				                    _("You must log in to this server"),gui::OK_CANCEL,
+									NULL,NULL,_("Login") + std::string(": "),&login);
 				if(res != 0 || login.empty()) {
 					return;
 				}
@@ -280,7 +311,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 	
 			data_res = network::receive_data(data,0,3000);
 			if(!data_res) {
-				throw network::error(string_table["connection_timeout"]);
+				throw network::error(_("Connection timed out"));
 			}
 
 			std::cerr << "login response: '" << data.write() << "'\n";
@@ -343,7 +374,12 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 			}
     
 			for(;;) {
-				data_res = gui::network_data_dialog(disp,string_table["getting_game_data"],sides);
+				data_res = gui::network_data_dialog(disp,_("Getting game data..."),sides);
+				if(data_res && sides.child("error")) {
+					gui::show_dialog(disp,NULL,"",(*sides.child("error"))["message"]);
+					break;
+				}
+
 				check_response(data_res,sides);
     
 				//if we have got valid side data
@@ -351,6 +387,10 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 					break;
 				}
 			}
+
+			if(sides.child("error")) {
+				continue;
+			}
 		} else {
 			sides = data;
 		}
@@ -362,6 +402,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 
 		int team_num = 0;
 		std::string team_name;
+		std::string team_leader;
 		
 		if(!observer) {
 			//search for an appropriate vacant slot. If a description is set
@@ -385,7 +426,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 			}
 
 			if(side_choice == sides_list.end()) {
-				gui::show_dialog(disp,NULL,"",string_table["no_sides_available"],gui::OK_ONLY);
+				gui::show_dialog(disp,NULL,"",_("There are no available sides in this game."),gui::OK_ONLY);
 				continue;
 			}
 
@@ -416,7 +457,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 			std::vector<std::string> choices;
 			for(config::child_list::const_iterator side =
 			    possible_sides.begin(); side != possible_sides.end(); ++side) {
-				choices.push_back(translate_string_default((**side)["id"],(**side)["name"]));
+				choices.push_back((**side)["name"]);
 
 				if(choices.back() == default_race) {
 					choice = side - possible_sides.begin();
@@ -425,23 +466,34 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 
 			//if the client is allowed to choose their team, instead of having
 			//it set by the server, do that here.
+			std::string leader;
+
 			if(allow_changes) {
+				std::vector<gui::preview_pane* > preview_panes;
+				leader_preview_pane leader_selector(disp, &units_data, possible_sides);
+				preview_panes.push_back(&leader_selector);
+
 				choice = size_t(gui::show_dialog(disp,NULL,"",
-				     string_table["client_choose_side"],gui::OK_ONLY,&choices));
+				     _("Choose your side:"),gui::OK_ONLY,&choices,&preview_panes));
+				leader = leader_selector.get_selected_leader();
 			}
 
 			assert(choice < possible_sides.size());
 
 			const config& chosen_side = *possible_sides[choice];
 			team_name = response["name"] = chosen_side["name"];
-			response["type"] = chosen_side["type"];
+			if(leader.empty()) {
+				team_leader = response["type"] = chosen_side["type"];
+			} else {
+				team_leader = response["type"] = leader;
+			}
 			response["recruit"] = chosen_side["recruit"];
 			response["music"] = chosen_side["music"];
 
 			network::send_data(response);
 		}
     
-		wait_for_start waiter(disp,sides,team_num,team_name);
+		wait_for_start waiter(disp,sides,units_data,team_num,team_name,team_leader);
 		std::vector<std::string> messages;
 		config game_data;
 		lobby::RESULT dialog_res = lobby::CONTINUE;
@@ -452,11 +504,11 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 		waiter.clear_widgets();
 
 		if(waiter.status == wait_for_start::GAME_CANCELLED) {
-			gui::show_dialog(disp,NULL,"",string_table["game_cancelled"],
+			gui::show_dialog(disp,NULL,"",_("The game has been cancelled"),
 			                 gui::OK_ONLY);
 			continue;
 		} else if(waiter.status == wait_for_start::SIDE_UNAVAILABLE) {
-			gui::show_dialog(disp,NULL,"",string_table["side_unavailable"],
+			gui::show_dialog(disp,NULL,"",_("The side you have chosen is no longer available"),
 			                 gui::OK_ONLY);
 			continue;			
 		} else if(dialog_res != lobby::CREATE) {
@@ -492,7 +544,7 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 			recorder = replay(replay_data_store);
 			if(!recorder.empty()) {
 				const int res = gui::show_dialog(disp,NULL,
-		               "", string_table["replay_game_message"],
+		               "", _("Show replay of game up to save point?"),
 					   gui::YES_NO);
 				//if yes, then show the replay, otherwise
 				//skip showing the replay
@@ -512,7 +564,11 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 
 		state.starting_pos = sides;
 		state.snapshot = sides;
-		state.can_recruit.clear();
+
+                for(std::map<std::string, player_info>::iterator i=state.players.begin();
+                    i!=state.players.end(); ++i) {
+                  i->second.can_recruit.clear();
+                }
     
 		recorder.set_save_info(state);
     
@@ -521,3 +577,238 @@ void play_multiplayer_client(display& disp, game_data& units_data, config& cfg,
 		recorder.clear();
 	}
 }
+
+leader_list_manager::leader_list_manager(const config::child_list& side_list,
+		const game_data* data, gui::combo* combo) :
+	side_list_(side_list), data_(data), combo_(combo)
+{
+#if 0
+	for(config::const_child_iterator itor = side_list.begin(); itor != side_list.end(); ++itor) {
+		std::cerr << "Child: " << *itor << "\n";
+		std::cerr << (**itor).write();
+	}
+#endif
+}
+
+void leader_list_manager::set_combo(gui::combo* combo)
+{
+	combo_ = combo;
+
+	if (combo_ != NULL) {
+		update_leader_list(0);
+	}
+}
+
+void leader_list_manager::update_leader_list(int side_index)
+{
+	const config& side = *side_list_[side_index];
+
+	leaders_.clear();
+
+	if(side["random_faction"] == "yes") {
+		if(combo_ != NULL) {
+			std::vector<std::string> dummy;
+			dummy.push_back("-");
+			combo_->enable(false);
+			combo_->set_items(dummy);
+			combo_->set_selected(0);
+		}
+		return;
+	} else {
+		if(combo_ != NULL)
+			combo_->enable(true);
+	}
+
+	if(!side["leader"].empty()) {
+		leaders_ = config::split(side["leader"]);
+	}
+	
+	const std::string default_leader = side["type"];
+	int default_index = 0;
+
+	std::vector<std::string>::const_iterator itor;
+
+	for (itor = leaders_.begin(); itor != leaders_.end(); ++itor) {
+		if (*itor == default_leader) {
+			break;
+		}
+		default_index++;
+	}
+
+	if (default_index == leaders_.size()) {
+		leaders_.push_back(default_leader);
+	}
+
+	std::vector<std::string> leader_strings;
+
+	for(itor = leaders_.begin(); itor != leaders_.end(); ++itor) {
+		
+		const game_data::unit_type_map& utypes = data_->unit_types;
+
+		//const std::string name = data_->unit_types->find(*itor).language_name();
+		if (utypes.find(*itor) != utypes.end()) {
+			const std::string name =  utypes.find(*itor)->second.language_name();
+			const std::string image = utypes.find(*itor)->second.image();
+
+			leader_strings.push_back("&" + image + "," + name);
+		} else {
+			leader_strings.push_back("?");
+		}
+	}
+
+	leaders_.push_back("random");
+	// FIXME: Maybe this should not code into the code.
+	leader_strings.push_back(_("&random-enemy.png,Random"));
+
+	if(combo_ != NULL) {
+		combo_->set_items(leader_strings);
+		combo_->set_selected(default_index);
+	}
+}
+
+void leader_list_manager::set_leader(const std::string& leader)
+{
+	if(combo_ == NULL)
+		return;
+
+	int leader_index = 0;
+	for(std::vector<std::string>::const_iterator itor = leaders_.begin();
+			itor != leaders_.end(); ++itor) {
+		if (leader == *itor) 
+			combo_->set_selected(leader_index);
+		++leader_index;
+	}
+}
+
+std::string leader_list_manager::get_leader()
+{
+	if(combo_ == NULL)
+		return _("?");
+
+	if(combo_->selected() >= leaders_.size())
+		return _("?");
+
+	return leaders_[combo_->selected()];
+}
+
+namespace {
+	static const SDL_Rect leader_pane_position = {-260,-370,260,370};
+	static const int leader_pane_border = 10;
+}
+
+leader_preview_pane::leader_preview_pane(display& disp, const game_data* data,
+		const config::child_list& side_list) :
+	gui::preview_pane(disp),
+	side_list_(side_list),
+	leader_combo_(disp, std::vector<std::string>()), 
+	leaders_(side_list, data, &leader_combo_),
+	selection_(0), data_(data)
+{
+
+	set_location(leader_pane_position);
+}
+
+void leader_preview_pane::process()
+{
+	int mousex, mousey;
+	const int mouse_flags = SDL_GetMouseState(&mousex,&mousey);
+	const bool left_button = mouse_flags&SDL_BUTTON_LMASK;
+
+	if(leader_combo_.process(mousex, mousey, left_button)) {
+		set_dirty();
+	}
+}
+
+void leader_preview_pane::draw()
+{
+	if(!dirty())
+		return;
+
+	bg_restore();
+
+	surface const screen = disp().video().getSurface();
+
+	const SDL_Rect area = { location().x+leader_pane_border, location().y+leader_pane_border,
+	                        location().w-leader_pane_border*2, location().h-leader_pane_border*2 };
+	SDL_Rect clip_area = area;
+	const clip_rect_setter clipper(screen,clip_area);
+
+	if(selection_ < side_list_.size()) {
+		const config& side = *side_list_[selection_];
+		std::string faction = side["name"];
+		const std::string recruits = side["recruit"];
+		const std::vector<std::string> recruit_list = config::split(recruits);
+		std::ostringstream recruit_string;
+
+		if(faction[0] == font::IMAGE) {
+			int p = faction.find_first_of(",");
+			if(p != std::string::npos && p < faction.size())
+				faction = faction.substr(p+1);
+		}
+		std::string leader = leaders_.get_leader();
+
+		const game_data::unit_type_map& utypes = data_->unit_types;
+		std::string leader_name;
+		std::string image;
+
+		if (utypes.find(leader) != utypes.end()) {
+			leader_name = utypes.find(leader)->second.language_name();
+			image = utypes.find(leader)->second.image();
+		}
+		for(std::vector<std::string>::const_iterator itor = recruit_list.begin();
+				itor != recruit_list.end(); ++itor) {
+
+			if (utypes.find(*itor) != utypes.end()) {
+				if(itor != recruit_list.begin())
+					recruit_string << ", ";
+				recruit_string << utypes.find(*itor)->second.language_name();
+			}
+		}
+
+		SDL_Rect image_rect = {area.x,area.y,0,0};
+
+		surface unit_image(image::get_image(image, image::UNSCALED));
+	
+		if(!unit_image.null()) {
+			image_rect.w = unit_image->w;
+			image_rect.h = unit_image->h;
+			SDL_BlitSurface(unit_image,NULL,screen,&image_rect);
+		}
+
+		font::draw_text(&disp(),area,font::SIZE_PLUS,font::NORMAL_COLOUR,faction,area.x + 80, area.y + 30);
+		const SDL_Rect leader_rect = font::draw_text(&disp(),area,font::SIZE_SMALL,font::NORMAL_COLOUR,
+				_("Leader: "),area.x, area.y + 80);
+		font::draw_wrapped_text(&disp(),area,font::SIZE_SMALL,font::NORMAL_COLOUR,
+				_("Recruits: ") + recruit_string.str(),area.x, area.y + 102,
+				area.w);
+
+		leader_combo_.set_location(leader_rect.x + leader_rect.w + 10, leader_rect.y + (leader_rect.h - leader_combo_.height()) / 2);
+	}
+
+
+	set_dirty(false);
+}
+
+bool leader_preview_pane::show_above() const
+{
+	return false;
+}
+
+bool leader_preview_pane::left_side() const
+{
+	return false;
+}
+
+void leader_preview_pane::set_selection(int selection)
+{
+	selection_ = selection;
+	leaders_.update_leader_list(selection_);
+	set_dirty();
+	leader_combo_.set_dirty();
+}
+
+std::string leader_preview_pane::get_selected_leader()
+{
+	return leaders_.get_leader();
+}
+
