@@ -1,4 +1,4 @@
-/* $Id: multiplayer_connect.cpp,v 1.57 2004/06/13 21:44:14 Sirp Exp $ */
+/* $Id: multiplayer_connect.cpp,v 1.76 2004/10/03 19:29:54 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -44,10 +44,10 @@ mp_connect::mp_connect(display& disp, std::string game_name,
 	    show_replay_(false), save_(false), join_(join),
 	    player_types_(), player_races_(), player_teams_(),
 	    player_colors_(), combos_type_(), combos_race_(),
-	    combos_team_(), combos_color_(), sliders_gold_(),
-	    launch_(gui::button(disp, string_table["im_ready"])),
-	    cancel_(gui::button(disp, string_table["cancel"])),
-	    ai_(gui::button(disp, string_table["ai_players"])),
+	    combos_leader_(), combos_team_(), combos_color_(), sliders_gold_(),
+	    ai_(gui::button(disp, _(" Computer vs Computer "))),
+	    launch_(gui::button(disp, _("I'm Ready"))),
+	    cancel_(gui::button(disp, _("Cancel"))),
 		message_full_(true)
 {
 	// Send Initial information
@@ -74,8 +74,6 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 	// Setup the game
 	config* level_ptr;
 
-	era_ = era;
-
 	if(scenario_data.child("side") == NULL) {
 		//Load a saved game
 		save_ = true;
@@ -89,19 +87,21 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 
 		load_game(*data_, game, *state_);
 
-		state_->available_units.clear();
-		state_->can_recruit.clear();
+		for(std::map<std::string, player_info>::iterator i = state_->players.begin(); i != state_->players.end(); ++i) {
+			i->second.available_units.clear();
+			i->second.can_recruit.clear();
+		}
 
 		if(state_->campaign_type != "multiplayer") {
 			gui::show_dialog(*disp_, NULL, "", 
-					 string_table["not_multiplayer_save_message"],
+					 _("This is not a multiplayer save"),
 					 gui::OK_ONLY);
 			return -1;
 		}
 
 		if(state_->version != game_config::version) {
 			const int res = gui::show_dialog(*disp_, NULL, "",
-						string_table["version_save_message"],
+						_("This save is from a different version of the game. Do you want to try to load it?"),
 						gui::YES_NO);
 			if(res == 1) {
 				return -1;
@@ -112,8 +112,7 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 		level_ptr = &scenario_data;
 
 		//make all sides untaken
-		for(config::child_itors i = level_ptr->child_range("side");
-		    i.first != i.second; ++i.first) {
+		for(config::child_itors i = level_ptr->child_range("side"); i.first != i.second; ++i.first) {
 			(**i.first)["taken"] = "";
 
 			//tell clients not to change their race
@@ -150,12 +149,21 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 
 	assert(level_ptr != NULL);
 
+	if(scenario_data["era"].empty()) {
+		era_ = era;
+	} else { 
+		era_ = scenario_data["era"];
+	}
+
 	//this will force connecting clients to be using the same version number as us.
 	(*level_ptr)["version"] = game_config::version;
 
 	level_ = level_ptr;
 	state_->label = level_->values["name"];
-	state_->gold = -10000;
+
+	for(std::map<std::string, player_info>::iterator i = state_->players.begin(); i != state_->players.end(); ++i) {
+		i->second.gold = -10000;
+	}
 
 	state_->scenario = scenario_data["id"];
 
@@ -174,7 +182,7 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 
 	if(sides.first == sides.second || possible_sides.empty()) {
 		gui::show_dialog(*disp_, NULL, "", 
-				 string_table["error_no_mp_sides"],
+				 _("No multiplayer sides."),
 				 gui::OK_ONLY);
 		std::cerr << "no multiplayer sides found\n";
 		return -1;
@@ -217,8 +225,13 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 		if(side["name"].empty())
 			side["name"] = (*possible_sides.front())["name"];
 
-		if(side["type"].empty() && save_ == false)
+		if(side["leader"].empty())
+			side["leader"] = (*possible_sides.front())["leader"];
+
+		if(side["type"].empty() && save_ == false) {
+			side["random_faction"] = (*possible_sides.front())["random_faction"];
 			side["type"] = (*possible_sides.front())["type"];
+		}
 
 		if(side["recruit"].empty())
 			side["recruit"] = (*possible_sides.front())["recruit"];
@@ -234,12 +247,13 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 	}
 
 	if((*level_)["objectives"] == "") {
-		(*level_)["objectives"] = string_table["mp_objectives"];
+		(*level_)["objectives"] = _("Victory\n\
+@Defeat enemy leader(s)");
 	}
 
 	if(save_ == false) {
 		(*level_)["experience_modifier"] = lexical_cast<std::string>(xpmodifier);
-		(*level_)["era"] = era;
+		(*level_)["era"] = era_;
 	}
 
 	lists_init();
@@ -253,14 +267,14 @@ int mp_connect::load_map(const std::string& era, config& scenario_data, int num_
 void mp_connect::lists_init()
 {
 	//Options
-	player_types_.push_back(string_table["network_controlled"]);
-	player_types_.push_back(string_table["human_controlled"]);
-	player_types_.push_back(string_table["ai_controlled"]);
-	player_types_.push_back(string_table["null_controlled"]);
+	player_types_.push_back(_("Network Player"));
+	player_types_.push_back(_("Local Player"));
+	player_types_.push_back(_("Computer Player"));
+	player_types_.push_back(_("Empty"));
 	player_types_.push_back("-----");
 	player_types_.push_back(preferences::login());
 
-	//Races
+	//Factions
 	const config::child_itors sides = level_->child_range("side");
 
 	const config* const era_cfg = cfg_->find_child("era","id",era_);
@@ -270,9 +284,8 @@ void mp_connect::lists_init()
 
 	const config::child_list& possible_sides = era_cfg->get_children("multiplayer_side");
 
-	for(std::vector<config*>::const_iterator race = possible_sides.begin();
-	    race != possible_sides.end(); ++race) {
-		player_races_.push_back(translate_string((**race)["name"]));
+	for(std::vector<config*>::const_iterator race = possible_sides.begin(); race != possible_sides.end(); ++race) {
+		player_races_.push_back((**race)["name"]);
 	}
 
 	//Teams
@@ -284,30 +297,33 @@ void mp_connect::lists_init()
 			team_name = lexical_cast<std::string>(team_num+1);
 		}
 
-		player_teams_.push_back(string_table["team"] + " " + team_name);
+		player_teams_.push_back(_("Team") + std::string(" ") + team_name);
+		(**sd)["colour"] = lexical_cast_default<std::string>(team_num+1);
+
+		player_leaders_.push_back(leader_list_manager(possible_sides, data_));
 	}
 
 	std::string prefix;
 	prefix.resize(1);
 	//Colors
 	prefix[0] = 1;
-	player_colors_.push_back(prefix + translate_string("red"));
+	player_colors_.push_back(prefix + _("Red"));
 	prefix[0] = 2;
-	player_colors_.push_back(prefix + translate_string("blue"));
+	player_colors_.push_back(prefix + _("Blue"));
 	prefix[0] = 3;
-	player_colors_.push_back(prefix + translate_string("green"));
+	player_colors_.push_back(prefix + _("Green"));
 	prefix[0] = 4;
-	player_colors_.push_back(prefix + translate_string("yellow"));
+	player_colors_.push_back(prefix + _("Yellow"));
 	prefix[0] = 5;
-	player_colors_.push_back(prefix + translate_string("purple"));
+	player_colors_.push_back(prefix + _("Purple"));
 	prefix[0] = 6;
-	player_colors_.push_back(prefix + translate_string("orange"));
+	player_colors_.push_back(prefix + _("Orange"));
 	prefix[0] = 7;
-	player_colors_.push_back(prefix + translate_string("grey"));
+	player_colors_.push_back(prefix + _("Grey"));
 	prefix[0] = 8;
-	player_colors_.push_back(prefix + translate_string("white"));
+	player_colors_.push_back(prefix + _("White"));
 	prefix[0] = 9;
-	player_colors_.push_back(prefix + translate_string("brown"));
+	player_colors_.push_back(prefix + _("Brown"));
 }
 
 void mp_connect::add_player(const std::string& name)
@@ -332,48 +348,50 @@ void mp_connect::set_area(const SDL_Rect& rect)
 
 	const int left = rect.x;
 	const int right = rect.x + rect.w;
-	const int center_x = rect.x + rect.w/2;
 	const int top = rect.y;
 	const int bottom = rect.y + rect.h;
-	const int center_y = rect.y + rect.h/2;
-
-	const int width = rect.w;
-	const int height = rect.h;
 
 	// Wait to players, Configure players
 	//gui::draw_dialog_background(left, right, width, height, *disp_, "menu");
 
+	gui::button* left_button = &launch_;
+	gui::button* right_button = &cancel_;
+
+#ifdef OK_BUTTON_ON_RIGHT
+	std::swap(left_button,right_button);
+#endif
+
 	//Buttons
-	cancel_.set_location(right - cancel_.width() - gui::ButtonHPadding,bottom-cancel_.height()-gui::ButtonVPadding);
-	launch_.set_location(right - cancel_.width() - launch_.width() - gui::ButtonHPadding*2,bottom-launch_.height()-gui::ButtonVPadding);
+	right_button->set_location(right - right_button->width() - gui::ButtonHPadding,bottom-right_button->height()-gui::ButtonVPadding);
+	left_button->set_location(right - right_button->width() - left_button->width() - gui::ButtonHPadding*2,bottom-left_button->height()-gui::ButtonVPadding);
 	
 	ai_.set_location(left+30,bottom-60);
 
 	//Title and labels
-	gui::draw_dialog_title(left,top,disp_,string_table["game_lobby"]);
+	gui::draw_dialog_title(left,top,disp_,_("Game Lobby"));
 
-	SDL_Rect labelr = font::draw_text(NULL,rect,14,font::GOOD_COLOUR,
-			                          string_table["player_type"],0,0);
-	font::draw_text(disp_,rect,14,font::GOOD_COLOUR,
-	                string_table["player_type"],(left+30)+(launch_.width()/2)-(labelr.w/2),top+35);
-	labelr = font::draw_text(NULL,rect,14,font::GOOD_COLOUR,string_table["race"],0,0);
+	SDL_Rect labelr = font::draw_text(NULL,rect,font::SIZE_NORMAL,font::GOOD_COLOUR,
+			                          _("Player/Type"),0,0);
+	font::draw_text(disp_,rect,font::SIZE_NORMAL,font::GOOD_COLOUR,
+	                _("Player/Type"),(left+30)+(launch_.width()/2)-(labelr.w/2),top+35);
+	labelr = font::draw_text(NULL,rect,font::SIZE_NORMAL,font::GOOD_COLOUR,_("Faction"),0,0);
 
-	font::draw_text(disp_,rect,14,font::GOOD_COLOUR,
-	                string_table["race"],(left+145)+(launch_.width()/2)-(labelr.w/2),top+35);
+	font::draw_text(disp_,rect,font::SIZE_NORMAL,font::GOOD_COLOUR,
+	                _("Faction"),(left+145)+(launch_.width()/2)-(labelr.w/2),top+35);
 	
-	labelr = font::draw_text(NULL,rect,14,font::GOOD_COLOUR,string_table["team"],0,0);
+	labelr = font::draw_text(NULL,rect,font::SIZE_NORMAL,font::GOOD_COLOUR,_("Team"),0,0);
 
-	font::draw_text(disp_,disp_->screen_area(),14,font::GOOD_COLOUR,
-	                string_table["team"],(left+260)+(launch_.width()/2)-(labelr.w/2),top+35);
+	font::draw_text(disp_,disp_->screen_area(),font::SIZE_NORMAL,font::GOOD_COLOUR,
+	                _("Team"),(left+260)+(launch_.width()/2)-(labelr.w/2),top+35);
 	
-	labelr = font::draw_text(NULL,rect,14,font::GOOD_COLOUR,string_table["color"],0,0);
+	labelr = font::draw_text(NULL,rect,font::SIZE_NORMAL,font::GOOD_COLOUR,_("Color"),0,0);
 
-	font::draw_text(disp_,disp_->screen_area(),14,font::GOOD_COLOUR,
-	                string_table["color"],(left+375)+(launch_.width()/2)-(labelr.w/2),top+35);
+	font::draw_text(disp_,disp_->screen_area(),font::SIZE_NORMAL,font::GOOD_COLOUR,
+	                _("Color"),(left+375)+(launch_.width()/2)-(labelr.w/2),top+35);
 	
-	labelr = font::draw_text(NULL,rect,14,font::GOOD_COLOUR,string_table["gold"],0,0);
-	font::draw_text(disp_,rect,14,font::GOOD_COLOUR,
-	                string_table["gold"],(left+480)+(launch_.width()/2)-(labelr.w/2),top+35);
+	labelr = font::draw_text(NULL,rect,font::SIZE_NORMAL,font::GOOD_COLOUR,_("Gold"),0,0);
+	font::draw_text(disp_,rect,font::SIZE_NORMAL,font::GOOD_COLOUR,
+	                _("Gold"),(left+480)+(launch_.width()/2)-(labelr.w/2),top+35);
 
 	//Per player settings
 	const config::child_itors sides = level_->child_range("side");
@@ -383,10 +401,9 @@ void mp_connect::set_area(const SDL_Rect& rect)
 		return;
 	}
 
-	const config::child_list& possible_sides = era_cfg->get_children("multiplayer_side");
-
 	combos_type_.clear();
 	combos_race_.clear();
+	combos_leader_.clear();
 	combos_team_.clear();
 	combos_color_.clear();
 	sliders_gold_.clear();
@@ -397,45 +414,57 @@ void mp_connect::set_area(const SDL_Rect& rect)
 		const int side_num = sd - sides.first;
 
 		//Player number
-		font::draw_text(disp_,rect, 24, font::GOOD_COLOUR,
-		                (*sd)->values["side"], left+10, top+53+(30*side_num));
+		font::draw_text(disp_,rect, font::SIZE_XLARGE, font::GOOD_COLOUR,
+		                (*sd)->values["side"], left+10, top+53+(60*side_num));
 
 		//Player type
 		combos_type_.push_back(gui::combo(*disp_, player_types_));
-		combos_type_.back().set_location(left+30,top+55+(30*side_num));
+		combos_type_.back().set_location(left+30,top+55+(60*side_num));
 
 		//Player race
 		combos_race_.push_back(gui::combo(*disp_, player_races_));
-		combos_race_.back().set_location(left+145,top+55+(30*side_num));
+		combos_race_.back().set_location(left+145,top+55+(60*side_num));
+
+		//Player leader
+		combos_leader_.push_back(gui::combo(*disp_, std::vector<std::string>()));
+		combos_leader_.back().set_location(left+145,top+85+(60*side_num));
 
 		//Player team
 		combos_team_.push_back(gui::combo(*disp_, player_teams_));
-		combos_team_.back().set_location(left+260,top+55+(30*side_num));
+		combos_team_.back().set_location(left+260,top+55+(60*side_num));
 		combos_team_.back().set_selected(side_num);
 
 		//Player color
 		combos_color_.push_back(gui::combo(*disp_, player_colors_));
-		combos_color_.back().set_location(left+375,top+55+(30*side_num));
+		combos_color_.back().set_location(left+375,top+55+(60*side_num));
 		combos_color_.back().set_selected(side_num);
 
 		SDL_Rect r;
 
 		//Player gold
 		r.x = left+490;
-		r.y = top+55+(30*side_num);
+		r.y = top+55+(60*side_num);
 		r.w = launch_.width()-5;
 		r.h = launch_.height();
 		sliders_gold_.push_back(gui::slider(*disp_, r));
 		sliders_gold_.back().set_min(20);
 		sliders_gold_.back().set_max(1000);
+		sliders_gold_.back().set_increment(25);
 		sliders_gold_.back().set_value(lexical_cast_default<int>((**sd)["gold"],100));
 		r.w = 30;
 		r.x = left+603;
 		gold_bg_.push_back(surface_restorer(&disp_->video(),r));
-		font::draw_text(disp_, disp_->screen_area(), 12, font::GOOD_COLOUR,
+		font::draw_text(disp_, disp_->screen_area(), font::SIZE_SMALL, font::GOOD_COLOUR,
 		                "100", r.x, r.y);
 	}
 
+	//Doing this after creating the combos, because growing vectors may
+	//move their elements in memory, and we need a stable pointer
+	for(sd = sides.first; sd != sides.second; ++sd) {
+		const int side_num = sd - sides.first;
+		player_leaders_[side_num].set_combo(&combos_leader_[side_num]);
+	}
+
 	std::cerr << "done set_area()\n";
 
 	update_whole_screen();
@@ -452,8 +481,6 @@ void mp_connect::gui_update()
 		return;
 	}
 
-	const config::child_list& possible_sides = era_cfg->get_children("multiplayer_side");
-
 	const config::child_itors sides = level_->child_range("side");
 	SDL_Rect rect;
 
@@ -464,7 +491,7 @@ void mp_connect::gui_update()
 		if (side["controller"] == "network") {
 			if (side["description"] == "") {
 				combos_type_[n].set_selected(0);
-			} else if (side["description"] == string_table["ai_controlled"]) {
+			} else if (side["description"] == _("Computer Player")) {
 				//When loading a game you did not create AI players are marked
 				//as network players, set back to AI
 				combos_type_[n].set_selected(2);
@@ -491,13 +518,19 @@ void mp_connect::gui_update()
 			combos_type_[n].set_selected(3);
 		}
 
-		//Player Race
+		//Player Faction
 		for (size_t m = 0; m != player_races_.size(); ++m) {
-			if (translate_string(side["name"]) == player_races_[m]) {
-				combos_race_[n].set_selected(m);
+			if (side["name"] == player_races_[m]) {
+				if (combos_race_[n].selected() != m) {
+					combos_race_[n].set_selected(m);
+					player_leaders_[n].update_leader_list(m);
+				}
 			}
 		}
 
+		// Player leader
+		player_leaders_[n].set_leader(side["type"]);
+
 		//Player Team
 		const std::string& team_name = side["team_name"];
 		if(team_name != "" && isdigit(team_name[0]))
@@ -509,11 +542,11 @@ void mp_connect::gui_update()
 		std::string str = side["gold"];
 		sliders_gold_[n].set_value(atoi(str.c_str()));
 		rect.x = rect_.x + 603;
-		rect.y = rect_.y + 55 + (30 * n);
+		rect.y = rect_.y + 55 + (60 * n);
 		rect.w = 30;
 		rect.h = launch_.height();
 		gold_bg_[n].restore();
-		font::draw_text(disp_, disp_->screen_area(), 12,
+		font::draw_text(disp_, disp_->screen_area(), font::SIZE_SMALL,
 				font::GOOD_COLOUR,
 		                side["gold"],
 				rect.x, rect.y);
@@ -527,11 +560,11 @@ void mp_connect::gui_update()
 			message_bg_.restore();
 			message_bg_ = surface_restorer();
 		} else {
-			SDL_Rect rect = font::draw_text(NULL,rect_,12,font::NORMAL_COLOUR,string_table["waiting_other_players"],0,0);
+			SDL_Rect rect = font::draw_text(NULL,rect_,font::SIZE_SMALL,font::NORMAL_COLOUR,_("Waiting for network players to join"),0,0);
 			rect.x = ai_.location().x + ai_.location().w + 10;
 			rect.y = ai_.location().y;
 			message_bg_ = surface_restorer(&disp_->video(),rect);
-			font::draw_text(disp_,rect,12,font::NORMAL_COLOUR,string_table["waiting_other_players"],rect.x,rect.y);
+			font::draw_text(disp_,rect,font::SIZE_SMALL,font::NORMAL_COLOUR,_("Waiting for network players to join"),rect.x,rect.y);
 		}
 	}
 }
@@ -578,7 +611,7 @@ lobby::RESULT mp_connect::process()
 				side["description"] = "";
 			} else if(combos_type_[n].selected() == 2){
 				side["controller"] = "ai";
-				side["description"] = string_table["ai_controlled"];
+				side["description"] = _("Computer Player");
 			} else if(combos_type_[n].selected() == 3) {
 				side["controller"] = "null";
 				side["description"] = "";
@@ -589,10 +622,10 @@ lobby::RESULT mp_connect::process()
 				side["description"] = preferences::login();
 				for(size_t m = 0; m != combos_type_.size(); ++m) {
 					if(m != n) {
-						if(combos_type_[m].selected() == 4){
-							combos_type_[m].set_selected(0);
+						if(combos_type_[m].selected() == 5){
+							combos_type_[m].set_selected(2);
 							config& si = **(sides.first+m);
-							si["controller"] = "network";
+							si["controller"] = "ai";
 							si["description"] = "";
 						}
 					}
@@ -609,18 +642,34 @@ lobby::RESULT mp_connect::process()
 			side["colour"] = lexical_cast_default<std::string>(combos_color_[n].selected()+1);
 		}
 
+			
 		//Player race
 		combos_race_[n].enable(!save_);
+		combos_leader_[n].enable(!save_);
 		combos_team_[n].enable(!save_);
 		combos_color_[n].enable(!save_);
 		
+		old_select = combos_race_[n].selected();
 		if(combos_race_[n].process(mousex, mousey, left_button)) {
 			const string_map& values =  possible_sides[combos_race_[n].selected()]->values;
+			side["random_faction"] = "";
 			for(string_map::const_iterator i = values.begin(); i != values.end(); ++i) {
 				std::cerr << "value: " << i->first << " , " << i->second<< std::endl;
 				side[i->first] = i->second;
 			}
 			level_changed = true;
+
+			if (combos_race_[n].selected() != old_select) {
+				player_leaders_[n].update_leader_list(combos_race_[n].selected());
+			}
+		}
+
+		//Player leader
+		if(combos_leader_[n].process(mousex, mousey, left_button)) {
+			std::stringstream str;
+			str << (combos_team_[n].selected()+1);
+			side["type"] = player_leaders_[n].get_leader();
+			level_changed = true;
 		}
 
 		//Player team
@@ -646,11 +695,11 @@ lobby::RESULT mp_connect::process()
 
 				SDL_Rect rect;
 				rect.x = rect_.x + 603;
-				rect.y = rect_.y + 55 + (30 * n);
+				rect.y = rect_.y + 55 + (60 * n);
 				rect.w = 30;
 				rect.h = launch_.height();
 				gold_bg_[n].restore();
-				font::draw_text(disp_, rect_, 12,font::GOOD_COLOUR,(*sides.first[n])["gold"],
+				font::draw_text(disp_, rect_, font::SIZE_SMALL,font::GOOD_COLOUR,(*sides.first[n])["gold"],
 						        rect.x, rect.y);
 				update_rect(rect);
 				level_changed = true;
@@ -672,7 +721,7 @@ lobby::RESULT mp_connect::process()
 		for(size_t m = 0; m != combos_team_.size(); ++m) {
 			config& si = **(sides.first+m);
 			si["controller"] = "ai";
-			si["description"] = string_table["ai_controlled"];
+			si["description"] = _("Computer Player");
 			combos_type_[m].set_selected(2);
 		}
 		level_changed = true;
@@ -685,11 +734,16 @@ lobby::RESULT mp_connect::process()
 
 		for(config::child_iterator side = sides.first; side != sides.second; ++side) {
 			int ntry = 0;
-			while((**side)["type"] == "random" && ntry < 1000) {
+			//std::cerr << "Checking random faction for " << (**side)["name"] << ". Random faction is " << (**side)["random_faction"] << "\n";
+			while((**side)["random_faction"] == "yes" && ntry < 1000) {
 				const int choice = rand()%real_sides.size();
 
 				(**side)["name"] = (*real_sides[choice])["name"];
-				(**side)["type"] = (*real_sides[choice])["type"];
+				(**side)["random_faction"] = (*real_sides[choice])["random_faction"];
+
+				(**side)["type"] = "random";
+
+				(**side)["leader"] = (*real_sides[choice])["leader"];
 				(**side)["recruit"] = (*real_sides[choice])["recruit"];
 				(**side)["music"] = (*real_sides[choice])["music"];
 
@@ -699,6 +753,21 @@ lobby::RESULT mp_connect::process()
 
 				++ntry;
 			}
+
+			if ((**side)["type"] == "random" || (**side)["type"].empty()) {
+				//std::cerr << "Choosing a random type for " << (**side)["name"] << "\n";
+				// Choose a random leader type.  
+				std::vector<std::string> types = 
+					config::split((**side)["leader"]);
+				if (!types.empty()) {
+					const int lchoice = rand() % types.size();
+					(**side)["type"] = types[lchoice];
+				} else {
+					lg::err(lg::config) << "Unable to find a type for side " << (**side)["name"] << "!\n";
+				}
+				//std::cerr << "Leader chosen: " << (**side)["type"] << "\n";
+				level_changed = true;
+			}
 		}
 
 		start_game = true;
@@ -740,6 +809,7 @@ void mp_connect::start_game()
 {
 	combos_type_.clear();
 	combos_race_.clear();
+	combos_leader_.clear();
 	combos_team_.clear();
 	combos_color_.clear();
 	sliders_gold_.clear();
@@ -777,7 +847,10 @@ void mp_connect::start_game()
 	//it was just there to tell clients about the replay data
 	level_->clear_children("replay");
 	std::vector<config*> story;
-	state_->can_recruit.clear();
+	for(std::map<std::string, player_info>::iterator i = state_->players.begin(); i != state_->players.end(); ++i) {
+		i->second.can_recruit.clear();
+	}
+
 	play_level(*data_, *cfg_, level_, disp_->video(), *state_, story);
 	recorder.clear();
 
@@ -801,8 +874,7 @@ void mp_connect::update_positions()
 
 void mp_connect::update_network()
 {
-	for(std::map<config*,network::connection>::const_iterator i = positions_.begin();
-	    i != positions_.end(); ++i) {
+	for(std::map<config*,network::connection>::const_iterator i = positions_.begin(); i != positions_.end(); ++i) {
 		if(!i->second) {
 			//We are waiting on someone
 			network::connection sock = network::accept_connection();
@@ -836,8 +908,7 @@ void mp_connect::update_network()
 		bool changes = false;
 
 		//a socket has disconnected. Remove its positions.
-		for(std::map<config*,network::connection>::iterator i = positions_.begin();
-		    i != positions_.end(); ++i) {
+		for(std::map<config*,network::connection>::iterator i = positions_.begin(); i != positions_.end(); ++i) {
 			if(i->second == e.socket) {
 				changes = true;
 				i->second = 0;
@@ -907,10 +978,6 @@ void mp_connect::update_network()
 
 				std::cerr << "client has taken a valid position\n";
 
-				//does the client already own the side, and is just updating
-				//it, or is it taking a vacant slot?
-				const bool update_only = pos->second == sock;
-
 				//broadcast to everyone the new game status
 				pos->first->values["controller"] = "network";
 				pos->first->values["taken"] = "yes";
@@ -924,6 +991,7 @@ void mp_connect::update_network()
 				}
 
 				if(cfg["type"].empty() == false) {
+					pos->first->values["random_faction"] = "";
 					pos->first->values["type"] = cfg["type"];
 				}
 
@@ -935,6 +1003,10 @@ void mp_connect::update_network()
 					pos->first->values["music"] = cfg["music"];
 				}
 
+				if(cfg["random_faction"].empty() == false) {
+					pos->first->values["random_faction"] = cfg["random_faction"];
+				}
+
 				if(cfg["terrain_liked"].empty() == false) {
 					pos->first->values["terrain_liked"] = cfg["terrain_liked"];
 				}
