@@ -11,6 +11,7 @@
 #include "preferences.hpp"
 #include "show_dialog.hpp"
 #include "sound.hpp"
+#include "statistics.hpp"
 #include "widgets/textbox.hpp"
 #include "widgets/button.hpp"
 #include "widgets/menu.hpp"
@@ -36,13 +37,16 @@ namespace lobby {
 RESULT enter(display& disp, config& game_data, const config& terrain_data, dialog* dlg,
 			 std::vector<std::string>& messages)
 {
+	statistics::fresh_stats();
+
+	std::cerr << "entered multiplayer lobby...\n";
 	const preferences::display_manager disp_manager(&disp);
 	const hotkey::basic_handler key_handler(&disp);
 	const video_change_detector disp_change_detector(disp.video());
 
 	CKey key;
 
-	scoped_sdl_surface background(image::get_image("misc/lobby.png",image::UNSCALED));
+	surface background(image::get_image("misc/lobby.png",image::UNSCALED));
 	background.assign(scale_surface(background,disp.x(),disp.y()));
 
 	if(background == NULL) {
@@ -110,11 +114,11 @@ RESULT enter(display& disp, config& game_data, const config& terrain_data, dialo
 				if(map_data != "") {
 					try {
 						gamemap map(terrain_data,map_data);
-						SDL_Surface* const mini = image::getMinimap(100,100,map,0);
+						const surface mini(image::getMinimap(100,100,map,0));
 	
 						//generate a unique id to show the map as
 						char buf[50];
-						sprintf(buf,"addr %d",(int)mini);
+						sprintf(buf,"addr %d",(int)(SDL_Surface*)mini);
 	
 						image::register_image(buf,mini);
 	
@@ -123,7 +127,7 @@ RESULT enter(display& disp, config& game_data, const config& terrain_data, dialo
 						std::cerr << "illegal map: " << e.msg_ << "\n";
 					}
 				} else {
-					str << "(" << translate_string("shroud") << "),";
+					str << "(" << _("Shroud") << "),";
 				}
 	
 				std::string name = (**i.first)["name"];
@@ -135,9 +139,9 @@ RESULT enter(display& disp, config& game_data, const config& terrain_data, dialo
 				const std::string& turn = (**i.first)["turn"];
 				const std::string& slots = (**i.first)["slots"];
 				if(turn != "") {
-					str << "," << translate_string("turn") << " " << turn;
+					str << "," << _("Turn") << " " << turn;
 				} else if(slots != "") {
-					str << "," << slots << " " << string_table[slots == "1" ? "vacant_slot" : "vacant_slots"];
+					str << "," << slots << " " << gettext(slots == "1" ? N_("Vacant Slot") : N_("Vacant Slots"));
 				}
 	
 				options.push_back(str.str());
@@ -152,10 +156,10 @@ RESULT enter(display& disp, config& game_data, const config& terrain_data, dialo
 		}
 
 		gui::menu games_menu(disp,options);
-		gui::button observe_game(disp,string_table["observe_game"]);
-		gui::button join_game(disp,string_table["join_game"]);
-		gui::button new_game(disp,string_table["create_new_game"]);
-		gui::button quit_game(disp,string_table["quit_button"]);
+		gui::button observe_game(disp,_("Observe Game"));
+		gui::button join_game(disp,_("Join Game"));
+		gui::button new_game(disp,_("Create Game"));
+		gui::button quit_game(disp,_("Quit"));
 
 		if(dlg != NULL) {
 			observe_game.hide();
@@ -315,7 +319,7 @@ RESULT enter(display& disp, config& game_data, const config& terrain_data, dialo
 				chat_invalidated = true;
 			}
 
-			if(dlg == NULL && (last_escape == false && key[SDLK_ESCAPE] || quit_game.process(mousex,mousey,left_button))){
+			if(last_escape == false && key[SDLK_ESCAPE] || dlg == NULL && quit_game.process(mousex,mousey,left_button)){
 				return QUIT;
 			}
 
