@@ -6,10 +6,14 @@
 
 #include <algorithm>
 #include <cassert>
+#include <cerrno>
 #include <queue>
+#include <iostream>
 #include <set>
 #include <vector>
 
+#include <signal.h>
+
 namespace {
 
 //We store the details of a connection in a map that must be looked up by its handle.
@@ -17,17 +21,20 @@ namespace {
 //the same, so it's all seamless to the user
 struct connection_details {
 	connection_details(TCPsocket sock, const std::string& host, int port)
-		: sock(sock), disconnected_at(0), host(host), port(port), remote_handle(0)
+		: sock(sock), host(host), port(port), remote_handle(0),
+	      connected_at(SDL_GetTicks()), sent(0), received(0)
 	{}
 
 	TCPsocket sock;
-	int disconnected_at;
 	std::string host;
 	int port;
 
 	//the remote handle is the handle assigned to this connection by the remote host.
 	//is 0 before a handle has been assigned.
 	int remote_handle;
+
+	int connected_at;
+	int sent, received;
 };
 
 typedef std::map<network::connection,connection_details> connection_map;
@@ -107,6 +114,16 @@ std::set<network::connection> bad_sockets;
 
 namespace network {
 
+connection_stats::connection_stats(int sent, int received, int connected_at)
+       : bytes_sent(sent), bytes_received(received), time_connected(SDL_GetTicks() - connected_at)
+{}
+
+connection_stats get_connection_stats(connection connection_num)
+{
+	connection_details& details = get_connection_details(connection_num);
+	return connection_stats(details.sent,details.received,details.connected_at);
+}
+
 const connection null_connection = 0;
 
 error::error(const std::string& msg, connection sock) : message(msg), socket(sock)
@@ -129,7 +146,14 @@ manager::manager() : free_(true)
 		return;
 	}
 
+	//on Unix-based systems, set sigpipe to be ignored
+#if !(defined(_WIN32) || defined(__APPLE__))
+	std::cerr << "ignoring SIGPIPE\n";
+	signal(SIGPIPE,SIG_IGN);
+#endif
+
 	if(SDLNet_Init() == -1) {
+		std::cerr << "could not initialize SDLNet; throwing error...\n";
 		throw error(SDL_GetError());
 	}
 
@@ -146,10 +170,19 @@ manager::~manager()
 	}
 }
 
-server_manager::server_manager(int port, bool create_server) : free_(false)
+server_manager::server_manager(int port, CREATE_SERVER create_server) : free_(false)
 {
-	if(create_server && !server_socket) {
-		server_socket = get_socket(connect("",port));
+	if(create_server != NO_SERVER && !server_socket) {
+		try {
+			server_socket = get_socket(connect("",port));
+		} catch(network::error& e) {
+			if(create_server == MUST_CREATE_SERVER) {
+				throw e;
+			} else {
+				return;
+			}
+		}
+
 		std::cerr << "server socket initialized: " << server_socket << "\n";
 		free_ = true;
 	}
@@ -163,6 +196,11 @@ server_manager::~server_manager()
 	}
 }
 
+bool server_manager::is_running() const
+{
+	return static_cast<bool>(server_socket);
+}
+
 size_t nconnections()
 {
 	return sockets.size();
@@ -183,7 +221,10 @@ connection connect(const std::string& host, int port)
 
 	TCPsocket sock = SDLNet_TCP_Open(&ip);
 	if(!sock) {
-		throw error("Could not connect to host");
+		throw error(hostname == NULL ? "Could not bind to port" :
+		                               "Could not connect to host");
+	} else {
+		//TODO: add code in here which sets the socket to non-blocking
 	}
 
 	//if this is a server socket
@@ -374,7 +415,8 @@ connection receive_data(config& cfg, connection connection_num, int timeout)
 	}
 
 	for(sockets_list::const_iterator i = sockets.begin(); i != sockets.end(); ++i) {
-		const TCPsocket sock = get_socket(*i);
+		connection_details& details = get_connection_details(*i);
+		const TCPsocket sock = details.sock;
 		if(SDLNet_SocketReady(sock)) {
 
 			//see if this socket is still waiting for it to be assigned its remote handle
@@ -402,11 +444,13 @@ connection receive_data(config& cfg, connection connection_num, int timeout)
 					throw error("Remote host disconnected",*i);
 				}
 
+				details.received += len;
+
 				len = SDLNet_Read32(num_buf);
 
 				std::cerr << "received packet length: " << len << "\n";
 
-				if(len > 10000000) {
+				if((len < 1) || (len > 10000000)) {
 					std::cerr << "bad length in network packet. Throwing error\n";
 					throw error("network error: bad length data",*i);
 				}
@@ -432,6 +476,8 @@ connection receive_data(config& cfg, connection connection_num, int timeout)
 				throw error("remote host disconnected",*i);
 			}
 
+			details.received += nbytes;
+
 			buf.upto += nbytes;
 			std::cerr << "received " << nbytes << "=" << buf.upto << "/" << buf.buf.size() << "\n";
 
@@ -454,8 +500,8 @@ connection receive_data(config& cfg, connection connection_num, int timeout)
 
 				cfg.read_compressed(buffer,schema->second.incoming);
 
-				std::cerr << "--- RECEIVED DATA from " << ((int)*i) << ": '"
-				          << cfg.write() << "'\n--- END RECEIVED DATA\n";
+//				std::cerr << "--- RECEIVED DATA from " << ((int)*i) << ": '"
+//				          << cfg.write() << "'\n--- END RECEIVED DATA\n";
 
 				
 				return *i;
@@ -478,7 +524,7 @@ void set_default_send_size(size_t max_size)
 	default_max_send_size = max_size;
 }
 
-void send_data(const config& cfg, connection connection_num, size_t max_size)
+void send_data(const config& cfg, connection connection_num, size_t max_size, SEND_TYPE mode)
 {
 	if(cfg.empty()) {
 		return;
@@ -515,39 +561,30 @@ void send_data(const config& cfg, connection connection_num, size_t max_size)
 	std::string value(4,'x');
 	value += cfg.write_compressed(schema->second.outgoing);
 
-	std::cerr << "--- SEND DATA to " << ((int)connection_num) << ": '"
-	          << cfg.write() << "'\n--- END SEND DATA\n";
+//	std::cerr << "--- SEND DATA to " << ((int)connection_num) << ": '"
+//	          << cfg.write() << "'\n--- END SEND DATA\n";
 
 	char buf[4];
 	SDLNet_Write32(value.size()+1-4,buf);
 	std::copy(buf,buf+4,value.begin());
 
-	//if the data is less than our maximum chunk, and there is no data queued to send
-	//to this host, then send all data now
-	if((max_size == 0 || value.size()+1 <= max_size) && send_queue.count(connection_num) == 0) {
-		std::cerr << "sending " << (value.size()+1) << " bytes\n";
-		const int res = SDLNet_TCP_Send(get_socket(connection_num),
-		                                const_cast<char*>(value.c_str()),
-		                                value.size()+1);
+	//place the data in the send queue
+	const send_queue_map::iterator itor = send_queue.insert(std::pair<network::connection,partial_buffer>(connection_num,partial_buffer()));
 
-		if(res != int(value.size()+1)) {
-			std::cerr << "sending data failed: " << res << "/" << value.size() << "\n";
-			throw error("Could not send data over socket",connection_num);
-		}
-	} else {
-		std::cerr << "cannot send all " << (value.size()+1) << " bytes at once. Placing in send queue.\n";
-		//place the data in the send queue
-		const send_queue_map::iterator itor =
-		     send_queue.insert(std::pair<network::connection,partial_buffer>(connection_num,partial_buffer()));
-
-		itor->second.buf.resize(value.size()+1);
-		std::copy(value.begin(),value.end(),itor->second.buf.begin());
-		itor->second.buf.back() = 0;
+	itor->second.buf.resize(value.size()+1);
+	std::copy(value.begin(),value.end(),itor->second.buf.begin());
+	itor->second.buf.back() = 0;
 
+	if(mode == SEND_DATA) {
 		process_send_queue(connection_num,max_size);
 	}
 }
 
+void queue_data(const config& cfg, connection connection_num)
+{
+	send_data(cfg,connection_num,0,QUEUE_ONLY);
+}
+
 void process_send_queue(connection connection_num, size_t max_size)
 {
 	if(connection_num == 0) {
@@ -566,7 +603,8 @@ void process_send_queue(connection connection_num, size_t max_size)
 		max_size = 8;
 	}
 
-	const TCPsocket sock = get_socket(connection_num);
+	connection_details& details = get_connection_details(connection_num);
+	const TCPsocket sock = details.sock;
 
 	std::pair<send_queue_map::iterator,send_queue_map::iterator> itor = send_queue.equal_range(connection_num);
 	if(itor.first != itor.second) {
@@ -581,12 +619,15 @@ void process_send_queue(connection connection_num, size_t max_size)
 		std::cerr << "sending " << bytes_to_send << " from send queue\n";
 
 		const int res = SDLNet_TCP_Send(sock,&buf[upto],bytes_to_send);
-		if(res != int(bytes_to_send)) {
+		if(res < 0 || res != int(bytes_to_send) && errno != EAGAIN) {
 			std::cerr << "sending data failed: " << res << "/" << bytes_to_send << "\n";
 			throw error("Sending queued data failed",connection_num);
 		}
 
-		upto += bytes_to_send;
+		std::cerr << "sent.\n";
+
+		upto += res;
+		details.sent += res;
 
 		//if we've now sent the entire item, erase it from the send queue
 		if(upto == buf.size()) {
@@ -641,4 +682,6 @@ std::pair<int,int> current_transfer_stats()
 		return std::pair<int,int>(current_connection->second.upto,current_connection->second.buf.size());
 }
 
+bool sends_queued() { return send_queue.empty() == false; }
+
 } //end namespace network
