@@ -24,11 +24,19 @@ private:
 //a server manager causes listening on a given port to occur
 //for the duration of its lifetime.
 struct server_manager {
-	//if create_server is false, then the object has no effect.
+
+	//parameter to pass to the constructor.
+	
+	enum CREATE_SERVER { MUST_CREATE_SERVER, //will throw exception on failure
+	                     TRY_CREATE_SERVER, //will swallow failure
+	                     NO_SERVER }; //won't try to create a server at all
+
 	//throws error.
-	server_manager(int port, bool create_server=true);
+	server_manager(int port, CREATE_SERVER create_server=MUST_CREATE_SERVER);
 	~server_manager();
 
+	bool is_running() const;
+
 private:
 	bool free_;
 };
@@ -74,6 +82,8 @@ connection receive_data(config& cfg, connection connection_num=0, int timeout=0)
 //sets the default maximum number of bytes to send to a client at a time
 void set_default_send_size(size_t send_size);
 
+enum SEND_TYPE { SEND_DATA, QUEUE_ONLY };
+
 //function to send data down a given connection, or broadcast
 //to all peers if connection_num is 0. throws error.
 //it will send a maximum of 'max_size' bytes. If cfg when serialized is more than
@@ -82,9 +92,15 @@ void set_default_send_size(size_t send_size);
 //appended to the send queue, and max_size bytes in the send queue will be sent.
 //if 'max_size' is 0, then the entire contents of the send_queue as well as 'cfg'
 //will be sent.
+//if 'mode' is set to 'QUEUE_ONLY', then no data will be sent at all: only placed
+//in the send queue. This will guarantee that this function never throws
+//network::error exceptions
 //
 //data in the send queue can be sent using process_send_queue
-void send_data(const config& cfg, connection connection_num=0, size_t max_size=0);
+void send_data(const config& cfg, connection connection_num=0, size_t max_size=0, SEND_TYPE mode=SEND_DATA);
+
+//function to queue data to be sent. queue_data(cfg,sock) is equivalent to send_data(cfg,sock,0,QUEUE_ONLY)
+void queue_data(const config& cfg, connection connection_num=0);
 
 //function to send any data that is in a connection's send_queue, up to a maximum
 //of 'max_size' bytes -- or the entire send queue if 'max_size' bytes is 0
@@ -99,6 +115,16 @@ std::string ip_address(connection connection_num);
 //function to see the number of bytes being processed on the current socket
 std::pair<int,int> current_transfer_stats();
 
+struct connection_stats
+{
+	connection_stats(int sent, int received, int connected_at);
+
+	int bytes_sent, bytes_received;
+	int time_connected;
+};
+
+connection_stats get_connection_stats(connection connection_num);
+
 struct error
 {
 	error(const std::string& msg="", connection sock=0);
@@ -108,6 +134,8 @@ struct error
 	void disconnect();
 };
 
+bool sends_queued();
+
 }
 
 #endif
