@@ -1,4 +1,4 @@
-/* $Id: pathfind.hpp,v 1.29 2004/06/16 14:01:36 Sirp Exp $ */
+/* $Id: pathfind.hpp,v 1.33 2004/09/19 11:13:56 silene Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -15,9 +15,10 @@
 
 #include "array.hpp"
 #include "gamestatus.hpp"
+#include "log.hpp"
 #include "map.hpp"
+#include "pathutils.hpp"
 #include "unit.hpp"
-#include "unit_types.hpp"
 
 #include <algorithm>
 #include <cassert>
@@ -27,11 +28,9 @@
 #include <list>
 #include <vector>
 
-//this module contains various pathfinding functions and utilities.
+#define LOG_PF lg::info(lg::engine)
 
-//function which, given a location, will place all adjacent locations in
-//res. res must point to an array of 6 location objects.
-void get_adjacent_tiles(const gamemap::location& a, gamemap::location* res);
+//this module contains various pathfinding functions and utilities.
 
 //a convenient type for storing a list of tiles adjacent to a certain tile
 typedef util::array<gamemap::location,6> adjacent_tiles_array;
@@ -39,12 +38,9 @@ typedef util::array<gamemap::location,6> adjacent_tiles_array;
 //function which, given a location, will find all tiles within 'radius' of that tile
 void get_tiles_radius(const gamemap::location& a, size_t radius, std::set<gamemap::location>& res);
 
-//function which tells if two locations are adjacent.
-bool tiles_adjacent(const gamemap::location& a, const gamemap::location& b);
-
-//function which gives the number of hexes between two tiles (i.e. the minimum
-//number of hexes that have to be traversed to get from one hex to the other)
-size_t distance_between(const gamemap::location& a, const gamemap::location& b);
+//function which, given a set of locations, will find all tiles within 'radius' of those tiles
+void get_tiles_radius(const gamemap& map, const std::vector<gamemap::location>& locs, size_t radius,
+	std::set<gamemap::location>& res);
 
 enum VACANT_TILE_TYPE { VACANT_CASTLE, VACANT_ANY };
 
@@ -167,7 +163,7 @@ paths::route a_star_search(const gamemap::location& src,
                            const gamemap::location& dst, double stop_at, T obj,
                            const std::set<gamemap::location>* teleports=NULL)
 {
-	std::cerr  << "a* search: " << src.x << ", " << src.y << " -> " << dst.x << ", " << dst.y << "\n";
+	LOG_PF << "a* search: " << src.x << ", " << src.y << " -> " << dst.x << ", " << dst.y << "\n";
 	using namespace detail;
 	typedef gamemap::location location;
 
@@ -206,7 +202,7 @@ paths::route a_star_search(const gamemap::location& src,
 
 			//if we have found a solution
 			if(locs[j] == dst) {
-				std::cerr << "found solution; calculating it...\n";
+				LOG_PF << "found solution; calculating it...\n";
 				paths::route rt;
 
 				for(location loc = lowest->second.loc; loc.valid(); ) {
@@ -226,7 +222,7 @@ paths::route a_star_search(const gamemap::location& src,
 
 				assert(rt.steps.front() == src);
 
-				std::cerr  << "exiting a* search (solved)\n";
+				LOG_PF << "exiting a* search (solved)\n";
 
 				return rt;
 			}
@@ -275,10 +271,12 @@ paths::route a_star_search(const gamemap::location& src,
 		}
 	}
 
-	std::cerr  << "aborted a* search\n";
+	LOG_PF << "aborted a* search\n";
 	paths::route val;
 	val.move_left = 100000;
 	return val;
 }
 
+#undef LOG_PF
+
 #endif
