@@ -1,4 +1,4 @@
-/* $Id: playlevel.cpp,v 1.141 2004/06/20 20:23:01 erl Exp $ */
+/* $Id: playlevel.cpp,v 1.155 2004/10/03 18:00:15 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -88,8 +88,7 @@ namespace {
 		std::set<int> placed;
 		std::set<gamemap::location> positions_taken;
 
-		for(std::vector<placing_info>::const_iterator i = placings.begin();
-		    i != placings.end() && placed.size() != sides.size(); ++i) {
+		for(std::vector<placing_info>::const_iterator i = placings.begin(); i != placings.end() && placed.size() != sides.size(); ++i) {
 			if(placed.count(i->side) == 0 && positions_taken.count(i->pos) == 0) {
 				placed.insert(i->side);
 				positions_taken.insert(i->pos);
@@ -112,9 +111,9 @@ namespace {
 }
 
 LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
-                        config* level, CVideo& video,
-                        game_state& state_of_game,
-						const std::vector<config*>& story)
+		config* level, CVideo& video,
+		game_state& state_of_game,
+		const std::vector<config*>& story)
 {
 	const int ticks = SDL_GetTicks();
 	std::cerr << "in play_level()...\n";
@@ -151,11 +150,13 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 		state_of_game.starting_pos = new_level;
 	}
 
+	const config& lvl = *level;
+
 	std::cerr << "generated map " << (SDL_GetTicks() - ticks) << "\n";
 
-	const statistics::scenario_context statistics_context(translate_string_default((*level)["id"],(*level)["name"]));
+	const statistics::scenario_context statistics_context(lvl["name"]);
 
-	const int num_turns = atoi(level->values["turns"].c_str());
+	const int num_turns = atoi(lvl["turns"].c_str());
 	gamestatus status(*level,num_turns);
 
 	gamemap map(game_config,map_data);
@@ -167,7 +168,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 	const verification_manager verify_manager(units);
 
-	const int xp_modifier = atoi((*level)["experience_modifier"].c_str());
+	const int xp_modifier = atoi(lvl["experience_modifier"].c_str());
 	const unit_type::experience_accelerator xp_mod(xp_modifier > 0 ? xp_modifier : 100);
 
 	std::vector<team> teams;
@@ -178,7 +179,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 	const config::child_list& unit_cfg = level->get_children("side");
 
-	if((*level)["modify_placing"] == "true") {
+	if(lvl["modify_placing"] == "true") {
 		std::cerr << "modifying placing...\n";
 		place_sides_in_preferred_locations(map,unit_cfg);
 	}
@@ -187,6 +188,24 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 	std::cerr << (SDL_GetTicks() - ticks) << "\n";
 
 	for(config::child_list::const_iterator ui = unit_cfg.begin(); ui != unit_cfg.end(); ++ui) {
+		std::string save_id = (**ui)["save_id"];
+
+		if(save_id.empty()) {
+			save_id=(**ui)["description"];
+		}
+
+		player_info *player = NULL;
+
+		if((**ui)["controller"] == "human" ||
+		   (**ui)["controller"] == "network" ||
+		   (**ui)["persistent"] == "1") {
+			player = state_of_game.get_player(save_id);
+
+			if(!player && !save_id.empty()) {
+				player=&state_of_game.players[save_id];
+			}
+		}
+
 		std::cerr << "initializing team...\n";
 
 		if(first_human_team == -1 && (**ui)["controller"] == "human") {
@@ -200,8 +219,8 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 		std::cerr << "found gold: '" << gold << "'\n";
 
 		int ngold = lexical_cast_default<int>(gold);
-		if(ui == unit_cfg.begin() && state_of_game.gold >= ngold) {
-			ngold = state_of_game.gold;
+		if(player && player->gold >= ngold) {
+			ngold = player->gold;
 		}
 
 		std::cerr << "set gold to '" << ngold << "'\n";
@@ -214,12 +233,11 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 			//search the recall list for leader units, and if there is
 			//one, use it in place of the config-described unit
-			if(ui == unit_cfg.begin()) {
-				for(std::vector<unit>::iterator it = state_of_game.available_units.begin();
-					it != state_of_game.available_units.end(); ++it) {
+			if(player) {
+				for(std::vector<unit>::iterator it = player->available_units.begin(); it != player->available_units.end(); ++it) {
 					if(it->can_recruit()) {
 						new_unit = *it;
-						state_of_game.available_units.erase(it);
+						player->available_units.erase(it);
 						break;
 					}
 				}
@@ -251,38 +269,86 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 		//if the game state specifies units that can be recruited for the player
 		//then add them
-		if(teams.size() == 1 && state_of_game.can_recruit.empty() == false) {
-			std::copy(state_of_game.can_recruit.begin(),state_of_game.can_recruit.end(),
-				std::inserter(teams.back().recruits(),teams.back().recruits().end()));
+		if(player && player->can_recruit.empty() == false) {
+			std::copy(player->can_recruit.begin(),player->can_recruit.end(),
+					std::inserter(teams.back().recruits(),teams.back().recruits().end()));
 		}
 		
-		if(teams.size() == 1) {
-			state_of_game.can_recruit = teams.back().recruits();
+		if(player) {
+			player->can_recruit = teams.back().recruits();
 		}
-
+		
 		//if there are additional starting units on this side
 		const config::child_list& starting_units = (*ui)->get_children("unit");
-		for(config::child_list::const_iterator su = starting_units.begin();
-		    su != starting_units.end(); ++su) {
+		for(config::child_list::const_iterator su = starting_units.begin(); su != starting_units.end(); ++su) {
 			unit new_unit(gameinfo,**su);
+
+			new_unit.set_side(lexical_cast_default<int>((**ui)["side"],1));
+
 			const std::string& x = (**su)["x"];
 			const std::string& y = (**su)["y"];
 
 			const gamemap::location loc(**su);
 			if(x.empty() || y.empty() || !map.on_board(loc)) {
-				state_of_game.available_units.push_back(new_unit);
+				if(player) {
+					player->available_units.push_back(new_unit);
+				}
 			} else {
 				units.insert(std::pair<gamemap::location,unit>(loc,new_unit));
 				std::cerr << "inserting unit for side " << new_unit.side() << "\n";
 			}
 		}
+
 	}
 
+	// Add all recruitable units as encountered so that information
+	// about them are displayed to the user in the help system.
+	for (std::vector<team>::const_iterator help_team_it = teams.begin();
+		 help_team_it != teams.end(); help_team_it++) {
+		std::cout << "Adding help units for team '" << help_team_it->name()
+				  << "'" << std::endl;
+		const std::set<std::string> &recruitable = help_team_it->recruits();
+		std::set<std::string> &enc_units = preferences::encountered_units();
+		std::cout << "Adding recruitable units: " << std::endl;
+		for (std::set<std::string>::const_iterator it = recruitable.begin();
+			 it != recruitable.end(); it++) {
+			std::cout << *it << std::endl;
+		}
+		std::cout << "Added all recruitable units" << std::endl;
+		std::copy(recruitable.begin(), recruitable.end(), 
+				  std::inserter(enc_units, enc_units.begin()));
+	}
+		
+	// Add all units that exist at the start to the encountered units so
+	// that information about them are displayed to the user in the help
+	// system.
+	for (unit_map::const_iterator help_unit_it = units.begin();
+		 help_unit_it != units.end(); help_unit_it++) {
+		const std::string name = help_unit_it->second.type().name();
+		preferences::encountered_units().insert(name);
+	}
+
+	// Add all units that are recallable as encountred units.
+	for(std::map<std::string, player_info>::iterator pi = state_of_game.players.begin(); pi!=state_of_game.players.end(); ++pi) {
+		for(std::vector<unit>::iterator help_recall_it = pi->second.available_units.begin(); help_recall_it != pi->second.available_units.end(); help_recall_it++) {
+			preferences::encountered_units().insert(help_recall_it->type().name());
+		}
+	}
+
+	// Add all terrains on the map as encountered terrains.
+	for (int map_x = 0; map_x < map.x(); map_x++) {
+		for (int map_y = 0; map_y < map.y(); map_y++) {
+			const gamemap::TERRAIN t = map.get_terrain(gamemap::location(map_x, map_y));
+			std::string s;
+			s += t;
+			preferences::encountered_terrains().insert(s);
+		}
+	}
 	std::cerr << "initialized teams... " << (SDL_GetTicks() - ticks) << "\n";
 
 	const config* theme_cfg = NULL;
-	if((*level)["theme"] != "") {
-		theme_cfg = game_config.find_child("theme","name",(*level)["theme"]);
+	if(lvl["theme"] != "") {
+		theme_cfg = game_config.find_child("theme","name",lvl["theme"]);
 	}
 
 	if(theme_cfg == NULL) {
@@ -291,17 +357,10 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 	std::cerr << "initializing display... " << (SDL_GetTicks() - ticks) << "\n";
 	const config dummy_cfg;
-	display gui(units,video,map,status,teams,theme_cfg != NULL ? *theme_cfg : dummy_cfg, game_config);
+	display gui(units,video,map,status,teams,theme_cfg != NULL ? *theme_cfg : dummy_cfg, game_config, *level);
 
 	std::cerr << "done initializing display... " << (SDL_GetTicks() - ticks) << "\n";
 
-	//object that will make sure that labels are removed at the end of the scenario
-	const font::floating_label_context labels_manager;
-
-	const halo::manager halo_manager(gui);
-
-	gui.labels().read(*level);
-
 	std::cerr << "a... " << (SDL_GetTicks() - ticks) << "\n";
 
 	if(first_human_team != -1) {
@@ -313,18 +372,24 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 	std::cerr << "b... " << (SDL_GetTicks() - ticks) << "\n";
 
-	if(recorder.skipping() == false) {
-		for(std::vector<config*>::const_iterator story_i = story.begin();
-		    story_i != story.end(); ++story_i) {
-			show_intro(gui,**story_i, state_of_game);
-		}
+	//this *needs* to be created before the show_intro and show_map_scene
+	//as that functions use the manager state_of_game 
+	game_events::manager events_manager(*level,gui,map,units,teams,
+	                                    state_of_game,status,gameinfo);
 
-		show_map_scene(gui,*level);
+	if(recorder.skipping() == false) {
+		for(std::vector<config*>::const_iterator story_i = story.begin(); story_i != story.end(); ++story_i) {
+			show_intro(gui,**story_i, *level);
+		}
 	}
 
+	//object that will make sure that labels are removed at the end of the scenario
+	const font::floating_label_context labels_manager;
+	const halo::manager halo_manager(gui);
+	gui.labels().read(*level);
 	std::cerr << "c... " << (SDL_GetTicks() - ticks) << "\n";
 
-	const std::string& music = level->values["music"];
+	const std::string& music = lvl["music"];
 	if(music != "") {
 		sound::play_music(music);
 	}
@@ -332,13 +397,11 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 	std::cerr << "d... " << (SDL_GetTicks() - ticks) << "\n";
 
 	victory_conditions::set_victory_when_enemies_defeated(
-						(*level)["victory_when_enemies_defeated"] != "no");
+						lvl["victory_when_enemies_defeated"] != "no");
 
 	std::cerr << "initializing events manager... " << (SDL_GetTicks() - ticks) << "\n";
 
-	game_events::manager events_manager(*level,gui,map,units,teams,
-	                                    state_of_game,status,gameinfo);
-	help::help_manager help_manager(&game_config, &gameinfo);
+	help::help_manager help_manager(&game_config, &gameinfo, &map);
 
 	//find a list of 'items' (i.e. overlays) on the level, and add them
 	const config::child_list& overlays = level->get_children("item");
@@ -357,7 +420,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 	try {
 		//if a team is specified whose turn it is, it means we're loading a game
 		//instead of starting a fresh one
-		const bool loading_game = (*level)["playing_team"].empty() == false;
+		const bool loading_game = lvl["playing_team"].empty() == false;
 		
 		gui.begin_game();
 		gui.adjust_colours(0,0,0);
@@ -379,7 +442,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 		bool replaying = (recorder.at_end() == false);
 	
-		int first_player = atoi((*level)["playing_team"].c_str());
+		int first_player = atoi(lvl["playing_team"].c_str());
 		if(first_player < 0 || first_player >= int(teams.size())) {
 			first_player = 0;
 		}
@@ -417,8 +480,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 
 			std::cerr << "turn: " << turn++ << "\n";
 
-			for(std::vector<team>::iterator team_it = teams.begin()+first_player;
-			    team_it != teams.end(); ++team_it) {
+			for(std::vector<team>::iterator team_it = teams.begin()+first_player; team_it != teams.end(); ++team_it) {
 				log_scope("player turn");
 				player_number = (team_it - teams.begin()) + 1;
 
@@ -484,7 +546,7 @@ LEVEL_RESULT play_level(game_data& gameinfo, const config& game_config,
 						                      player_number,status,state_of_game);
 					} catch(replay::error& e) {
 						std::cerr << "caught replay::error\n";
-						gui::show_dialog(gui,NULL,"",string_table["bad_save_message"],gui::OK_ONLY);
+						gui::show_dialog(gui,NULL,"",_("The file you have tried to load is corrupt"),gui::OK_ONLY);
 
 						replaying = false;
 					}
@@ -503,7 +565,7 @@ redo_turn:
 					std::cerr << "is human...\n";
 
 					if(first_time && team_it == teams.begin()) {
-						if(level->values["objectives"].empty() == false) {
+						if(lvl["objectives"].empty() == false) {
 							dialogs::show_objectives(gui,*level);
 						}
 					}
@@ -532,6 +594,8 @@ redo_turn:
 					util::scoped_ptr<ai_interface> ai_obj(create_ai(team_it->ai_algorithm(),ai_info));
 					ai_obj->play_turn();
 
+					gui.recalculate_minimap();
+					clear_shroud(gui,status,map,gameinfo,units,teams,player_number-1);
 					gui.invalidate_unit();
 					gui.invalidate_game_status();
 					gui.invalidate_all();
@@ -623,8 +687,8 @@ redo_turn:
 	} catch(end_level_exception& end_level) {
 
 		if((end_level.result == DEFEAT || end_level.result == VICTORY) && is_observer(teams)) {
-			gui::show_dialog(gui,NULL,string_table["observer_endgame_heading"],
-			                          string_table["observer_endgame"], gui::OK_ONLY);
+			gui::show_dialog(gui,NULL,_("Game Over"),
+			                          _("The game is over."), gui::OK_ONLY);
 			return end_level.result;
 		}
 
@@ -637,8 +701,8 @@ redo_turn:
 			}
 
 			gui::show_dialog(gui,NULL,
-			                 string_table["defeat_heading"],
-			                 string_table["defeat_message"],
+			                 _("Defeat"),
+			                 _("You have been defeated!"),
 			                 gui::OK_ONLY);
 			return DEFEAT;
 		} else if(end_level.result == VICTORY || end_level.result == LEVEL_CONTINUE || end_level.result == LEVEL_CONTINUE_NO_SAVE) {
@@ -649,55 +713,82 @@ redo_turn:
 
 			//add all the units that survived the scenario
 			for(std::map<gamemap::location,unit>::iterator un = units.begin(); un != units.end(); ++un) {
-				if(un->second.side() == 1) {
+				player_info *player=state_of_game.get_player(teams[un->second.side()-1].save_id());
+
+				if(player) {
 					un->second.new_turn();
 					un->second.new_level();
-					state_of_game.available_units.push_back(un->second);
+					player->available_units.push_back(un->second);
 				}
 			}
 
 			//'continue' is like a victory, except it doesn't announce victory,
 			//and the player returns 100% of gold.
 			if(end_level.result == LEVEL_CONTINUE || end_level.result == LEVEL_CONTINUE_NO_SAVE) {
-				state_of_game.gold = teams[0].gold();
+				for(std::vector<team>::iterator i=teams.begin(); i!=teams.end(); ++i) {
+					player_info *player=state_of_game.get_player(i->save_id());
+					if(player) {
+						player->gold = i->gold();
+					}
+				}
+
 				return end_level.result == LEVEL_CONTINUE_NO_SAVE ? LEVEL_CONTINUE_NO_SAVE : VICTORY;
 			}
 
-			const int remaining_gold = teams[0].gold();
-			const int finishing_bonus_per_turn = map.villages().size()*game_config::village_income + game_config::base_income;
-			const int turns_left = maximum<int>(0,status.number_of_turns() - status.turn());
-			const int finishing_bonus = end_level.gold_bonus ?
-			              (finishing_bonus_per_turn * turns_left) : 0;
-			state_of_game.gold = ((remaining_gold+finishing_bonus)*80)/100;
 
-			gui::show_dialog(gui,NULL,string_table["victory_heading"],
-			                 string_table["victory_message"],gui::OK_ONLY);
 			std::stringstream report;
-			report << string_table["remaining_gold"] << ": "
-			       << remaining_gold << "\n";
-			if(end_level.gold_bonus) {
-				report << string_table["early_finish_bonus"] << ": "
-				       << finishing_bonus_per_turn
-					   << " " << string_table["per_turn"] << "\n"
-				       << string_table["turns_finished_early"] << ": "
-				       << turns_left << "\n"
-				       << string_table["bonus"] << ": "
-					   << finishing_bonus << "\n"
-				       << string_table["gold"] << ": "
-					   << (remaining_gold+finishing_bonus);
+
+			for(std::vector<team>::iterator i=teams.begin(); i!=teams.end(); ++i) {
+				player_info *player=state_of_game.get_player(i->save_id());
+
+				const int remaining_gold = i->gold();
+				const int finishing_bonus_per_turn = map.villages().size()*game_config::village_income + game_config::base_income;
+				const int turns_left = maximum<int>(0,status.number_of_turns() - status.turn());
+				const int finishing_bonus = end_level.gold_bonus ?
+				             (finishing_bonus_per_turn * turns_left) : 0;
+
+				if(player) {
+					player->gold = ((remaining_gold+finishing_bonus)*80)/100;
+
+					if(state_of_game.players.size()>1) {
+						if(i!=teams.begin()) {
+							report << "\n";
+						}
+
+						report << i->save_id() << ": " << "\n";
+					}
+
+					report << _("Remaining gold") << ": "
+					       << remaining_gold << "\n";
+					if(end_level.gold_bonus) {
+						report << _("Early finish bonus") << ": "
+						       << finishing_bonus_per_turn
+						       << " " << _("per turn") << "\n"
+						       << _("Turns finished early") << ": "
+						       << turns_left << "\n"
+						       << _("Bonus") << ": "
+						       << finishing_bonus << "\n"
+						       << _("Gold") << ": "
+						       << (remaining_gold+finishing_bonus);
+					}
+
+					// xgettext:no-c-format
+					report << "\n" << _("80% of gold is retained for the next scenario") << "\n" << _("Retained Gold") << ": " << player->gold;
+				}
 			}
 
-			report << "\n" << string_table["fifty_percent"] << "\n"
-				   << string_table["retained_gold"] << ": "
-				   << state_of_game.gold;
+			gui::show_dialog(gui,NULL,_("Victory"),
+			                 _("You have emerged victorious!"),gui::OK_ONLY);
 
-			gui::show_dialog(gui,NULL,"",report.str(),gui::OK_ONLY);
+			if(state_of_game.players.size()>0) {
+				gui::show_dialog(gui,NULL,"",report.str(),gui::OK_ONLY);
+			}
 			return VICTORY;
 		}
 	} //end catch
 	catch(replay::error& e) {
 		std::cerr << "caught replay::error\n";
-		gui::show_dialog(gui,NULL,"",string_table["bad_save_message"],
+		gui::show_dialog(gui,NULL,"",_("The file you have tried to load is corrupt"),
 		                 gui::OK_ONLY);
 		return QUIT;
 	}
@@ -712,7 +803,7 @@ redo_turn:
 					        game_config,level,key,gui,
 							map,teams,player_number,units,turn_info::BROWSE_NETWORKED,textbox_info,replay_sender);
 
-		turn_data.save_game(string_table["save_game_error"],gui::YES_NO);
+		turn_data.save_game(_("A network disconnection has occured, and the game cannot continue. Do you want to save the game?"),gui::YES_NO);
 		if(disconnect) {
 			throw network::error();
 		} else {
