@@ -1,4 +1,4 @@
-/* $Id: playlevel.hpp,v 1.16 2004/06/11 04:07:37 Sirp Exp $ */
+/* $Id: playlevel.hpp,v 1.18 2004/09/11 21:22:57 gruikya Exp $ */
 /*
    Copyright (C) 2003 by David White <davidnwhite@optusnet.com.au>
    Part of the Battle for Wesnoth Project http://wesnoth.whitevine.net
@@ -14,7 +14,7 @@
 #define PLAY_LEVEL_HPP_INCLUDED
 
 #include "actions.hpp"
-#include "ai.hpp"
+#include "ai_interface.hpp"
 #include "config.hpp"
 #include "dialogs.hpp"
 #include "display.hpp"
@@ -45,8 +45,8 @@ struct end_level_exception {
 };
 
 LEVEL_RESULT play_level(game_data& gameinfo, const config& terrain_config,
-                        config* level, CVideo& video,
-                        game_state& state_of_game,
-						const std::vector<config*>& story);
+		config* level, CVideo& video,
+		game_state& state_of_game,
+		const std::vector<config*>& story);
 
 #endif
